<?php

/**
 * @file
 * Functions and classes for the answers entities.
 */

define('LIMESURVEY_SYNC_ANSWER_COMPLETED', 1);
define('LIMESURVEY_SYNC_ANSWER_COMPLETED_EDITABLE', 2);
define('LIMESURVEY_SYNC_ANSWER_EDITING', 3);
define('LIMESURVEY_SYNC_ANSWER_VIRGIN', 4);

/**
 * Class managing building answer entities.
 *
 * Building entity_info, entity_property_info, schema, entity default values and
 * updating entity table if needed. Take care of Drupal fields, LimeSurvey fixed
 * fields and Limesurver variables fields.
 */
class LimesurveySyncAnswer extends LimesurveySync {
  public $ls_sid;
  public $ls_entity_type = 'answer';
}

/**
 * The Controller for answer entities
 */
class LimesurveySyncAnswerController extends LimesurveySyncController {
  public $ls_entity_type = 'answer';

  /**
   * Provide the answer entity status.
   *
   * Status are 'completed', 'completed_editable', 'editing' and 'virgin'.
   *
   * @param $entity
   *   The entity object.
   *
   * @return
   *   A numeric defined as a constant corresponding to a defined LS stage.
   */
  protected function lss_status($entity) {
    if (is_array($entity)) {
      $entity = (object) $entity;
    }
    $unix_empty_date = '1980-01-01 00:00:00';
    module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
    $answer_completed = ($entity->ls_submitdate !== NULL && ($entity->ls_submitdate != 'N' || $entity->ls_submitdate == limesurvey_sync_api_fixdate_ls2dp($unix_empty_date)));
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
    $survey_properties = limesurvey_sync_survey_properties($this->ls_sid);
    if ($answer_completed) {
      return (!$survey_properties['alloweditaftercompletion'] || $survey_properties['expires'] > REQUEST_TIME) ? LIMESURVEY_SYNC_ANSWER_COMPLETED : LIMESURVEY_SYNC_ANSWER_COMPLETED_EDITABLE;
    }
    else {
      // Browse all variable fields to detect if they empty.
      $prefix = 'ls_' . $this->ls_sid . 'X';
      $virgin = TRUE;
      foreach ($entity as $field => $value) {
        if (strpos($field, $prefix) !== FALSE) {
          if ($value !== NULL) {
            $virgin = FALSE;
            break;
          }
        }
      }
    }
    return (!$virgin) ? LIMESURVEY_SYNC_ANSWER_EDITING : LIMESURVEY_SYNC_ANSWER_VIRGIN;
  }

  /**
   * Builds a structured array representing the entity's content.
   *
   * The content built for the entity will vary depending on the $view_mode
   * parameter.
   *
   * @param $entity
   *   An entity object.
   * @param $view_mode
   *   View mode, e.g. 'full', 'teaser'...
   * @param $langcode
   *   (optional) A language code to use for rendering. Defaults to the global
   *   content language of the current request.
   *
   * @return
   *   The renderable array.
   */
  public function buildContent($entity, $view_mode = 'full', $langcode = NULL, $content = array()) {

    module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
    $render_admin = limesurvey_sync_api_fields_render($this->ls_entity_type, $entity, $entity->ls_sid, 'known');
    $content['answer_admin_datas'] = $render_admin + array(
      '#theme' => 'limesurvey_sync_admin_data_inline',
    );

    $render = limesurvey_sync_api_fields_render($this->ls_entity_type, $entity, $entity->ls_sid);
    $content['answer_datas'] = $render + array(
      //'#theme' => 'limesurvey_sync_answer_data_list' // Not working.
      '#theme' => 'limesurvey_sync_answer_data_table',
    );

    // @TODO : build a view mode to display the answer as a form, using
    // limesurvey_sync_answer_form() function.

    $build = parent::buildContent($entity, $view_mode, $langcode, $content);

    return $build;
  }

}

/**
 * The UI Controller for answer entities.
 */
class LimesurveySyncAnswerUIController extends LimesurveySyncUIController {
  public $ls_entity_type = 'answer';
  protected $menu_weight = -2;
  protected $ls_sub_entity = TRUE;
  protected $disallow_add = TRUE;
  protected $disallow_edit = FALSE;

}

/**
 * The entity info class for answer entities.
 */
class LimesurveySyncInfosAnswer extends LimesurveySyncInfos {
  public $ls_entity_type = 'answer';
  //public $root_ui = 'limesurvey';
  public $root_admin_ui = 'admin/structure/limesurvey/surveys';
  protected $revision = TRUE;
  protected $primary_key = 'dp_aid';
  protected $date_fields = TRUE;
  protected $ls_sync_field = TRUE;
  protected $ls_key = 'ls_id';
  protected $ls_date_key = 'ls_datestamp';
  protected $sub_entity = TRUE;
  protected $label = 'Answer';
  protected $plural_label = 'Answers';
  protected $exportable = TRUE;

  protected function schema_dp_table($sid = NULL) {
    return array(
      'description' => 'Stores answers from the LimeSurvey surveys #' . $sid . '.',
    );
  }

  protected function create_dp_fields_default_values($sid = NULL) {

    return array(
      'lss_status' => 1,
      'lss_sync' => 1,
    );
  }

  protected function schema_dp_fields($sid = NULL, $revision = FALSE) {

    $schema = parent::schema_dp_fields($sid, $revision);
    $schema['lss_status'] = array(
      'type' => 'int',
      'unsigned' => TRUE,
      'not NULL' => TRUE,
      'default' => 1,
      'size' => 'tiny',
      'description' => 'statut ID (1 and 2 for completed, 3 for editing, 4 for virgin)',
    );
    return $schema;
  }

  /**
   * Return the corresponding Drupal database schema of a LimeSurvey field.
   *
   * This fonction handles LimeSurvey answer fields that can be met on
   * all LimeSurvey surveys. The LimeSurvey datetime fields will be converted to
   * timestamp on Drupal.
   *
   * @param $field
   *   A string. The LimeSurvey field name.
   *
   * @return
   *   An array of Drupal schema corresponding to the provided LimeSurvey field or
   *   FALSE if the provided field is not a generic LimeSurvey field.
   */
  protected function schema_ls_fields_known($field) {
    // Note : by default on drupal 'unsight' -> FALSE and 'not null' -> FALSE.
    $fields = array(
      'ls_id' => array(
        'description' => 'Primary Key of the answer table',
        'type' => 'int',
        'unsigned' => TRUE,
        'not null' => TRUE,
      ),
      // Defaults to NULL in order to avoid a brief period of potential
      // deadlocks on the index.
      'ls_token' => array(
        'type' => 'varchar',
        'length' => 36,
        'default' => NULL,
        'description' => 'Invitation token.',
      ),
      'ls_submitdate' => array(
        'type' => 'int',
        'unsigned' => TRUE,
        'default' => NULL,
        'description' => 'Date the answer was submited.',
      ),
      'ls_lastpage' => array(
        'type' => 'int',
        'unsigned' => TRUE,
        'default' => NULL,
        'description' => 'Last page of the answer when editing it.',
      ),
      'ls_startlanguage' => array(
        'type' => 'varchar',
        'length' => 20,
        'not null' => TRUE,
        'description' => 'The started LimeSurvey language.',
        // no default value.
      ),
      'ls_startdate' => array(
        'type' => 'int',
        'not null' => TRUE,
        // no default value.
        'description' => 'The started date.',
      ),
      'ls_datestamp' => array(
        'type' => 'int',
        'not null' => TRUE,
        // no default value.
        'description' => 'The last edition date.',
      ),
      'ls_ipaddr' => array(
        'type' => 'text',
        'size' => 'tiny', // 'normal' is the limesurvey setting.
        'default' => NULL,
        'description' => 'The IP adress of the user of fill this answer.',
      ),
      'ls_refurl' => array(
        'type' => 'text',
        'size' => 'tiny',
        'default' => NULL,
        'description' => 'The referrer URL of the user of fill this answer.',
      ),
    );

    return (isset($fields[$field])) ? $fields[$field] : FALSE;
  }

  /**
   * Return the corresponding Drupal database schema of a LimeSurvey field.
   *
   * This fonction handles LimeSurvey token or answer fields that are specific to
   * a particular LimeSurvey survey. For the token table it corresponds to the
   * attribute fields, and for the answer table it corresponds to the questions
   * fields. The LimeSurvey datetime fields will be converted to timestamp on
   * Drupal.
   *
   * @param $type
   *   A string, 'token' or 'answer'.
   * @param $field
   *   A string. The LimeSurvey field name.
   * @param $sid
   *   A numeric. The LimeSurvey survey ID.
   *
   * @return
   *   An array of Drupal schema corresponding to the provided LimeSurvey field or
   *   FALSE if the provided field is not a variable LimeSurvey field.
   */
  protected function schema_ls_fields_variables($field, $sid) {
    // Helps on http://manual.limesurvey.org/Question_Objects.
    // http://manual.limesurvey.org/Question_types#An_explanation_of_subquestions.
    // Retreive fieldmap.
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_import');
    $fieldmap = limesurvey_sync_answ_create_field_map($sid);
    $suffix_description = '';
    foreach (array('subquestion', 'subquestion1', 'subquestion2') as $subquestion) {
      if (!empty($fieldmap[$field][$subquestion])) {
        $suffix_description .= $fieldmap[$field][$subquestion];
      }
    }
    $field_base = array(
      'not null' => FALSE,
      'default' => NULL,
      'description' => $fieldmap[$field]['question'] . ((!empty($suffix_description)) ? ', ' . $suffix_description : ''),
    );
    if (isset($fieldmap[$field])) {
      switch ($fieldmap[$field]['type']) {
        case '5': // point choice	FiveList	5
          // on mysql : varchar(1).
          $field_type = array(
            'type' => 'varchar',
            'length' => 1,
          );
          break;
        case '!': //List (dropdown)	Select	!	List
          // on mysql : varchar(5).
          $field_type = array(
            'type' => 'varchar',
            'length' => 5,
          );
          break;
        case 'L': //List (radio)	List	L
          // on mysql : varchar(5).
          $field_type = array(
            'type' => 'varchar',
            'length' => 5,
          );
          break;
        case 'O': //List with comment	CommentList	O	List
          // on mysql : varchar(5).
          $field_type = array(
            'type' => 'varchar',
            'length' => 5,
          );
          break;
        case 'F': //Array	RadioArray	F	Array
          // on mysql : varchar(5).
          $field_type = array(
            'type' => 'varchar',
            'length' => 5,
          );
          break;
        case 'B': //Array (10 point choice)	TenRadioArray	B	RadioArray
          // on mysql : varchar(5).
          $field_type = array(
            'type' => 'varchar',
            'length' => 5,
          );
          break;
        case 'A': //Array (5 point choice)	FiveRadioArray	A	RadioArray
          // on mysql : varchar(5).
          $field_type = array(
            'type' => 'varchar',
            'length' => 5,
          );
          break;
        case 'E': //Array (Increase/Same/Decrease)	IDRadioArray	E	RadioArray
          // on mysql : varchar(5).
          $field_type = array(
            'type' => 'varchar',
            'length' => 5,
          );
          break;
        case ':': //Array (Numbers)	NumberArray	:	Array
          $field_type = array(
            'type' => 'text',
            'size' => 'normal',
          );
          break;
        case ';': //Array (Texts)	TextArray	;	Array
          $field_type = array(
            'type' => 'text',
            'size' => 'normal',
          );
          break;
        case 'C': //Array (Yes/No/Uncertain)	YNRadioArray	C	RadioArray
          // on mysql : varchar(5).
          $field_type = array(
            'type' => 'varchar',
            'length' => 5,
          );
          break;
        case 'H': //Array by column	ColumnRadioArray	H	RadioArray
          // on mysql : varchar(5).
          $field_type = array(
            'type' => 'varchar',
            'length' => 5,
          );
          break;
        case '1': //Array dual scale	DualRadioArray	1	RadioArray
          // on mysql : varchar(5).
          $field_type = array(
            'type' => 'varchar',
            'length' => 5,
          );
          break;
        case 'D': //Date/Time	Date	D
          // on mysql : datetime --> Need convertion to timestamp !
          // But I keep it as datetime because it uses the user browser timezone.
          $field_type = array(
            'type' => 'varchar',
            'length' => 19,
          );
          break;

        case '*': //Equation	Equation	*
          $field_type = array(
            'type' => 'text',
            'size' => 'normal',
          );
          break;
        case '|': //File upload	File	pipe character
          if (drupal_substr($fieldmap[$field]['fieldname'], -10) == '_filecount') {
            $field_type = array(
              'type' => 'int',
              'size' => 'normal',
            );
          }
          else {
            // For storing the file. Size medium is 16MB, siez big is 4GB.
            $field_type = array(
              'type' => 'text',
              'size' => 'big',
            );
          }
          break;
        case 'G': //Gender	Gender	G
          // on mysql : varchar(1).
          $field_type = array(
            'type' => 'varchar',
            'length' => 1,
          );
          break;
        case 'I': //Language switch	Language	I
          // on mysql : varchar(20).
          $field_type = array(
            'type' => 'varchar',
            'length' => 20,
          );
          break;
        case 'K': //Multiple numerical input	Multinumerical	K
          // on mysql : float;
          $field_type = array(
            'type' => 'float',
            'size' => 'normal',
          );
          break;
        case 'N': //Numerical input	Numerical	N
          // on mysql : decimal(30,10);
          $field_type = array(
            'type' => 'numeric',
            'size' => 'normal',
            'precision' => 30,
            'scale' => 10,
          );
          break;
        case 'R': //Ranking	Ranking	R
          $field_type = array(
            'type' => 'varchar',
            'length' => 5,
          );
          break;
        case 'X': //Text display	Display	X
          // on mysql : varchar(1).
          $field_type = array(
            'type' => 'varchar',
            'length' => 1,
          );
          break;
        case 'Y': //Yes/No	YN	Y
          // on mysql : varchar(1).
          $field_type = array(
            'type' => 'varchar',
            'length' => 1,
          );
          break;
        case 'U': //Huge free text	HugeText	U	Text
          $field_type = array(
            'type' => 'text',
            'size' => 'normal',
          );
          break;
        case 'T': //Long free text	LongText	T	Text
          $field_type = array(
            'type' => 'text',
            'size' => 'normal',
          );
          break;
        case 'Q': //Multiple short text	Multitext	Q
          $field_type = array(
            'type' => 'text',
            'size' => 'normal',
          );
          break;
        case 'S': //Short free text	ShortText	S	Text
          $field_type = array(
            'type' => 'text',
            'size' => 'normal',
          );
          break;
        case 'M': //Multiple choice	Check	M
          // on mysql : varchar(5).
          $field_type = array(
            'type' => 'varchar',
            'length' => 5,
          );
          break;
        case 'P': //Multiple choice with comments	CommentCheck	P	Check
          // on mysql : varchar(5).
          $field_type = array(
            'type' => 'varchar',
            'length' => 5,
          );
          break;
        default:
          return FALSE;

      }
      return array_merge($field_type, $field_base);
    }
  }


  protected function entity_property_info_dp_fields($sid = NULL) {

    $info = parent::entity_property_info_dp_fields($sid);
    $info['lss_status'] = array(
      'label' => t('DP response status'),
      'description' => t('statut ID for the answer (1 and 2 for completed, 3 for editing, 4 for virgin).'),
      'type' => 'integer',
      'validation callback' => 'entity_metadata_validate_integer_positive',
      'schema field' => 'lss_status',
      'options list' => 'limesurvey_sync_api_answer_lss_status_list',
    );
    return $info;
  }

  /**
   * Return the corresponding Drupal entity_property_info of a LimeSurvey field.
   *
   * This fonction is used for generating the drupal hook_entity_property_info().
   * The main benefit of this fonction is to display the questions values instead
   * of the fieldname into Drupal Views. Handle common LimeSurvey fields of the
   * token and answer tables.
   *
   * @param $field
   *   A string. The LimeSurvey field name.
   * @param $sid
   *   A numeric. The LimeSurvey survey ID.
   *
   * @return
   *   An array of Drupal entity_property_info corresponding to the provided
   *   LimeSurvey field or FALSE if the provided field is not a generic LimeSurvey
   *   field.
   */
  protected function entity_property_info_ls_fields_known($field, $sid = NULL) {
    // Note : by default on drupal 'unsight' -> FALSE and 'not null' -> FALSE.
    // all known fields, they may be only visible for administrator users.
    $permission = array('setter permission' => "administer limesurvey sync {$this->ls_entity_type}");
    $fields = array(
      'ls_id' => array(
        'label' => t('_ID, LimeSurvey answer ID'),
        'description' => t('LimeSurvey answer ID into the LimeSurvey database.'),
        'type' => 'integer',
        'validation callback' => 'entity_metadata_validate_integer_positive',
        'schema field' => 'ls_id',
        'required' => TRUE,
      ) + $permission,
      'ls_token' => array(
        'label' => t('LS admin :Token value'),
        'description' => t('Answer token value (survey#%sid)', array('%sid' => $sid)),
        'type' => 'token',
        'schema field' => 'ls_token',
      ) + $permission,
      'ls_submitdate' => array(
        'label' => t('LS admin : Submit date'),
        'description' => t('Answer submit date from (survey#%sid)', array('%sid' => $sid)),
        'type' => 'date',
        'schema field' => 'ls_submitdate',
      ) + $permission,
      'ls_lastpage' => array(
        'label' => t('LS admin : Last page'),
        'description' => t('Answer last page (survey#%sid)', array('%sid' => $sid)),
        'type' => 'integer',
        'validation callback' => 'entity_metadata_validate_integer_positive',
        'schema field' => 'ls_lastpage',
      ) + $permission,
      'ls_startlanguage' => array(
        'label' => t('LS admin : Start language'),
        'description' => t('Answer start language (survey#%sid)', array('%sid' => $sid)),
        'schema field' => 'ls_startlanguage',
        'type' => 'token', //'limesurvey_sync_answer_ls_startlanguage', // 'text',
        //'setter callback' => 'entity_property_verbatim_set',
        'options list' => 'limesurvey_sync_survey_ls_language_list',
      ) + $permission,
      'ls_startdate' => array(
        'label' => t('LS admin : Start date'),
        'description' => t('Answer start date from (survey#%sid)', array('%sid' => $sid)),
        'type' => 'date',
        'schema field' => 'ls_startdate',
      ) + $permission,
      'ls_datestamp' => array(
        'label' => t('LS admin : Date stamp'),
        'description' => t('The last answer edition date from (survey#%sid)', array('%sid' => $sid)),
        'type' => 'date',
        'schema field' => 'ls_datestamp',
      ) + $permission,
      'ls_ipaddr' => array(
        'label' => t('LS admin : IP address'),
        'description' => t('Answer IP address from (survey#%sid)', array('%sid' => $sid)),
        'type' => 'uri',
        'schema field' => 'ls_ipaddr',
      ) + $permission,
      'ls_refurl' => array(
        'label' => t('LS admin : Referrer URL'),
        'description' => t('Answer referrer URL (survey#%sid)', array('%sid' => $sid)),
        'type' => 'uri',
        'schema field' => 'ls_refurl',
      ) + $permission,
    );

    return (isset($fields[$field])) ? $fields[$field] : FALSE;
  }

  /**
   * Return the corresponding Drupal entity_property_info of a LimeSurvey field.
   *
   * This fonction is used for generating the drupal hook_entity_property_info().
   * The main benefit of this fonction is to display the questions values instead
   * of the fieldname into Drupal Views. Handle variable LimeSurvey fields of the
   * token and answer tables.
   *
   * @param $field
   *   A string. The LimeSurvey field name.
   * @param $sid
   *   A numeric. The LimeSurvey survey ID.
   *
   * @return
   *   An array of Drupal entity_property_info corresponding to the provided
   *   LimeSurvey field or FALSE if the provided field is not a variable
   *   LimeSurvey field.
   */
  protected function entity_property_info_ls_fields_variables($field, $sid) {
    // Helps on http://manual.limesurvey.org/Question_Objects.
    // http://manual.limesurvey.org/Question_types#An_explanation_of_subquestions.
    // Retreive fieldmap.
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_import');
    $fieldmap = limesurvey_sync_answ_create_field_map($sid);
    $suffix_description = '';
    foreach (array('subquestion', 'subquestion1', 'subquestion2') as $subquestion) {
      if (!empty($fieldmap[$field][$subquestion])) {
        $suffix_description .= $fieldmap[$field][$subquestion];
      }
    }
    // Sanitized !
    // No html in 'label'.
    $fieldname_no_prefix = $fieldmap[$field]['fieldname'];
    $description = strip_tags($fieldmap[$field]['question'] . ((!empty($suffix_description)) ? ', ' . $suffix_description : ''));
    $field_base = array(
      'schema field' => $field,
      'label' => t('LS data @field : !data', array('!data' => htmlspecialchars_decode($description, ENT_QUOTES), '@field' => $fieldname_no_prefix)),
      'description' => t('Answer %data (survey #%sid)', array('%data' => $description, '%sid' => $sid)),
    );
    if (isset($fieldmap[$field])) {
      switch ($fieldmap[$field]['type']) {
        case 'Y': //Yes/No	YN	Y
        case '5': // point choice	FiveList	5
        case 'G': //Gender	Gender	G
        case 'E': //Array (Increase/Same/Decrease)	IDRadioArray	E	RadioArray
        case 'M': //Multiple choice	Check	M
        case 'P': //Multiple choice with comments	CommentCheck	P	Check
          $field_type = array(
            'type' => 'text',
            'options list' => 'limesurvey_sync_api_answer_ls_question_' . drupal_strtolower($fieldmap[$field]['type']) . '_list',
          );
          break;
        case "L": //List (radio)	List	L
        case "!": //List (dropdown)	Select	!	List
        case "O": //List with comment	CommentList	O	List
        case "^":
        case "I": //Language switch	Language	I
        case "R": //Ranking	Ranking	R
          // Code answer without scale.
          $field_type = array(
            'type' => 'text',
            'options list' => 'limesurvey_sync_api_answer_ls_question_code_list',
          );
          break;

        case "F": //Array	RadioArray	F	Array
        case "H": //Array by column	ColumnRadioArray	H	RadioArray
        case "1": //Array dual scale	DualRadioArray	1	RadioArray
          // Code answer with scale.
          $field_type = array(
            'type' => 'text',
            'options list' => 'limesurvey_sync_api_answer_ls_question_code_list',
          );
          break;

        case 'B': //Array (10 point choice)	TenRadioArray	B	RadioArray
        case 'A': //Array (5 point choice)	FiveRadioArray	A	RadioArray
        case ':': //Array (Numbers)	NumberArray	:	Array
        case ';': //Array (Texts)	TextArray	;	Array
        case '*': //Equation	Equation	*
        case 'X': //Text display	Display	X
        case 'U': //Huge free text	HugeText	U	Text
        case 'T': //Long free text	LongText	T	Text
        case 'Q': //Multiple short text	Multitext	Q
        case 'S': //Short free text	ShortText	S	Text
          $field_type = array('type' => 'text');
          break;
        case 'D': //Date/Time	Date	D
          // on mysql : datetime --> Need convertion to timestamp !
          // No : The datetime is stored as a text.
          $field_type = array('type' => 'text');
          break;
        case '|': //File upload	File	pipe character
          if (drupal_substr($fieldmap[$field]['fieldname'], -10) == '_filecount') {
            $field_type = array(
              'type' => 'int',
              'validation callback' => 'entity_metadata_validate_integer_positive',
            );
          }
          else {
            $field_type = array(
              'type' => 'text',
            );
          }
          break;
          // on mysql : varchar(1).
        case 'K': //Multiple numerical input	Multinumerical	K
          // on mysql : float;
        case 'N': //Numerical input	Numerical	N
          // on mysql : decimal(30,10);
          $field_type = array('type' => 'decimal');
          break;
        default:
          return FALSE;

      }
      return array_merge($field_type, $field_base);


    }
  }

  /**
   * Return the corresponding Drupal default value of a LS known field.
   *
   * This fonction is used for creating a new answer or token Drupal entity.
   * Handle common LimeSurvey fields of the token and answer tables.
   *
   * @param $field
   *   A string. The LimeSurvey field name.
   * @param $sid
   *   A numeric. The LimeSurvey survey ID. Default is NULL.
   *
   * @return
   *   A string or numeric or NULL : the Drupal default value for the provided
   *   LimeSurvey field or FALSE if the provided field is not a generic (known)
   *   LimeSurvey field.
   *
   * @see LimesurveySyncInfos::entity_default_values()
   */
  protected function create_ls_fields_default_values_known($field, $sid = NULL) {
    switch ($field) {
      case 'ls_id':
      case 'ls_token':
      case 'ls_submitdate':
      case 'ls_lastpage':
      case 'ls_startlanguage':
      case 'ls_startdate':
      case 'ls_ipaddr':
      case 'ls_refurl':
      case 'ls_datestamp':
        return NULL;
    }
    return FALSE;
  }

  /**
   * Return the corresponding Drupal default value of a LS variable field.
   *
   * This fonction is used for creating a new answer or token Drupal entity.
   * Handle variable LimeSurvey fields of the token and answer tables.
   *
   * @param $field
   *   A string. The LimeSurvey field name.
   * @param $sid
   *   A numeric. The LimeSurvey survey ID. Default is NULL.
   *
   * @return
   *   A string or numeric or NULL : the Drupal default value for the provided
   *   LimeSurvey field or FALSE if the provided field is not a variable
   *   LimeSurvey field.
   *
   * @see LimesurveySyncInfos::entity_default_values()
   */
  protected function create_ls_fields_default_values_variables($field, $sid = NULL) {
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_import');
    $fieldmap = limesurvey_sync_answ_create_field_map($sid);
    if (isset($fieldmap[$field])) {
      // Null as default value for all variable LS fields.
      return NULL;
    }
    return FALSE;
  }
}

/**
 * Helper for the limesurvey_sync_ls_api_fields_convert_values() function.
 *
 * Manage converting datas from LimeSurvey to Drupal for common LimeSurvey fields.
 *
 * @param $field
 *   A string. The LimeSurvey field name.
 * @param $value
 *   A string or numeric or decimal or NULL. The LimeSurvey value to convert.
 *
 * @return
 *   A string or numeric or decimal or NULL : the Drupal converted value.
 */
function limesurvey_sync_answer_fields_convert_values_known($field, $value, $way) {
  switch ($field) {
    case 'ls_id':
    case 'ls_token':
    case 'ls_lastpage':
    case 'ls_startlanguage':
    case 'ls_ipaddr':
    case 'ls_refurl':
      // Sanitize.
      if ($way == 'ls2dp') {
        return (($value === NULL) ? NULL : check_plain($value));
      }
      else {
        // The entity_load function return null values as empty strings.
        return (($value === NULL || $value === '') ? NULL : check_plain($value));
      }
      break;
    case 'ls_submitdate':
    case 'ls_startdate':
    case 'ls_datestamp':
      if ($way == 'ls2dp') {
        // Need to convert those values from datetime to timestamp.
        if ($value === NULL) {
          return NULL;
        }
        module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
        return limesurvey_sync_api_fixdate_ls2dp(check_plain($value));
        break;
      }
      else {
        // Need to convert those values from datetime to timestamp.
        if ($value === NULL) {
          return NULL;
        }
        module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
        return limesurvey_sync_api_fixdate_dp2ls(check_plain($value));
        break;

      }
    default:
      return FALSE;
  }

}

/**
 * Helper for the limesurvey_sync_ls_api_fields_convert_values() function.
 *
 * Manage converting datas from LimeSurvey to Drupal for variable LimeSurvey
 * fields.
 *
 * @param $sid
 *   A numeric. The LimeSurvey survey ID.
 * @param $field
 *   A string. The LimeSurvey field name.
 * @param $value
 *   A string or numeric or decimal or NULL. The LimeSurvey value to convert.
 *
 * @return
 *   A string or numeric or decimal or NULL : the Drupal converted value.
 */
function limesurvey_sync_answer_fields_convert_values_variables($sid, $field, $value, $way) {
  // Helps on http://manual.limesurvey.org/Question_Objects.
  // http://manual.limesurvey.org/Question_types#An_explanation_of_subquestions.
  // Retreive fieldmap.
  module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_import');
  $fieldmap = limesurvey_sync_answ_create_field_map($sid);
  $suffix_description = '';
  foreach (array('ls_subquestion', 'ls_subquestion1', 'ls_subquestion2') as $subquestion) {
    if (!empty($fieldmap[$field][$subquestion])) {
      $suffix_description .= $fieldmap[$field][$subquestion];
    }
  }
  if (isset($fieldmap[$field])) {
    switch ($fieldmap[$field]['type']) {
      case '5': // point choice	FiveList	5
      case '!': //List (dropdown)	Select	!	List
      case 'L': //List (radio)	List	L
      case 'O': //List with comment	CommentList	O	List
      case 'F': //Array	RadioArray	F	Array
      case 'B': //Array (10 point choice)	TenRadioArray	B	RadioArray
      case 'A': //Array (5 point choice)	FiveRadioArray	A	RadioArray
      case 'E': //Array (Increase/Same/Decrease)	IDRadioArray	E	RadioArray
      case ':': //Array (Numbers)	NumberArray	:	Array
      case ';': //Array (Texts)	TextArray	;	Array
      case 'C': //Array (Yes/No/Uncertain)	YNRadioArray	C	RadioArray
      case 'H': //Array by column	ColumnRadioArray	H	RadioArray
      case '1': //Array dual scale	DualRadioArray	1	RadioArray
      case '*': //Equation	Equation	*
      case 'G': //Gender	Gender	G
      case 'I': //Language switch	Language	I
      case 'K': //Multiple numerical input	Multinumerical	K
      case 'N': //Numerical input	Numerical	N
      case 'R': //Ranking	Ranking	R
      case 'X': //Text display	Display	X
      case 'Y': //Yes/No	YN	Y
      case 'U': //Huge free text	HugeText	U	Text
      case 'T': //Long free text	LongText	T	Text
      case 'Q': //Multiple short text	Multitext	Q
      case 'S': //Short free text	ShortText	S	Text
      case 'M': //Multiple choice	Check	M
      case 'P': //Multiple choice with comments	CommentCheck	P	Check
        // Sanitize.
        return (($value === NULL) ? NULL : check_plain($value));
        break;
      case 'D': //Date/Time	Date	D
        // This datetime is stored as datetime (varchar) into drupal.
        return (($value === NULL) ? NULL : check_plain($value));
        //module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
        //return limesurvey_sync_api_fixdate_ls2dp(check_plain($value));
        break;
      case '|': //File upload	File	pipe character
        if (drupal_substr($fieldmap[$field]['fieldname'], -10) == '_filecount') {
        }
        else {
        }
        return (($value === NULL) ? NULL : check_plain($value));
        break;
      default:
        return FALSE;

    }


  }
  return FALSE;

}

/**
 * Builder for the answer form page.
 *
 * Not available yet. Will be usefull for 'dp2ls' synchronizations.
 *
 * @return
 *   An array, the form elements.
 */
function limesurvey_sync_answer_form($form, &$form_state, $entity, $op = 'edit', $entity_type) {
  $form['debug'] = array('#markup' => 'Coding in progress.');
  module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
  $render = limesurvey_sync_api_fields_render($entity->ls_entity_type, $entity, $entity->ls_sid, 'variable');

  // set ls_lang.
  $ls_lang = limesurvey_sync_api_set_ls_lang_from_current_dp_lang($entity->ls_sid);

  $render_array = $render['for theme'];
  $ls_values = $render['ls_values'];

  $form['ls_fields'] = array(
    '#type' => 'fieldset',
    '#title' => t('The response'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    //'#tree' => TRUE,
    //'#element_validate' => array('limesurvey_sync_answer_form_validate'),
  );
  //Delete the validation function entity_ui_controller_form_validate (WSOD).
  $form['#validate'] = array('limesurvey_sync_answer_form_validate');

  // Note : element_children() breaks the association key to gid, qid, etc...
  module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.theme');
  foreach (limesurvey_sync_api_element_children($render_array) as $gid => $group) {
    $group_name = 'groupname_' . $gid;
    // @TODO : do not use fieldset but a div wrapper instead : it seems to add
    // #tree to TRUE.
    $form['ls_fields'][$group_name] = array(
      '#type' => 'fieldset',
      '#title' => $render_array[$gid]['#ls_group_name'],
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      //'#tree' => TRUE,
    );

    foreach (limesurvey_sync_api_element_children($group) as $qid => $questions) {
      $base = $render_array[$gid][$qid];
      $ls_mandatory = $help = array();
      if (isset($base['#ls_mandatory'])) {
        $ls_mandatory = array('#required' => ($base['#ls_mandatory'] == 'Y'));
      }
      if (isset($base['#ls_help'])) {
        $ls_help = array('#description' => $base['#ls_help']);
      }
      $title = $base['#ls_question'];
      $element = array();
      if ($base['#ls_multi_entry']) {
        $weight_question = 0;
        $question_name = 'question_' . $qid;
        $fieldset = FALSE;
        $options = $values = array();
        foreach (limesurvey_sync_api_element_children($questions) as $aid => $sub_question) {
          $fieldname = $base[$aid]['#ls_fieldname'];
          if (empty($weight_question)) {
            $weight_question = $base[$aid]['#weight'];
          }
          switch ($base['#ls_type']) {
            case 'M': // checkboxes buttons
              if (isset($base[$aid]['#ls_subquestion'])) {
                $options[$fieldname] = $base[$aid]['#ls_subquestion'];
                if ($ls_values[$fieldname]['#ls_value_source'] == 'Y') {
                  $values[] = $fieldname;
                }
                // @todo : need a custom validate function to field correctly
                // the fieldname values or try with the #type 'checbox' instead.
              }
              break;
            case 'Q': // Multipls short text
            case 'K': //Multiple numerical input	Multinumerical	K
              $fieldset = TRUE;
              $sq_name = $fieldname;
              $element[$sq_name] = array(
                '#type' => 'textfield',
                '#title' => $base[$aid]['#ls_subquestion'],
                '#default_value' => $ls_values[$fieldname]['#ls_value_displayed'],
                '#size' => ($base['#ls_type'] == 'Q') ? 20 : 10,
                // '#attributes' => array('class' => array('container-inline')),
                '#weight' => $base[$aid]['#weight'],
                '#prefix' => '<div class="container-inline form-item">',
                '#suffix' => '</div>',
              );
              if ($base['#ls_type'] == 'K') {
                $element[$sq_name]['#element_validate'] = array('limesurvey_sync_answer_is_numeric_validate');
              }
              break;
            case 'C': // Yes/No/Uncertain
              $options = limesurvey_sync_api_answer_ls_question_c_list();
              $fieldset = TRUE;
              $sq_name = $fieldname;
              $element[$sq_name] = array(
                '#type' => 'radios',
                '#title' => $base[$aid]['#ls_subquestion'],
                '#default_value' => (string) $ls_values[$fieldname]['#ls_value_source'],
                '#options' => $options,
                // '#attributes' => array('class' => array('container-inline')),
                '#weight' => $base[$aid]['#weight'],
                '#prefix' => '<div class="container-inline form-item">',
                '#suffix' => '</div>',
              );
              break;
            case 'A': //Array (5 point choice)	FiveRadioArray	A	RadioArray
            case 'B': //Array (10 point choice)	TenRadioArray	B	RadioArray
            case 'E': //Array (Increase/Same/Decrease)	IDRadioArray	E	RadioArray
              $options = ($base['#ls_type'] == 'E') ? limesurvey_sync_api_answer_ls_question_e_list() : limesurvey_sync_api_answer_ls_question_5_list(($base['#ls_type'] == 'B') ? 10 : 5);
              $fieldset = TRUE;
              $sq_name = $fieldname;
              $element[$sq_name] = array(
                '#type' => 'radios',
                '#title' => $base[$aid]['#ls_subquestion'],
                '#default_value' => (string) $ls_values[$fieldname]['#ls_value_source'],
                '#options' => $options,
                // '#attributes' => array('class' => array('container-inline')),
                '#weight' => $base[$aid]['#weight'],
                '#prefix' => '<div class="container-inline form-item">',
                '#suffix' => '</div>',
              );
              break;

          }

        }
        // all sub questions have been parsed.
        switch ($base['#ls_type']) {
          case 'M': // checkboxes buttons
            $element = array(
              '#type' => 'checkboxes',
              '#title' => $title,
              '#default_value' => $values,
              '#options' => $options,
            );
            break;
        }
        if (!$fieldset) {
          $element += $ls_mandatory;
          $element += $ls_help;
          $element += array('#weight' => $weight_question);
          $form['ls_fields'][$group_name][$question_name] = $element;
        }
        else {
          $form['ls_fields'][$group_name][$question_name] = array(
            '#type' => 'fieldset',
            '#title' => $title,
            '#collapsible' => FALSE,
            '#collapsed' => FALSE,
          );
          $form['ls_fields'][$group_name][$question_name] += $ls_mandatory;
          $form['ls_fields'][$group_name][$question_name] += $ls_help;
          $form['ls_fields'][$group_name][$question_name] += array('#weight' => $weight_question);
          // element contain multiple key (each fore a subquestion).
          $form['ls_fields'][$group_name][$question_name] += $element;
        }
      }
      else {
        // No mutli entry question.
        $fieldname = $base['#ls_fieldname'];
        $question_name = $fieldname;
        switch ($base['#ls_type']) {
          case 'L': // radio buttons
            $options = array();
            $aid = '';
            // Retrieve all from the LS answerS table.
            module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_answer');
            $options = limesurvey_sync_answer_retrieve_answers($base['#ls_sid'], $base['#ls_qid'], $ls_lang);
            $options['-oth-'] = t('Other');
            $element = array(
              '#type' => 'radios',
              '#title' => $title,
              '#default_value' => $ls_values[$fieldname]['#ls_value_source'],
              '#options' => $options,
            );
            break;

          case 'N': // Numerical input
            $element = array(
              '#type' => 'textfield',
              '#title' => $title,
              '#default_value' => $ls_values[$fieldname]['#ls_value_displayed'],
              // Add a validation callback for checking if numeric.
              '#element_validate' => array('limesurvey_sync_answer_is_numeric_validate'),
              '#size' => 10,
            );
            break;
          case 'U': //Huge free text	HugeText	U	Text
          case 'T': //Long free text	LongText	T	Text
            $element = array(
              '#type' => 'textarea',
              '#title' => $title,
              '#cols' => ($base['#ls_type'] == 'T') ? 40 : 70,
              '#rows' => ($base['#ls_type'] == 'T') ? 5 : 30,
            );
            break;
          case 'S': //Short free text	ShortText	S	Text
            $element = array(
              '#type' => 'textfield',
              '#title' => $title,
              '#size' => 50,
            );
            break;
          case 'G': //Gender	Gender	G
          case 'Y': //Yes/No	YN	Y
          case '5': // point choice FiveList 5
            if ($base['#ls_type'] == 'Y') {
              $options = limesurvey_sync_api_answer_ls_question_y_list();
            }
            elseif ($base['#ls_type'] == 'G') {
              $options = limesurvey_sync_api_answer_ls_question_g_list();
            }
            elseif ($base['#ls_type'] == '5') {
              $options = limesurvey_sync_api_answer_ls_question_5_list();
            }
            $element = array(
              '#type' => 'radios',
              '#title' => $title,
              '#default_value' => (string) $ls_values[$fieldname]['#ls_value_source'],
              '#options' => $options,
              '#attributes' => array('class' => array('container-inline')),
            );
            break;
          case "!": //List (dropdown)	Select	!	List
            // Retrieve all from the LS answerS table.
            module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_answer');
            $options = limesurvey_sync_answer_retrieve_answers($base['#ls_sid'], $base['#ls_qid'], $ls_lang);
            $options['-oth-'] = t('Other');
            $element = array(
              '#type' => 'select',
              '#title' => $title,
              '#default_value' => (string) $ls_values[$fieldname]['#ls_value_source'],
              '#options' => $options,
            );
            break;
          case 'D': //Date/Time	Date	D
          case 'X': //Text display	Display	X
            $element = array(
              '#type' => '#markup',
              '#markup' => $ls_values[$fieldname]['#ls_value_displayed'],
              '#prefix' => "<div><div><strong>$title</strong></div>",
              '#suffix' => '</div>',
              //'#attributes' => array('class' => array('container-inline')),
            );
            break;

        }
        if (!empty($element)) {
          $element += $ls_mandatory;
          $element += $ls_help;
          $element += array('#weight' => $base['#weight']);
          $form['ls_fields'][$group_name][$question_name] = $element;

        }
      }

    }
    //@TODO : add more question types.
    // Currently : 19 types of question are ok for 29 existings.



  }

  $form['actions'] = array(
    '#type' => 'actions',
  );
  $arg = array('@type' => $entity->ls_entity_type);
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => empty($entity->is_new) ? t('Update LimeSurvey @type', $arg) : t('Save LimeSurvey @type', $arg),
    '#weight' => 40,
  );
  return $form;
}

/**
 * Builder for the answer form page validation.
 *
 * Not available yet. Will be usefull for 'dp2ls' synchronizations.
 *
 * @return
 *   An array, the form elements.
 */
function limesurvey_sync_answer_form_validate($form, &$form_state) {
  // Debug : don't save.
  form_set_error('', t('Answer is not available for saving, coding in progress...'));
}


function limesurvey_sync_answer_is_numeric_validate($form, &$form_state) {
  $fieldname = $form['#parents'];
  if (is_string($fieldname)) {
    if (!is_numeric($form_state['values'][$fieldname])) {
      form_set_error($fieldname, t('the element @title should be numeric.', array('@type' => $form['#title'])));
    }
  }
  else {
    // multi entry
    // @TODO
  }

}

function limesurvey_sync_api_answer_lss_status_list() {
  $status = array(
    LIMESURVEY_SYNC_ANSWER_COMPLETED,
    LIMESURVEY_SYNC_ANSWER_COMPLETED_EDITABLE,
    LIMESURVEY_SYNC_ANSWER_EDITING,
    LIMESURVEY_SYNC_ANSWER_VIRGIN,
  );
  $list = array();
  module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
  foreach ($status as $value) {
    $list[$value] = limesurvey_sync_api_display_field('answer', 'lss_status', $value);
  }
  return $list;
}

function limesurvey_sync_api_answer_ls_question_g_list() {
  return array(
    'M' => t('Male'),
    'F' => t('Female'),
    '' => t('No answer'),
  );
}

function limesurvey_sync_api_answer_ls_question_y_list() {
  return array(
    'Y' => t('Yes'),
    'N' => t('No'),
    '' => t('No answer'),
  );
}

function limesurvey_sync_api_answer_ls_question_5_list($max = 5) {
  $options = array();
  for ($i = 1; $i <= $max; $i++) {
    $options[$i] = $i;
  }
  $options[''] = t('No answer');
  return $options;
}

function limesurvey_sync_api_answer_ls_question_c_list() {
  return array(
    'Y' => t('Yes'),
    'N' => t('No'),
    'U' => t('Uncertain'),
    '' => t('No answer'), // No answer.
  );
}

function limesurvey_sync_api_answer_ls_question_e_list() {
  return array(
    'I' => t('Increase'),
    'D' => t('Decrease'),
    'S' => t('Same'),
    '' => t('No answer'), // No answer.
  );
}

function limesurvey_sync_api_answer_ls_question_m_list() {
  return array(
    'Y' => t('Yes'),
    '' => '', // No answer.
  );
}

function limesurvey_sync_api_answer_ls_question_p_list() {
  return limesurvey_sync_api_answer_ls_question_m_list();
}

function limesurvey_sync_api_answer_ls_question_code_list($field, $instance, $op) {
  // Remove prefix.
  module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_fields');
  limesurvey_sync_ls_api_map_fieldnames($field);
  // Retrieve qid and sid from the fieldname.
  $split = explode('X', $field);
  $sid = $split[0];
  $qid = $split[2];
  // Manage scale from the fieldname.
  $scale = 0;
  if (strpos($field, LIMESURVEY_SYNC_REPLACE_HASK_KEY) === FALSE) {
    $split2 = explode(LIMESURVEY_SYNC_REPLACE_HASK_KEY, $field);
    $suffix = $split2[1];
    if (is_numeric($suffix)) {
      $scale = $suffix;
    }
  }
  module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
  $ls_lang = limesurvey_sync_api_set_ls_lang_from_current_dp_lang($sid);
  module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_answer');
  $options = limesurvey_sync_answer_retrieve_answers($sid, $qid, $scale, $ls_lang);
  $options['-oth-'] = t('Other');

  return $options;
}

/**
 * Return a drupal LS field for display.
 *
 * As an example, return timestamp as formated date, LS language code as human
 * language, etc...
 *
 * @param $field
 *   A string. The Drupal ls_field to display (ex : 'startlanguage').
 * @param $value
 *   A string or any. corresponding Drupal LS field to display (ex:
 *   'de-informal').
 * @param $return
 *   A string. Specify what kind of information return about the field value :
 *   'value' for the displayed text corresponding to the value or 'help' for an
 *   help text expliciting the value or 'css',  the css class corresponding
 *   to this value. Default to 'value'.
 * @param $ls_sid
 *   A numeric. corresponding to the survey id. Default is FALSE.
 *
 * @return
 *   A string. The value ready to be displayed or an help text or a css class
 *   name.
 *
 * @see limesurvey_sync_display_field()
 */
function limesurvey_sync_answer_display_field($field, $value, $return = 'value', $ls_sid = FALSE) {

  $date_fields = array('ls_submitdate', 'ls_startdate', 'ls_datestamp');
  if (in_array($field, $date_fields)) {
    if ($value != '') {
      if (!in_array($return, array('help', 'css'))) {
        return format_date($value);
      }
      else {
        return '';
        // No need to explicit css or help.
      }
    }
    switch ($field) {
      case 'ls_submidate':
        return t('Not completed');
      case 'ls_startdate':
        return t('Not started');
      case 'ls_datestamp':
        return t('Empty');
    }
  }

  if ($field == 'ls_startlanguage') {
    return limesurvey_sync_api_display_field('answer', 'ls_lang', $value, $return, $ls_sid);
  }

  if ($field == 'lss_status') {
    switch ($value) {
      case LIMESURVEY_SYNC_ANSWER_COMPLETED:
        switch ($return) {
          case 'help':
            return t("The answer has been completed and is closed.");
          case 'css':
            return 'limesurvey-sync-field-lss-status-answer-completed';
          default:
            return t('Completed');
        }

      case LIMESURVEY_SYNC_ANSWER_COMPLETED_EDITABLE:
        switch ($return) {
          case 'help':
            return t('The answer has been completed but it can be edited.');
          case 'css':
            return 'limesurvey-sync-field-lss-status-answer-completed-editable';
          default:
            return t('Completed editable');
        }

      case LIMESURVEY_SYNC_ANSWER_EDITING:
        switch ($return) {
          case 'help':
            return t('The answer is being completed.');
          case 'css':
            return 'limesurvey-sync-field-lss-status-answer-editing';
          default:
            return t('Editing');
        }

      case LIMESURVEY_SYNC_ANSWER_VIRGIN:
        switch ($return) {
          case 'help':
            return t('The answer is empty.');
          case 'css':
            return 'limesurvey-sync-field-lss-status-answer-virgin';
          default:
            return t('Virgin');
        }

      default:
        switch ($return) {
          case 'help':
            return t('Unknown answer status.');
          case 'css':
            return 'limesurvey-sync-field-lss-status-answer-unknown';
          default:
            return t('Virgin');
        }
        return t('Unknown');
    }
  }

  if (!in_array($return, array('help', 'css'))) {
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_import');
    module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
    $ls_lang = limesurvey_sync_api_set_ls_lang_from_current_dp_lang($ls_sid);
    return limesurvey_sync_answ_getextendedanswer($field, $value, $ls_sid, $ls_lang);
  }
  else {
    return '';
    // No need to explicit css or help.
  }
}

