<?php

/**
 * @file
 * Functions and classes for the all entities.
 */

define('LIMESURVEY_SYNC_SYNCHRONIZED', 1);
define('LIMESURVEY_SYNC_DESYNCHRONIZED', 2);
define('LIMESURVEY_SYNC_UNSYNCHRONIZED', 3);

// Fieldname for LS question type '1' contain an '#" that break Drupal queries.
// I replace it.
define('LIMESURVEY_SYNC_REPLACE_HASK_KEY', 'z');

/**
 * Class managing building dynamic entities.
 *
 * Building entity_info, entity_property_info, schema, entity default values and
 * updating entity table if needed. Take care of Drupal fields, LimeSurvey fixed
 * fields and Limesurver variables fields.
 */
class LimesurveySyncInfos {
  // Uri for the overview page.
  public $root_admin_ui = 'admin/structure/limesurvey';
  // Uri for the view page.
  public $root_ui = 'limesurvey';
  protected $entity_keys = array();
  // 'survey', or 'response' or 'answer' or 'token'.
  protected $ls_entity_type;
  protected $primary_key = '';
  // If the entity contains 'created' and 'changed' dates fields.
  protected $date_fields = FALSE;
  // If the entity contains 'uid' user field.
  protected $user_field = FALSE;
  // If the entity contains 'language' language field.
  protected $language_field = FALSE;
  // If the entity contains 'lss_sync' synchronized field.
  protected $ls_sync_field = FALSE;
  // If the entity manage revision.
  protected $revision = FALSE;
  // If the entity is fieldable.
  protected $fieldable = FALSE;
  // If the entity is exportable.
  protected $exportable = FALSE;
  // The LS primary key.
  protected $ls_key = '';
  // The LS storing date.
  protected $ls_date_key = '';
  // Created during install : 'survey' and 'response'.
  protected $table_create_during_install = FALSE;
  // Generate many entities for one call : ex : limesurvey_sync_answer_xxxxx,
  // limesurvey_sync_answeranswer_yyyyyy
  protected $sub_entity = FALSE;
  // Correspond to entity_info['bundle of'].
  protected $bundle = FALSE;
  // Default label.
  protected $label = 'Entity';
  // Default plural label.
  protected $plural_label = 'Entities';

  /**
   * Inform the system about one or more entity types
   *
   * Building entity_info corresponding to settings above.
   *
   *   @see hook_entity_info()
   */
  public function entity_info() {

    $return = array();

    // Add bundle info but bypass entity_load() as we cannot use it here.
    $surveys = array();
    if (db_table_exists('limesurvey_sync_survey')) {
      $surveys = db_select('limesurvey_sync_survey', 'lss')->fields('lss')->execute()->fetchAllAssoc('type');
    }
    if ($this->sub_entity) {
      // Create an entity per survey.
      $iterate = $surveys;
    }
    else {
      // Create only one entity.
      $fake_survey = (object) array(
        'ls_sid' => FALSE,
      );
      $iterate = array(
        $fake_survey,
      );
    }

    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_fields');
    // Create one entity type per survey ID.
    foreach ($iterate as $info) {
      $entity_info = array();
      // Do not create the entity if the table does not exists on LS, except for
      // the survey : it will display a link to set the synchronization.
      if ($this->sub_entity && !limesurvey_sync_ls_api_ls_table_exists($this->ls_entity_type, $info->ls_sid)) {
        continue;
      }

      // Create the entity name :
      $entity_name = limesurvey_sync_api_entityname($this->ls_entity_type, $info->ls_sid);
      $entity_table = limesurvey_sync_api_tablename($this->ls_entity_type, $info->ls_sid);
      $revision_table = limesurvey_sync_api_tablename($this->ls_entity_type, $info->ls_sid, TRUE);

      // set the classes name.
      $for_classes = drupal_ucfirst(str_replace('_', '', $this->ls_entity_type));
      // Don't use entity load here.
      $path_admin = limesurvey_sync_api_root_uri($this->ls_entity_type, $info->ls_sid, TRUE);
      if ($this->sub_entity) {
        $path_admin = str_replace($info->type, '%limesurvey_sync_survey', $path_admin);
      }

      $sid_suffix = ($this->sub_entity) ? ' ' . $info->ls_sid : '';
      $entity_info = array(
        'label' => $this->label . $sid_suffix,
        'plural label' => $this->plural_label . $sid_suffix,
        'entity class' => "LimesurveySync{$for_classes}",
        'controller class' => "LimesurveySync{$for_classes}Controller",
        'uri callback' => 'entity_class_uri',
        'module' => 'limesurvey_sync',
        'translation' => array(
          'locale' => TRUE,
        ),
        'base table' => $entity_table,
      );
      if ($this->revision) {
        $entity_info['revision table'] = $revision_table;
      }

      // Build the entity keys according to the setting of each entity.
      $entity_keys = $this->entity_keys;
      if (!empty($this->primary_key)) {
        $entity_keys['id'] = $this->primary_key;
      }
      if ($this->revision) {
        $entity_keys['revision'] = 'vid';
      }
      if ($this->language_field) {
        $entity_keys['language'] = 'language';
      }
      if ($this->bundle) {
        $entity_keys['bundle'] = $this->bundle;
      }
      if ($this->ls_key) {
        // Add custom entries : corresponding LS primary key.
        $entity_keys['ls key'] = $this->ls_key;
      }
      if ($this->ls_date_key) {
        // Add custom entries : LS primary last change date field.
        $entity_keys['ls date key'] = $this->ls_date_key;
      }
      $entity_info['fieldable'] = $this->fieldable;
      $entity_info['exportable'] = $this->exportable;
      $entity_info['entity keys'] = $entity_keys;
      $entity_info['access callback'] = 'limesurvey_sync_api_entity_access';
      // Enable the entity API's admin UI.
      $entity_info['admin ui']['path'] = $path_admin;
      $entity_info['admin ui']['controller class'] = "LimesurveySync{$for_classes}UIController";

      if ($this->sub_entity) {
        // Set dynamic titles for answers and tokens.
        $entity_info['label callback'] = 'limesurvey_sync_api_entity_class_label';
      }

      if ($this->ls_entity_type == 'survey') {
        $entity_info['bundle of'] = 'limesurvey_sync_response';
        $entity_info['admin ui']['menu wildcard'] = '%limesurvey_sync_survey';
      }
      elseif ($this->ls_entity_type == 'response') {
        $entity_info['admin ui']['menu wildcard'] = '%limesurvey_sync_response';
      }

      // Manage bundles.
      $entity_info['bundles'] = array();
      if ($this->ls_entity_type != 'response') {
        $bundle_label = ($this->ls_entity_type == 'survey') ? t('Manage surveys') : $this->ls_entity_type;
        // Create 1 bundle : to allow to add fields to if fiedable is TRUE.
        $entity_info['bundles'][$entity_name] = array(
          'label' => $bundle_label,
          'admin' => array(
            'path' => $path_admin,
            'access arguments' => array("administer limesurvey sync {$this->ls_entity_type}"),
          ),
        );
      }
      else {
        // For response.
        foreach ($surveys as $survey_type => $survey_object) {
          $path_bundle_root = limesurvey_sync_api_root_uri('survey', $survey_object->ls_sid, TRUE);
          $id_count = count(explode('/', $path_bundle_root));
          $entity_info['bundles'][$survey_type] = array(
            'label' => $survey_object->label,
            'admin' => array(
              'path' => $path_bundle_root . '/manage/%limesurvey_sync_survey',
              'real path' => $path_bundle_root . '/manage/' . str_replace('_', '-', $survey_type),
              'bundle argument' => $id_count + 1,
              'access arguments' => array("administer limesurvey sync {$this->ls_entity_type}"),
            ),
          );
        }
        $entity_info['bundle keys']['bundle'] = $this->bundle;
      }
      if ($this->ls_entity_type == 'survey') {
        //$entity_info['admin ui']['menu wildcard'] = '%limesurvey_sync_survey';
      }

      $entity_info['view modes'] = array(
        // @todo we should support view modes.
        'full' => array(
          'label' => t('Full'),
          'custom settings' => FALSE,
        ),
      );
      $entity_info['views controller class'] = 'EntityDefaultViewsController';

      // Save the entity info.
      $return[$entity_name] = $entity_info;
    }

    return $return;
  }

  /**
   * Define the current version of the database schema (hook_schema()).
   *
   * Building schema correponding to settings above and add LS fields.
   *
   * @param $ls_sid
   *   A numeric, the LS survey ID. Default is NULL.
   *
   * @return
   *   An array, the tables schema keyed by the table names.
   *
   *   @see hook_schema()
   */
  public function schema($ls_sid = NULL) {

    $schema = array();

    $surveys = array();
    if ($this->sub_entity) {
      if (db_table_exists('limesurvey_sync_survey')) {
        $query = db_select('limesurvey_sync_survey', 'lss')->fields('lss');
        if (!empty($ls_sid)) {
          $query->condition('ls_sid', $ls_sid);
        }
        $surveys = $query->execute()->fetchAllAssoc('type');
      }
      $iterate = $surveys;
    }
    else {
      $fake_survey = (object) array(
        'ls_sid' => FALSE,
      );
      $iterate = array(
        $fake_survey,
      );
    }
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_fields');
    // Create one entity type per survey ID.
    foreach ($iterate as $info) {

      // Do not create the table if the table does not exists on LS for answer
      // and token.
      if ($this->sub_entity && !limesurvey_sync_ls_api_ls_table_exists($this->ls_entity_type, $info->ls_sid)) {
        continue;
      }
      $entity_name = limesurvey_sync_api_entityname($this->ls_entity_type, $info->ls_sid);
      $entity_table = limesurvey_sync_api_tablename($this->ls_entity_type, $info->ls_sid);
      $schema[$entity_table] = $this->schema_dp_table($info->ls_sid);
      $ls_fields = $this->schema_ls_fields($info->ls_sid);

      $schema[$entity_table]['fields'] = array_merge($this->schema_dp_fields($info->ls_sid), $ls_fields);
      if (!isset($schema[$entity_table]['primary key'])) {
        $schema[$entity_table]['primary key'] = array(
          $this->primary_key,
        );
      }
      if ($this->user_field) {
        $schema[$entity_table]['foreign keys']['uid'] = array(
          'table' => 'user',
          'columns' => array(
            'uid' => 'uid',
          ),
        );
      }

      // Create the revision table.
      if ($this->revision) {
        $revision_table = limesurvey_sync_api_tablename($this->ls_entity_type, $info->ls_sid, TRUE);
        $schema[$revision_table] = array(
          'description' => 'Stores ' . $this->ls_entity_type . ' revisions' . ((!empty($info->ls_sid)) ? ' from the limeSurvey surveys #' . $info->ls_sid . '.' : '.'),
          'fields' => array(),
        );
        // Fix primary key.
        $schema[$revision_table]['primary key'] = array(
          'vid',
        );
        $schema[$revision_table]['fields'] = array_merge($this->schema_dp_fields($info->ls_sid, TRUE), $ls_fields);
      }
    }

    return $schema;
  }

  /**
   * Provide schema table informations.
   *
   * Especially indexes, table description. Fields are defined outside from here.
   *
   * @param $ls_sid
   *   A numeric, the LS survey ID. Default is NULL.
   *
   * @return
   *   An array, the table infos keyed by properties ('description', etc...).
   *
   * @see LimesurveySyncInfos::schema()
   */
  protected function schema_dp_table($ls_sid = NULL) {

    return array();
  }

  /**
   * Define schema for a fixed (known) LS field.
   *
   * For token : Handled tid, completed, firstname, etc... fields and for answer
   * handled id, lastpage, startlanguage, etc.. fields.
   *
   * @param $field
   *   A string, the LS field name.
   *
   * @return
   *   An array, the schema field or FALSE if it is not a fixed field.
   *
   * @see LimesurveySyncInfos::schema()
   */
  protected function schema_ls_fields_known($field) {
    return FALSE;
  }

  /**
   * Return the corresponding Drupal database schema of a LimeSurvey field.
   *
   * This fonction handles LimeSurvey token or answer fields that are specific to
   * a particular LimeSurvey survey. For the token table it corresponds to the
   * attribute fields, and for the answer table it corresponds to the questions
   * fields. The LimeSurvey datetime fields will be converted to timestamp on
   * Drupal.
   *
   * @param $field
   *   A string, the LS field name.
   * @param $sid
   *   A numeric, the LS survey ID.
   *
   * @return
   *   An array of Drupal schema corresponding to the provided LimeSurvey field or
   *   FALSE if the provided field is not a variable LimeSurvey field.
   *
   * @see LimesurveySyncInfos::schema()
   */
  protected function schema_ls_fields_variables($field, $sid) {
    return FALSE;
  }

  /**
   * Define schema for LS fields.
   *
   * Merge known and variable LS fields schema, returning all LS fields schema.
   * If a LS field exists on LS but is not defined on Drupal, provide a watchdog
   * information about it.
   *
   * @param $sid
   *   A numeric, the LS survey ID.
   *
   * @return
   *   An array, the exhaustive LS schema fields or an empty array if there is
   *   no LS fields.
   *
   * @see LimesurveySyncInfos::schema()
   */
  protected function schema_ls_fields($sid = NULL) {

    if (!in_array($this->ls_entity_type, array(
      'token',
      'answer',
    ))) {
      return array();
    }
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_fields');
    $existing_ls_fields = limesurvey_sync_ls_api_retrieve_ls_fields($this->ls_entity_type, $sid);
    // If the token table does not exists on limesurvey, return nothing.
    $ls_current_fields = $existing_ls_fields;
    $fields_schema = array();
    foreach ($ls_current_fields as $key => $current_ls_field) {
      $field_schema = $this->schema_ls_fields_known($current_ls_field);
      if ($field_schema) {
        $fields_schema[$current_ls_field] = $field_schema;
        unset($existing_ls_fields[$key]);
      }
    }

    if (!empty($existing_ls_fields)) {
      $remaining_fields = $existing_ls_fields;
      foreach ($remaining_fields as $key => $variable_ls_field) {

        $field_schema = $this->schema_ls_fields_variables($variable_ls_field, $sid);
        if ($field_schema) {
          $fields_schema[$variable_ls_field] = $field_schema;
          unset($existing_ls_fields[$key]);
        }
      }
    }

    if (!empty($existing_ls_fields) && !db_table_exists(limesurvey_sync_api_tablename($this->ls_entity_type, $sid))) {
      $arg = array(
        '%fields' => implode(', ', $existing_ls_fields),
        '%sid' => $sid,
        '%type' => $this->ls_entity_type,
      );
      watchdog('limesurvey_sync', 'the LimeSurvey fields %fields have not been imported into Drupal (from %type %sid).', $arg, WATCHDOG_WARNING);
      drupal_set_message(t('the LimeSurvey fields %fields have not been imported into Drupal (from %type %sid).', $arg), 'warning');
    }

    return $fields_schema;
  }

  /**
   * Define schema for DP fields.
   *
   * DP fields are 'uid', 'created', 'changed', 'vid', etc...
   *
   * @param $sid
   *   A numeric, the LS survey ID. Default is NULL.
   * @param $revision
   *   A Boolean, TRUE for revision table fields. Default to FALSE.
   *
   * @return
   *   An array, the DP schema fields keyed by there name.
   *
   * @see LimesurveySyncInfos::schema()
   */
  protected function schema_dp_fields($sid = NULL, $revision = FALSE) {
    // set date fields.
    // Fix primary key.
    if (!$revision) {
      $primay_field = array(
        $this->primary_key => array(
          'description' => 'Primary Key: Unique ' . $this->ls_entity_type . ' identifier.',
          'type' => 'serial',
          'unsigned' => TRUE,
          'not null' => TRUE,
        ),
      );
    }
    else {
      $primay_field = array(
        $this->primary_key => array(
          'description' => 'The {' . limesurvey_sync_api_tablename($this->ls_entity_type, $sid) . '} this version belongs to.',
          'type' => 'int',
          'unsigned' => TRUE,
          'not null' => TRUE,
          'default' => 0,
        ),
      );
    }

    $vid_field = array();
    if ($this->revision) {
      if (!$revision) {
        $vid_field = array(
          'vid' => array(
            'description' => 'The current {' . limesurvey_sync_api_tablename($this->ls_entity_type, $sid, TRUE) . '}.vid version identifier.',
            'type' => 'int',
            'unsigned' => TRUE,
            'not null' => FALSE,
            'default' => NULL,
          ),
        );
      }
      else {
        $vid_field = array(
          'vid' => array(
            'type' => 'serial',
            'unsigned' => TRUE,
            'not null' => TRUE,
            'description' => 'The primary identifier for this revision.',
          ),
        );
      }
    }

    $revision_log = array();
    if ($revision) {
      $revision_log = array(
        'log' => array(
          'description' => 'The log entry explaining the changes in this version.',
          'type' => 'text',
          'not null' => TRUE,
          'size' => 'big',
        ),
      );
    }

    $dates_fields = array();
    if ($this->date_fields) {
      if (!$revision) {
        $dates_fields = array(
          'created' => array(
            'description' => 'The Unix timestamp when the ' . $this->ls_entity_type . ' was created.',
            'type' => 'int',
            'not null' => TRUE,
            'default' => 0,
          ),
          'changed' => array(
            'description' => 'The Unix timestamp when the ' . $this->ls_entity_type . ' was most recently saved.',
            'type' => 'int',
            'not null' => TRUE,
            'default' => 0,
          ),
        );
      }
      else {
        $dates_fields = array(
          'timestamp' => array(
            'description' => 'The Unix timestamp when this version was created.',
            'type' => 'int',
            'not null' => TRUE,
            'default' => 0,
          ),
        );
      }
    }

    // Set user field
    $user_field = array();
    if ($this->user_field) {
      if (!$revision) {
        $user_field = array(
          'uid' => array(
            'description' => 'The {users}.uid that owns this ' . $this->ls_entity_type . '; initially, this is the user that created it.',
            'type' => 'int',
            'not null' => TRUE,
            'default' => 0,
          ),
        );
      }
      else {
        $user_field = array(
          'revision_uid' => array(
            'description' => 'The {users}.uid that created this version.',
            'type' => 'int',
            'not null' => TRUE,
            'default' => 0,
          ),
        );
      }
    }

    $language_field = array();
    if ($this->language_field) {
      $language_field = array(
        'language' => array(
          'description' => 'The Drupal {languages}.language of this ' . $this->ls_entity_type . '.',
          'type' => 'varchar',
          'length' => 12,
          'not null' => TRUE,
          'default' => '',
        ),
      );
    }

    $sync_field = array();
    if ($this->ls_sync_field) {
      $sync_field = array(
        'lss_sync' => array(
          'type' => 'int',
          'unsigned' => TRUE,
          'not NULL' => TRUE,
          'default' => 3,
          'size' => 'tiny',
          'description' => 'sync ID (1 for synchronized, 2 for desynchronized, 3 for unsynchronized)',
        ),
      );
    }

    // If I want to make survey entity exportable I need to add thoses fields.
    $exportable_schema = array(
      'status' => array(
        'type' => 'int',
        'not null' => TRUE,
        // Set the default to ENTITY_CUSTOM without using the constant as it is
        // not safe to use it at this point.
        'default' => 0x01,
        'size' => 'tiny',
        'description' => 'The exportable status of the entity.',
      ),
      'module' => array(
        'description' => 'The name of the providing module if the entity has been defined in code.',
        'type' => 'varchar',
        'length' => 255,
        'not null' => FALSE,
      ),
    );

    return array_merge($primay_field, $vid_field, $dates_fields, $revision_log, $user_field, $language_field, $exportable_schema, $sync_field);
  }

  /**
   * Update schema fields regarding changes on the LS database.
   *
   * This method is called on saving a survey entity, compare the current schema
   * stored into a variable and add colomns if needed (as an example : when a
   * question has been added to a survey on the LS admin interface). Set a
   * watchdog entry if add colomns. For the response entity : add corresponding
   * entity reference fields.
   *
   * @return
   *   A boolean, TRUE if the schema has been updated (so the cached schema need
   *   to be rebuild).
   *
   * @see LimesurveySyncInfos::schema()
   * @see LimesurveySyncSurveyController::save()
   */
  public function update_table() {

    if ($this->table_create_during_install || !db_table_exists('limesurvey_sync_survey')) {
      return array();
    }

    //$new_schema = array();
    $surveys = db_select('limesurvey_sync_survey', 'lss')->fields('lss')->execute()->fetchAllAssoc('type');
    $rebuild_schema = FALSE;

    // Create one entity type per survey ID.
    foreach ($surveys as $info) {

      // Create table if not exists.
      $object = limesurvey_sync_api_info_class_object($this->ls_entity_type);
      $schema = $object->schema($info->ls_sid);
      $ls_fields = array_keys($this->schema_ls_fields($info->ls_sid));
      $base_table_name = limesurvey_sync_api_tablename($this->ls_entity_type, $info->ls_sid);
      $variable_name = "{$base_table_name}_ls_fields";
      $stored_ls_fields = variable_get($variable_name, array());
      $update = FALSE;
      $diff = array();
      // Check if the table schema is still valid.
      // It is important that the schema() method returns the base table first,
      // then the revision table.
      foreach ($schema as $table_name => $info_schema) {
        if (!db_table_exists($table_name)) {
          // table does not exists : save it.
          db_create_table($table_name, $info_schema);
          // save it into a variable.
          $update = TRUE;
          $rebuild_schema = TRUE;
          watchdog('limesurvey_sync', 'the LimeSurvey sync table %table has been created.', array(
            '%table' => $table_name,
          ), WATCHDOG_NOTICE);
        }
        else {
          // The table exists : look for changes on fields.
          // It is important that the schema() method returns the base table
          // first, then the revision table.
          if ($table_name == $base_table_name) {
            $diff = array_diff($ls_fields, $stored_ls_fields);
          }
          if (!empty($diff)) {
            // Somes colomns have been added from LS.
            $added_field = array();
            foreach ($diff as $field) {
              if (!db_field_exists($table_name, $field)) {
                db_add_field($table_name, $field, $info_schema['fields'][$field]);
                $added_field[] = $field;
              }
            }
            // save it into a variable.
            $update = TRUE;
            if (!empty($added_field)) {
              $rebuild_schema = TRUE;
              watchdog('limesurvey_sync', 'the LimeSurvey sync table %table has been updated, fields added : %fields.', array(
                '%table' => $table_name,
                '%fields' => implode(', ', $added_field),
              ), WATCHDOG_NOTICE);
            }
          }

          // @TODO : check for deleted fields on LS :
          // $diff = array_diff($stored_ls_fields, $ls_fields);
          // delete them only if there are all NULL into DP.

        }
      }

      // Tables have been created or updated.
      if ($update) {
        // Create or update the bundle.
        // Save the new schema into a variable. Thanks to this, if a new
        // question or attribute is added from LimeSurvey, the system will be
        // able to detect it and add it into Drupal.
        // Save only the base table schema.
        $stored_ls_fields = variable_set($variable_name, $ls_fields);

      }
    }
    if ($rebuild_schema) {
      drupal_get_schema(NULL, TRUE);
    }
    // Return TRUE if the schema as changed.
    return $rebuild_schema;
  }

  /**
   * Provide hook_entity_property_info() for the entity.
   *
   * Thoses informations will be used into views, rules, token modules and the
   * entity view page.
   *
   * @return
   *   An array, the entity_property_info() informations.
   *
   *   @see hook_entity_property_info()
   */
  public function entity_property_info() {

    $property_info = array();

    $surveys = array();
    if ($this->sub_entity) {
      if (db_table_exists('limesurvey_sync_survey')) {
        $surveys = db_select('limesurvey_sync_survey', 'lss')->fields('lss')->execute()->fetchAllAssoc('type');
      }
      $iterate = $surveys;
    }
    else {
      $fake_survey = (object) array(
        'ls_sid' => FALSE,
      );
      $iterate = array(
        $fake_survey,
      );
    }
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_fields');
    // Create one entity type per survey ID.
    foreach ($iterate as $info) {
      // Do not return infos if the table does not exists on LS.
      if ($this->sub_entity && !limesurvey_sync_ls_api_ls_table_exists($this->ls_entity_type, $info->ls_sid)) {
        continue;
      }
      // Create the first entity name : register starting a survey from Drupal.
      $entity_name = limesurvey_sync_api_entityname($this->ls_entity_type, $info->ls_sid);
      $special_fields = array_merge($this->entity_property_info_dp_fields($info->ls_sid), $this->entity_property_info_ls_fields($info->ls_sid));
      // Set the url.
      $arg = array('%type' => $this->ls_entity_type);
      $url = array(
        'url' => array(
          'label' => t('URL'),
          'description' => t('The URL of the %type', $arg),
          'getter callback' => 'entity_metadata_entity_get_properties',
          'type' => 'uri',
          'computed' => 1,
          'entity views field' => 1,
        ),
      );
      // Add the ls_entity_type_field.
      $ls_entity_type_field = array(
        'ls_entity_type' => array(
          'label' => t('LimeSurvey entity type'),
          'description' => t('The type of LimeSurvey entity (%type)', $arg),
          'type' => 'text',
          'entity views field' => TRUE,
        ),
      );

      $property_info[$entity_name]['properties'] = array_merge($url, $ls_entity_type_field, $special_fields);
    }

    return $property_info;
  }

  /**
   * Provided entity_property_info() informations for LS known fields.
   *
   * For token : Handled tid, completed, firstname, etc... fields and for answer
   * handled id, lastpage, startlanguage, etc.. fields.
   *
   * @param $field
   *   A string, the LS field name.
   * @param $sid
   *   A numeric, the LS survey ID.
   *
   * @return
   *   An array, the entity_property_info() field or FALSE if it is not a fixed
   *   field.
   *
   * @see LimesurveySyncInfos::entity_property_info()
   */
  protected function entity_property_info_ls_fields_known($field, $sid) {
    return FALSE;
  }

  /**
   * Provided entity_property_info() informations for LS variable fields.
   *
   * For token : Handled attribute_1, attribute_2, etc... fields and for answer
   * handled 79447X7X1731, 79447X7X1732, etc.. fields.
   *
   * @param $field
   *   A string, the LS field name.
   * @param $sid
   *   A numeric, the LS survey ID.
   *
   * @return
   *   An array, the entity_property_info() field or FALSE if it is not an
   *   existing variable field.
   *
   * @see LimesurveySyncInfos::entity_property_info()
   */
  protected function entity_property_info_ls_fields_variables($field, $sid) {
    return FALSE;
  }

  /**
   * Provided entity_property_info() informations for LS fields.
   *
   * Merge known and variable LS fields entity_property_info, returning all LS
   * fields entity_property_info. If a LS field exists on LS but is not defined
   * on Drupal, provide a watchdog information about it.
   *
   * @param $sid
   *   A numeric, the LS survey ID. Default is NULL.
   *
   * @return
   *   An array, the exhaustive LS entity_property_info() fields or an empty
   *   array if there is no LS fields.
   *
   * @see LimesurveySyncInfos::entity_property_info()
   */
  protected function entity_property_info_ls_fields($sid = NULL) {

    if (!in_array($this->ls_entity_type, array(
      'token',
      'answer',
    ))) {
      return array();
    }

    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_fields');
    $existing_ls_fields = limesurvey_sync_ls_api_retrieve_ls_fields($this->ls_entity_type, $sid);
    $ls_current_fields = $existing_ls_fields;
    $fields_entity_property_info = array();
    foreach ($ls_current_fields as $key => $current_ls_field) {
      $field_entity_property_info = $this->entity_property_info_ls_fields_known($current_ls_field, $sid);
      if ($field_entity_property_info) {
        $fields_entity_property_info[$current_ls_field] = $field_entity_property_info;
        unset($existing_ls_fields[$key]);
      }
    }

    if (!empty($existing_ls_fields)) {
      $remaining_fields = $existing_ls_fields;
      foreach ($remaining_fields as $key => $variable_ls_field) {
        $field_entity_property_info = $this->entity_property_info_ls_fields_variables($variable_ls_field, $sid);
        if ($field_entity_property_info) {
          $fields_entity_property_info[$variable_ls_field] = $field_entity_property_info;
          unset($existing_ls_fields[$key]);
        }
      }
    }

    if (!empty($existing_ls_fields) && !db_table_exists(limesurvey_sync_api_tablename($this->ls_entity_type, $sid))) {
      $arg = array(
        '%fields' => implode(', ', $existing_ls_fields),
        '%sid' => $sid,
        '%type' => $this->ls_entity_type,
      );
      watchdog('limesurvey_sync', 'the LimeSurvey fields %fields have no defined properties into Drupal (from %type %sid).', $arg, WATCHDOG_WARNING);
      drupal_set_message(t('the LimeSurvey fields %fields have no defined properties into Drupal (from %type %sid).', $arg), 'warning');
    }

    // Add ls_sid field.
    $ls_sid_field = array(
      'ls_sid' => array(
        'label' => t('_ID, LimeSurvey survey ID'),
        'description' => t('LimeSurvey survey ID into the LimeSurvey database.'),
        'type' => 'integer',
        'validation callback' => 'entity_metadata_validate_integer_positive',
        'entity views field' => TRUE,
      ),
    );
    $fields_entity_property_info += $ls_sid_field;
    return (!empty($fields_entity_property_info)) ? $fields_entity_property_info : array();
  }

  /**
   * Provided entity_property_info() informations for DP fields.
   *
   * DP fields are 'uid', 'created', 'changed', 'vid', etc...
   *
   * @param $sid
   *   A numeric, the LS survey ID. Default is NULL.
   *
   * @return
   *   An array, the DP entity_property_info() fields.
   *
   * @see LimesurveySyncInfos::entity_property_info()
   */
  protected function entity_property_info_dp_fields($sid = NULL) {
    // No html for label.
    // set some prefix for sorting :
    // '_ID' for dp and LS ids.
    // 'DP'for drupal fields.
    // 'LS admin for ls admin fields.
    // 'LS datas' then the field name without prefix for answer datas and token
    // attributes.
    // No need to add the survey ID : provided by views.

    $survey_arg = (!empty($sid)) ? array(
      '@survey' => t(' (survey #@sid)', array(
        '@sid' => $sid,
      )),
    ) : array();
    $arg_label = array_merge(array(
      '@type' => $this->ls_entity_type,
    ), $survey_arg);
    $survey_desc = (!empty($sid)) ? array(
      '%survey' => t(' (survey #@sid)', array(
        '@sid' => $sid,
      )),
    ) : array();
    $arg_desc = array_merge(array(
      '%type' => $this->ls_entity_type,
    ), $survey_desc);

    // Set primary key.
    $primary_field = array(
      $this->primary_key => array(
        'label' => t('_ID, Drupal @type ID', $arg_label),
        'description' => t('Internal, numeric ID.'),
        'type' => 'integer',
        'validation callback' => 'entity_metadata_validate_integer_positive',
        'schema field' => $this->primary_key,
      ),
    );
    // set date fields.
    $dates_fields = array();
    if ($this->date_fields) {
      $dates_fields = array(
        'created' => array(
          'label' => t('DP @type created'),
          'description' => t('The Unix timestamp when the %type was created%survey.', $arg_desc),
          'type' => 'date',
          'schema field' => 'created',
        ),
        'changed' => array(
          'label' => t('DP @type changed date'),
          'description' => t('The Unix timestamp when the %type was most recently saved%survey.', $arg_desc),
          'type' => 'date',
          'schema field' => 'created',
        ),
      );
    }

    // Set user field
    $user_field = array();
    if ($this->user_field) {
      $user_field = array(
        'uid' => array(
          'label' => t('DP @type author', $arg_label),
          'description' => t('The author of the %type%survey.', $arg_desc),
          'type' => 'user',
          'schema field' => 'uid',
        ),
      );
    }

    $language_field = array();
    if ($this->language_field) {
      $language_field = array(
        'language' => array(
          'label' => t('DP @type language@survey', $arg_label),
          'description' => t('The %type Drupal language.%survey', $arg_desc),
          'type' => 'text',
          'schema field' => 'language',
        ),
      );
    }

    // set the vid field.
    $vid_field = $log_field = array();
    if ($this->revision) {

      $vid_field = array(
        'vid' => array(
          'label' => t('DP the @type version ID', $arg_label),
          'description' => t('The %type Drupal  version ID%survey.', $arg_desc),
          'type' => 'text',
          'schema field' => 'vid',
        ),
      );

      $log_field = array(
        'log' => array(
          'label' => t("DP revision log message"),
          'type' => 'text',
          'description' => t("In case a new revision is to be saved, the log entry explaining the changes for this version."),
          //'setter callback' => 'entity_property_verbatim_set',
          //'access callback' => '_limesurvey_sync_api_revision_access',
        ),
      );

    }

    $sync_field = array();
    if ($this->ls_sync_field) {
      $sync_field = array(
        'lss_sync' => array(
          'label' => t('DP synchronization status'),
          'description' => t('sync ID (1 for synchronized, 2 for desynchronized, 3 for unsynchronized)'),
          'type' => 'integer',
          'schema field' => 'lss_sync',
          'options list' => 'limesurvey_sync_api_lss_sync_status_list',
          'setter permission' => "administer limesurvey sync {$this->ls_entity_type}",
        ),
      );
    }

    return array_merge($primary_field, $dates_fields, $user_field, $language_field, $vid_field, $log_field, $sync_field);
  }


  /**
   * Provide default values on creating entity.
   *
   * @return
   *   An array, the defaut values keyed by there field name.
   */
  public function entity_default_values($ls_sid = NULL) {

    $default_values = $this->create_dp_fields_default_values($ls_sid);

    $special_fields = ($this->sub_entity) ? $this->create_ls_fields_default_values($ls_sid) : array();

    // set date fields.
    $dates_fields = array();
    if ($this->date_fields) {
      // 'created' and 'changed' fields are handled into the save() method.
      $dates_fields = array(
        'created' => NULL,
        'changed' => NULL,
      );
    }

    // Set user field
    $user_field = array();
    if ($this->user_field) {
      // 'user' field is handled into the save() method.
      global $user;
      $user_field = array('user' => $user->uid);
    }

    $language_field = array();
    if ($this->language_field) {
      // 'language' field is handled into the save() method.
      global $language;
      $language_field = array('language' => $language->language);
    }
    // set the vid field.
    $vid_field = array();
    if ($this->revision) {
      // 'vid' field is handled into the save() method.
      $vid_field = array('vid' => NULL);
    }
    $default_values = array_merge($default_values, $vid_field, $dates_fields, $user_field, $language_field, $special_fields);

    return $default_values;
  }

  /**
   * Provide default values on creating entity.
   *
   * @param $sid
   *   A numeric, the LS survey ID. Default is NULL.
   *
   * @return
   *   An array, the defaut values keyed by there field name.
   *
   * @see LimesurveySyncInfos::entity_default_values()
   */
  protected function create_dp_fields_default_values($ls_sid = NULL) {

    return array();
  }

  /**
   * Provide default values on creating entity for LS fields.
   *
   * Merge known and variable LS fields default values. If a LS field exists on
   * LS but is not defined on Drupal, provide a watchdog information about it.
   *
   * @param $sid
   *   A numeric, the LS survey ID. Default is NULL.
   *
   * @return
   *   An array, the exhaustive defaut values keyed by there field name for LS
   *   fields.
   *
   * @see LimesurveySyncInfos::entity_default_values()
   */
  protected function create_ls_fields_default_values($sid = NULL) {

    if (!in_array($this->ls_entity_type, array(
      'token',
      'answer',
    ))) {
      return array();
    }

    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_fields');
    $existing_ls_fields = limesurvey_sync_ls_api_retrieve_ls_fields($this->ls_entity_type, $sid);
    $ls_current_fields = $existing_ls_fields;
    $default_values = array();
    foreach ($ls_current_fields as $key => $current_ls_field) {
      $field_default_values = $this->create_ls_fields_default_values_known($current_ls_field, $sid);
      if ($field_default_values !== FALSE) {
        $default_values[$current_ls_field] = $field_default_values;
        unset($existing_ls_fields[$key]);
      }
    }

    if (!empty($existing_ls_fields)) {
      $remaining_fields = $existing_ls_fields;
      foreach ($remaining_fields as $key => $variable_ls_field) {
        $field_default_values = $this->create_ls_fields_default_values_variables($variable_ls_field, $sid);
        if ($field_default_values !== FALSE) {
          $default_values[$variable_ls_field] = $field_default_values;
          unset($existing_ls_fields[$key]);
        }
      }
    }

    if (!empty($existing_ls_fields) && !db_table_exists(limesurvey_sync_api_tablename($this->ls_entity_type, $sid))) {
      $arg = array(
        '%fields' => implode(', ', $existing_ls_fields),
        '%sid' => $sid,
        '%type' => $this->ls_entity_type,
      );
      watchdog('limesurvey_sync', 'the LimeSurvey fields %fields have no default value into Drupal (from %type %sid).', $arg, WATCHDOG_WARNING);
      drupal_set_message(t('the LimeSurvey fields %fields have no default value into Drupal (from %type %sid).', $arg), 'warning');
    }

    return (!empty($default_values)) ? $default_values : array();
  }

  /**
   * Return the corresponding Drupal default value of a LS known field.
   *
   * This fonction is used for creating a new answer or token Drupal entity.
   * Handle common LimeSurvey fields of the token and answer tables.
   *
   * @param $field
   *   A string. The LimeSurvey field name.
   * @param $sid
   *   A numeric. The LimeSurvey survey ID. Default is NULL.
   *
   * @return
   *   A string or numeric or NULL : the Drupal default value for the provided
   *   LimeSurvey field or FALSE if the provided field is not a generic (known)
   *   LimeSurvey field.
   *
   * @see LimesurveySyncInfos::entity_default_values()
   */
  protected function create_ls_fields_default_values_known($field, $sid = NULL) {
    return FALSE;
  }

  /**
   * Return the corresponding Drupal default value of a LS variable field.
   *
   * This fonction is used for creating a new answer or token Drupal entity.
   * Handle variable LimeSurvey fields of the token and answer tables.
   *
   * @param $field
   *   A string. The LimeSurvey field name.
   * @param $sid
   *   A numeric. The LimeSurvey survey ID. Default is NULL.
   *
   * @return
   *   A string or numeric or NULL : the Drupal default value for the provided
   *   LimeSurvey field or FALSE if the provided field is not a variable
   *   LimeSurvey field.
   *
   * @see LimesurveySyncInfos::entity_default_values()
   */
  protected function create_ls_fields_default_values_variables($field, $sid = NULL) {
    return FALSE;
  }

  /**
   * Provide hook_permission for the entity.
   *
   * @return
   *   An array, the entity permissions.
   *
   * @see hook_permission()
   */
  public function permission() {
    // We set up permisssions to manage entity types, manage all entities and the
    // permissions for each individual entity
    // @TODO : need to translate this throw an handler.
    $arg = array(
      '@type' => $this->ls_entity_type,
      '@types' => $this->ls_entity_type . 's',
    );
    $permissions = array();
    $permissions = array(
      "administer limesurvey sync {$this->ls_entity_type}" => array(
        'title' => t('Administer LimeSurvey @types', $arg),
        'description' => t('Create and delete, synchronize LimeSurvey @types, and set their permissions.', $arg),
      ),
    );

    $no_bundles = ($this->ls_entity_type == 'survey');
    if (!$no_bundles) {
      // Create a set of permissions per survey.
      $iterate = limesurvey_sync_survey_get_surveys();
    }
    else {
      // Create only one set of permissions.
      $fake_survey = (object) array('type' => FALSE);
      $iterate = array($fake_survey);
    }

    // Generate permissions per LimeSurvey surveys
    foreach ($iterate as $type) {
      // Don't provide permission for if no associated entities.
      if ($this->sub_entity && !db_table_exists(limesurvey_sync_api_tablename($this->ls_entity_type, $type->ls_sid))) {
        continue;
      }
      $type_machine = $type_human =  '';
      if (!$no_bundles) {
        $type_machine = check_plain($type->type);
        $type_human = $type->getTranslation('label');
      }


      $perm_machine_suffix = ($no_bundles) ?  "{$this->ls_entity_type}" : "{$type_machine} {$this->ls_entity_type}";
      $perm_human_prefix =  ($no_bundles) ?  '' : t('%type_name:', array('%type_name' => $type_human)) . ' ';
      // Adding answer or token entities from user interface is not possible.
      // I may active it if dp2ls synchronization is enable.
      if (!$this->sub_entity) {
        $permissions += array(
          "add {$perm_machine_suffix}" => array(
            'title' => $perm_human_prefix . t('Create new @type', $arg),
          ),
        );
      }
      // Update permission is used for tokens and answers synchronization.
      $permissions += array(
        "update own {$perm_machine_suffix}" => array(
          'title' => $perm_human_prefix . t('Edit own @type', $arg),
        ),
        "update any {$perm_machine_suffix}" => array(
          'title' => $perm_human_prefix . t('Edit any @type', $arg),
        ),
        "view own {$perm_machine_suffix}" => array(
          'title' => $perm_human_prefix . t('View own @type', $arg),
        ),
        "view any {$perm_machine_suffix}" => array(
          'title' => $perm_human_prefix . t('View any @type', $arg),
        ),
        "delete own {$perm_machine_suffix}" => array(
          'title' => $perm_human_prefix . t('Delete own @type', $arg),
        ),
        "delete any {$perm_machine_suffix}" => array(
          'title' => $perm_human_prefix . t('Delete any @type', $arg),
        ),
      );
      // Handle revision permissions.
      if ($this->revision) {
        $permissions += array(
          "view {$perm_machine_suffix} revisions" => array(
            'title' => $perm_human_prefix . t('View any @type revisions', $arg),
          ),
          "revert {$perm_machine_suffix} revisions" => array(
            'title' => $perm_human_prefix . t('Revert any @type revisions', $arg),
          ),
          "delete {$perm_machine_suffix} revisions" => array(
            'title' => $perm_human_prefix . t('Delete any @type revisions', $arg),
          ),
        );
      }
    }
    return $permissions;
  }
}

/**
 * Class managing entities.
 *
 * Extend the common class 'Entity' for entities.
 */
class LimesurveySync extends Entity {
  public $ls_entity_type = NULL;

  /**
   * Creates a new entity.
   *
   * Retreive the LS survey ID from the entity type name, making sure to find it
   * for answers and token entities.
   *
   * @see entity_create()
   */
  public function __construct(array $values = array(), $entityType = NULL) {

    $with_suffix = array('token', 'answer');
    if (in_array($this->ls_entity_type, $with_suffix)) {
      $ls_sid = limesurvey_sync_api_retrieve_ls_sid_from_entityname($entityType, $this->ls_entity_type);
      if (empty($ls_sid)) {
        throw new Exception('Cannot create an instance of ' . $this->ls_entity_type . ' entity without a specified survey id.');
      }
      else {
        $this->ls_sid = $ls_sid;
      }
    }
    parent::__construct($values, $entityType);
  }

  /**
   * Synchronize the entity from LS to DP .
   *
   * @param $all
   *   A string, 'all' or 'all_skip_import' for synchronizing all correspopnding
   *   entities, or FALSE for synchronizing the provided entity. Default to
   *   FALSE.
   */
  public function ls_synchronize($all = FALSE, $way = 'ls2dp') {
    $arg = ($all != 'all') ? $this : 'all';
    return entity_get_controller($this->entityType)->ls_synchronize($arg, $way);
  }

  /**
   * Returns the label of the entity.
   *
   * Modules may alter the label by specifying another 'label callback' using
   * hook_entity_info_alter(). Return a dynamic label for token and answer
   * entities.
   *
   * @see entity_label()
   */
  protected function defaultLabel() {
    if (!empty($this->entityInfo['entity keys']['ls key'])) {
      $ls_key = $this->entityInfo['entity keys']['ls key'];
      return limesurvey_sync_api_default_label($this, $ls_key);
    }
  }

  /**
   * Returns the uri of the entity just as entity_uri().
   *
   * Modules may alter the uri by specifying another 'uri callback' using
   * hook_entity_info_alter(). Set uri from LimesurveySyncInfos::root_ui value.
   *
   * @see entity_uri()
   */
  protected function defaultUri() {

    return array(
      'path' => limesurvey_sync_api_root_uri($this->ls_entity_type, $this->ls_sid) . "/{$this->identifier()}",
    );
  }

  /**
   * Returns whether the survey type is locked, thus may not be deleted or renamed.
   *
   */
  public function isLocked() {

    return isset($this->status) && empty($this->is_new) && (($this->status & ENTITY_IN_CODE) || ($this->status & ENTITY_FIXED));
  }
}

/**
 *  A controller implementing exportables stored in the database.
 *
 * Extend the common class 'EntityAPIControllerExportable' for entities.
 */
class LimesurveySyncController extends EntityAPIControllerExportable {
  public $ls_entity_type = '';
  public $ls_sid = '';
  public $ls_key = '';

  /**
   * Overridden.
   *
   * Retreive the LS survey ID from the entity type name, making sure to find it
   * for answers and token entities.
   */
  public function __construct($entityType) {

    $with_suffix = array('token', 'answer');
    $ls_sid = limesurvey_sync_api_retrieve_ls_sid_from_entityname($entityType, $this->ls_entity_type);
    if (empty($ls_sid) && in_array($this->ls_entity_type, $with_suffix)) {
      throw new Exception('Cannot create an instance of ' . $this->ls_entity_type . ' entity without a specified survey id.');
    }
    else {
      $this->ls_sid = $ls_sid;
    }
    parent::__construct($entityType);
  }

  /**
   * Overridden.
   *
   * Set default values on creating entity.
   */
  public function create(array $values = array()) {
    // Add values that are specific to our LimesurveySyncSurvey
    $entity_infos = limesurvey_sync_api_info_class_object($this->ls_entity_type);
    $ls_sid = (!empty($values['ls_sid'])) ? $values['ls_sid'] : $this->ls_sid;
    $fields = $entity_infos->entity_default_values($ls_sid);
    $values += $fields;
    $entity = parent::create($values);
    return $entity;
  }

  /**
   * Overridden.
   *
   * Add dynamic title for answers and token entities.
   */
  public function load($ids = array(), $conditions = array()) {

    $entities = parent::load($ids, $conditions);
    if (empty($this->entityInfo['entity keys']['label']) && !empty($this->entityInfo['entity keys']['ls key'])) {
      // For answers and token entities.
      $ls_key = $this->entityInfo['entity keys']['ls key'];
      foreach ($entities as $entity) {
        // Add dynamic title.
        if (!isset($entity->title)) {
          $entity->title = limesurvey_sync_api_default_label($entity, $ls_key);
        }

        // Provide uid from the response table.
        // Note : uid may be an array if various values exists. Pick the first one corresponding to the most recent
        if (!isset($entity->uid)) {
          $uid = limesurvey_sync_api_retreive_uid_from_ref_entity($entity);
          $entity->uid = (!is_array($uid)) ? $uid : current($uid);
        }
      }
    }
    return $entities;
  }

  /**
   * Provide the LimeSurvey Sync module entity status.
   *
   * Corresponding to LS states (editing, completed, etc...). each subclass will
   * implement it.
   *
   * @param $entity
   *   The entity object.
   *
   * @return
   *   A numeric defined as a constant corresponding to a defined LS stage.
   */
  protected function lss_status($entity) {
    // Each class will implement it.
    return FALSE;
  }

  /**
   * Provide the LimeSurvey Sync module entity synchronzation status.
   *
   * Check if the Drupal entity is binded to a valid LimeSurvey entry.
   *
   * @param $entity
   *   The entity object.
   * @param $existing_ls_enties_keyed_by_ls_key
   *   An array : the LimeSurvey entries keyed by the LS primary key. For
   *   performance raisons, this parameter will usually contain only one key
   *   corresponding of the one from the DP entry. Default is empty.
   *
   * @return
   *   A numeric defined as a constant : LIMESURVEY_SYNC_SYNCHRONIZED (1) for
   *   synchronized, LIMESURVEY_SYNC_DESYNCHRONIZED (2) for desynchronized (this
   *   entry had exist into the ls table but has been deleted into the ls table)
   *   or LIMESURVEY_SYNC_UNSYNCHRONIZED (3) for unsynchronized (this dp entry
   *   has never had a reference into the ls table).
   */
  protected function lss_sync($entity, $existing_ls_enties_keyed_by_ls_key = array()) {

    $ls_key = isset($this->entityInfo['entity keys']['ls key']) ? $this->entityInfo['entity keys']['ls key'] : '';
    if (!$ls_key || empty($entity->$ls_key)) {
      // This dp entry has never had a reference into the ls table. This
      // situation should not exists.
      return LIMESURVEY_SYNC_UNSYNCHRONIZED;
    }
    elseif (array_key_exists($entity->$ls_key, $existing_ls_enties_keyed_by_ls_key)) {
      return LIMESURVEY_SYNC_SYNCHRONIZED;
    }
    else {
      // This entry had exist into the ls table but has been deleted into
      // the ls table.
      return LIMESURVEY_SYNC_DESYNCHRONIZED;
    }
  }

  /**
   * Permanently saves the given entity, synchronizing it.
   *
   * On saving entity, update/add informations from LimeSurvey to Drupal. Reset
   * the flag 'need_save_ls2dp' to FALSE. Manage date user, log fields. In case
   * of failures, an exception is thrown.
   *
   * @param $entity
   *   The entity to save.
   *
   * @return
   *   SAVED_NEW or SAVED_UPDATED is returned depending on the operation
   *   performed.
   */
  public function save($entity, DatabaseTransaction $transaction = NULL) {

    // Entity can be saved using entity_save(). In order to avoid calling it
    // twice and to synchronize datas on saving, add a flag.
    // Revisions should not be synchronized.
    $is_default_revision = entity_revision_is_default($this->entityType, $entity);
    $need_save_ls2dp = (!isset($entity->need_save_ls2dp) || $entity->need_save_ls2dp);
    $not_activate_token = (!isset($entity->activate_token) || !$entity->activate_token);
    $need_save_dp2ls = (isset($entity->need_save_dp2ls) && $entity->need_save_dp2ls);
    if ($is_default_revision) {
      $dp_key = $this->idKey;
      $dp_key_value = (!empty($entity->$dp_key)) ? $entity->$dp_key : 'this';
      if ($need_save_dp2ls) {
        $synchronize_set = $this->ls_synchronize($entity, 'dp2ls');
        // If synchronization needed : apply the changes.
        if (!empty($synchronize_set['need_sync'])) {
          $diff = $synchronize_set['need_sync'][$dp_key_value]['diff'];
          // Update the LS entry.
          module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_fields');
          $succeed = limesurvey_sync_ls_api_edit_row($entity, $diff);
        }
        if ($succeed) {
          // Reset the flag.
          $entity->need_save_dp2ls = FALSE;
          // Save revision on each changes on LS.
          $entity->revision = TRUE;
          // Set a log message expliciting diff fields.
          if (empty($entity->log)) {
            $entity->log = '';
          }
          else {
            $entity->log .= ' ';
          }
          $entity->log .= t('Sync DP -> LS : changes on @fields.', array('@fields' => implode(', ', array_keys($diff))));
        }

        // Sync ls2dp now for lss_status and lss_sync.
        $need_save_ls2dp = TRUE;
      }

      if ($need_save_ls2dp && $not_activate_token) {

        $synchronize_set = $this->ls_synchronize($entity);

        // If synchronization needed : apply the changes.
        if (!empty($synchronize_set['need_sync'])) {
          $diff = $synchronize_set['need_sync'][$dp_key_value]['diff'];
          foreach ($diff as $key => $value) {
            $entity->$key = $value;
          }
          // Reset the flag.
          $entity->need_save_ls2dp = FALSE;
          // Save revision on each changes on LS.
          $entity->revision = TRUE;
          // Set a log message expliciting diff fields.
          if (empty($entity->log)) {
            $entity->log = '';
          }
          else {
            $entity->log .= ' ';
          }
          $entity->log .= t('Sync LS -> DP : changes on @fields.', array('@fields' => implode(', ', array_keys($diff))));
        }
      }
    }

    if (in_array('created', $this->entityInfo['schema_fields_sql']['base table'])) {
      // Care about setting created and changed values. But do not automatically
      // set a created values for already existing surveys.
      if (empty($entity->created) && !empty($entity->is_new)) {
        $entity->created = REQUEST_TIME;
      }
    }
    if (in_array('changed', $this->entityInfo['schema_fields_sql']['base table'])) {
      $entity->changed = REQUEST_TIME;
    }
    if (in_array('status', $this->entityInfo['schema_fields_sql']['base table']) && !empty($entity->is_new)) {
      $entity->status = ENTITY_CUSTOM;
      $entity->module = 'limesurvey_sync';
    }

    if (!empty($this->entityInfo['entity keys']['revision'])) {
      if (in_array('timestamp', $this->entityInfo['schema_fields_sql']['revision table'])) {
        $entity->timestamp = REQUEST_TIME;
      }

      if (in_array('log', $this->entityInfo['schema_fields_sql']['revision table'])) {

        if ((!empty($entity->is_new)) || !empty($entity->revision)) {
          // When inserting either a new entity or a new node revision, $entity->log
          // must be set because {revision}.log is a text column and therefore
          // cannot have a default value. However, it might not be set at this
          // point (for example, if the user submitting a entity form does not have
          // permission to create revisions), so we ensure that it is at least an
          // empty string in that case.
          // @todo: Make the {revision}.log column nullable so that we can
          // remove this check.
          if (!isset($entity->log)) {
            $entity->log = '';
          }
        }
        elseif (!isset($entity->log) || $entity->log === '') {
          // If we are updating an existing entity without adding a new revision, we
          // need to make sure $entity->log is unset whenever it is empty. As long as
          // $entity->log is unset, drupal_write_record() will not attempt to update
          // the existing database column when re-saving the revision; therefore,
          // this code allows us to avoid clobbering an existing log entry with an
          // empty one.
          unset($entity->log);
        }
      }
    }

    if (in_array('uid', $this->entityInfo['schema_fields_sql']['base table'])) {
      global $user;
      if (!isset($entity->uid) && !empty($entity->is_new)) {
        $entity->uid = 0;
      }
      $entity->revision_uid = $user->uid;
    }

    if (in_array('language', $this->entityInfo['schema_fields_sql']['base table'])) {
      if (empty($entity->language) && !empty($entity->is_new)) {
        global $language;
        $entity->language = $language->language;
      }
    }
    // Use entity_revision_set_default($entity_type, $entity) for make-current.

    $return = parent::save($entity, $transaction);

    // Watchdog.
    $survey_label = '';
    if (in_array($this->ls_entity_type, array('survey', 'participant'))) {
      $survey_label = $this->ls_entity_type;
    }
    elseif ($this->ls_entity_type == 'response') {
      $survey_type = $entity->type;
    }
    else {
      // answer and token.
      $survey_type = limesurvey_sync_api_typename('survey', $entity->ls_sid);
    }
    if (!$survey_label && $survey_type) {
      $survey = limesurvey_sync_survey_get_surveys($survey_type);
      $survey_label = (!empty($survey->label)) ? $survey->label : '';
    }
    $watchdog_args = array(
      '@type' => drupal_strtoupper(drupal_substr($this->ls_entity_type, 0, 1)) . drupal_substr($this->ls_entity_type, 1),
      '%title' => entity_label($this->entityType, $entity),
      '@survey' => $survey_label,
    );
    $link = l(t('view'), current(entity_uri($this->entityType, $entity)));
    if (in_array($return, array(SAVED_NEW, SAVED_UPDATED))) {
      $severity = WATCHDOG_NOTICE;
      if ($return == SAVED_NEW) {
        $message = '@survey: %title @type has been created.';
      }
      else {
        $message = '@survey: %title @type has been updated.';
      }
    }
    else {
      $severity = WATCHDOG_ERROR;
      $message = '@survey: %title @type saving process has failed.';
    }
    watchdog('limesurvey_sync', $message, $watchdog_args, $severity, $link);
  }

  /**
   * Test the synchronization from LimeSurvey to Drupal or Dp2LS..
   *
   * Compare the DP entry to the current LS one. Update the entity adding a flag
   * 'need_save_ls2dp' to TRUE or FALSE. This function can be called for one
   * entity or for all entities from this entity type.
   *
   * @param $entity
   *   The entity object to save or 'all' for all entities from this entity
   *   type, or 'all_skip_import' for all entities from this entity
   *   type but without returning (for permformance raisons) entities that
   *   have never been saved on DP.
   * @param $way
   *   A string, the synchronization way. 'ls2dp' or 'dp2ls'. Default is 'ls2dp'.
   *
   * @return
   *   An array. Keys are ['ok'][dp_primary_key_value] if synchronization is
   *   not needed; ['need_sync'] if synchronization is needed, with subkeys :
   *   ['need_sync'][dp_primary_key_value]['dp'] the DP current entity object,
   *   ['need_sync'][dp_primary_key_value]['diff'] an array of the changes keyed
   *   by the properties; ['need_sync'][dp_primary_key_value]['dp_date'] the
   *   last changed date on DP and ['need_sync'][dp_primary_key_value]['ls_date']
   *   the last changed date on LS (used to compare witch one is the most
   *   recent, for selecting ls2dp or dp2ls synchronization in future
   *   developpements). At last ['need_import'][ls_primary_key_value] as the
   *   converted to DP entity object to save for LS entries that have no entry
   *   in DP. ['need_import'] only exists with the 'all' parameter. Note : for
   *   entity to create, the dp_primary_key_value from
   *   ['need_sync'][dp_primary_key_value] is 'this'. It should be taken in
   *   consideration when browsing the result of the ls_synchronize()
   *   method.
   */
  public function ls_synchronize(&$entity, $way = 'ls2dp') {

    $dp_key = $this->idKey; // entityInfo['entity keys']['id'];
    $ls_key = isset($this->entityInfo['entity keys']['ls key']) ? $this->entityInfo['entity keys']['ls key'] : '';
    $this_ls_id = FALSE;
    $this_dp_id = FALSE;

    // Retrieve from Drupal.
    if ($entity != 'all' && $entity != 'all_skip_import') {
      // If an antity is provided sync only this one.
      // The entity may have been altered : use this one.
      $entities = array($entity);
      if (!empty($entity->$dp_key)) {
        $this_dp_id = $entity->$dp_key;
      }
      else {
        // Only one entity : Use 'this' index.
        $this_dp_id = 'this';
      }

      // For retreiving from LS.
      if ($ls_key && !empty($entity->$ls_key)) {
        $this_ls_id = $entity->$ls_key;
      }
      else { //if (!$ls_key) {
        // The response entity arrive here.
        // Try to create/update an entity with no ID references to LS.
        // Note : To add a LS token, save it first on LS, then synchronize it
        // using its new ID.
      }
    }
    else {
      // Sync all entities.
      // Retrieve from Drupal.
      $entities = entity_load($this->entityType);
      ksort($entities);
    }

    $synchronize_set = array();
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_fields');
    // Retrieve form LS.
    if ($this->ls_entity_type == 'response') {
      $ls_ids = array();
    }
    elseif ($this->ls_entity_type == 'survey') {
      module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
      $all_sids = limesurvey_sync_survey_properties('all');
      $ls_ids = ($all_sids) ? $all_sids : array();
    }
    else {
      if ($entity == 'all_skip_import' && $ls_key) {
        $selsected_ls_ids = array();
        foreach ($entities as $record) {
          $selsected_ls_ids[] = $record->$ls_key;
        }
        if (!empty($selsected_ls_ids)) {
          $this_ls_id = $selsected_ls_ids;
        }
      }
      $ls_rows = limesurvey_sync_ls_api_retrieve_ls_row($this->ls_entity_type, $this->ls_sid, $this_ls_id);
      $ls_ids = ($ls_rows) ? $ls_rows : array();
    }
    $synchronize_set['need_import'] = $ls_ids;

    foreach ($entities as $record) {
      $dp_key_value = ($this_dp_id != 'this') ? $record->$dp_key : 'this';
      // Break the association object. Work on array.
      $dp_row = (array) $record;
      $ls_row_converted = array(
        'lss_status' => $this->lss_status($record),
        'lss_sync' => $this->lss_sync($record, $ls_ids),
      );
      // Don't change the values for LS fields if not synchronized.
      if ($ls_row_converted['lss_sync'] == LIMESURVEY_SYNC_SYNCHRONIZED) {
        // This dp entry does exists in both dp and ls : an update may be needed.
        // On the survey and response entity, only sync and status are concerned.
        if (!in_array($this->ls_entity_type, array('survey', 'response'))) {
          $ls_row = $ls_ids[$record->$ls_key];
          // Convert LS fields to drupal.
          $ls_row_converted = array_merge($ls_row_converted, limesurvey_sync_ls_api_fields_convert_values($ls_row, $this->ls_entity_type, $this->ls_sid, 'ls2dp'));
          // And update status values.
          $ls_row_converted['lss_status'] = $this->lss_status($ls_row_converted);
        }
        // Delete entries that have been handled.
        if ($ls_key) {
          unset($synchronize_set['need_import'][$record->$ls_key]);
        }
      }
      // @TODO : for dp2ls on all entities, look for desynchronized entities,
      // proposing to recreate them on LS.
      // Compare both.
      if ($way == 'ls2dp') {
        $diff = array_diff_assoc($ls_row_converted, $dp_row);
      }
      else {
        $ls_fields = array_flip(limesurvey_sync_ls_api_retrieve_ls_fields($entity->ls_entity_type, $entity->ls_sid));
        // Select into $tochange the LS fields.
        $dp_row_ls_fields = array_intersect_key($dp_row, $ls_fields);
        $diff = array_diff_assoc($dp_row_ls_fields, $ls_row_converted);
      }
      if (!empty($diff)) {
        // The entity need to be resync and save !
        $dp_row_sync = $dp_row;
        foreach ($diff as $field => $value) {
          $dp_row_sync[$field] = $value;
        }
        if (isset($this->entityInfo['entity keys']['revision'])) {
          $dp_row_sync['log'] = format_plural(count($diff), 'Change on the field %fields', 'Change on the fields : %fields', array(
            '%fields' => implode(', ', array_keys($diff)),
          ));
          $dp_row_sync['revision'] = TRUE;
        }

        // Save the result into an object. Use entity_save() on it to save it.
        $synchronize_set['need_sync'][$dp_key_value]['dp'] = (object) $dp_row;
        $synchronize_set['need_sync'][$dp_key_value]['diff'] = $diff;
        // Save Dates, it may have an interest for saving the most recent one.
        if (!empty($this->entityInfo['entity keys']['ls date key']) && !empty($dp_row_sync['changed'])) {
          // No date for survey.
          $synchronize_set['need_sync'][$dp_key_value]['dp_date'] = $dp_row_sync['changed'];
          $synchronize_set['need_sync'][$dp_key_value]['ls_date'] = $ls_row_converted[$this->entityInfo['entity keys']['ls date key']];
        }
        // if it match the entity requested : update it with the new values.
        if ($entity != 'all' && $this_dp_id && $this_dp_id == $dp_key_value) {
          // $entity is provided by reference. update it with a flag.
          $entity->{"need_save_$way"} = TRUE;
        }
      }
      else {
        // If no change, do not store them for performance raisons.
        $synchronize_set['ok'][$dp_key_value] = TRUE;
        // if it match the entity requested : set a no sync flag..
        if ($entity != 'all' && $this_dp_id && $this_dp_id == $dp_key_value) {
          // $entity is provided by reference. Update it.
          $entity->{"need_save_$way"} = FALSE;
        }
      }
    }

    // All entities from Drupal have been explored. Look for remaining entities
    // to import from LS.
    // Don't try to import survey : it only manually.
    if ($way == 'ls2dp' && !empty($synchronize_set['need_import']) && $this->ls_entity_type != 'survey') {
      foreach ($synchronize_set['need_import'] as $ls_id => $ls_row) {
        $ls_row_converted = limesurvey_sync_ls_api_fields_convert_values($ls_row, $this->ls_entity_type, $this->ls_sid, 'ls2dp');
        $ls_row_converted['lss_sync'] = LIMESURVEY_SYNC_SYNCHRONIZED;
        $ls_row_converted['lss_status'] = $this->lss_status($ls_row_converted);
        $entity_to_import = entity_create($this->entityType, $ls_row_converted);
        // Add a flag to not synchronize twice from the save() method.
        $entity_to_import->need_save_ls2dp = TRUE;
        // Remplace into the synchronize_set et LS array by the new entity
        // object. It can be created using entity_save().
        $synchronize_set['need_import'][$ls_id] = $entity_to_import;
        if ($entity != 'all' && $ls_id == $this_ls_id) {
          // $entity is provided by reference. update it with a flag.
          //$entity = $synchronize_set['need_import'][$ls_id];
          $entity->need_save_ls2dp = TRUE;
        }
      }
    }

    if (empty($synchronize_set['need_import'])) {
      unset($synchronize_set['need_import']);
    }

    return $synchronize_set;
  }

  /**
   * Builds a structured array representing the entity's content.
   *
   * The content built for the entity will vary depending on the $view_mode
   * parameter.
   *
   * @param $entity
   *   An entity object.
   * @param $view_mode
   *   View mode, e.g. 'full', 'teaser'...
   * @param $langcode
   *   (optional) A language code to use for rendering. Defaults to the global
   *   content language of the current request.
   *
   * @return
   *   The renderable array.
   */
  public function buildContent($entity, $view_mode = 'full', $langcode = NULL, $content = array()) {

    // Our additions to the $build render array
    // $wrapper = entity_metadata_wrapper($this->entityType, $entity);
    $build = parent::buildContent($entity, $view_mode, $langcode, $content);

    return $build;
  }
}

/**
 * Default controller for providing UI.
 *
 * Extend the common class 'EntityDefaultUIController' for entities.
 */
class LimesurveySyncUIController extends EntityDefaultUIController {
  // Edit the menu from EntityDefaultUIController::hook_menu().
  public $ls_entity_type = '';
  public $ls_sid = '';
  protected $menu_weight = 0;
  protected $ls_sub_entity = FALSE;
  protected $hide_export_status = TRUE;

  // Config : For now I disallow to edit and create answer and token.
  // I may change my mind on this later.
  protected $disallow_add = FALSE;
  protected $disallow_edit = FALSE;
  // response overview and add link are into the 'content' root, other are into
  // 'admin' root.
  protected $move_to_content_tab = FALSE;

  /**
   * Overridden.
   *
   * Retreive the LS survey ID from the entity type name, making sure to find it
   * for answers and token entities.
   */
  public function __construct($entity_type, $entity_info) {

    $with_suffix = array('token', 'answer');
    $ls_sid = limesurvey_sync_api_retrieve_ls_sid_from_entityname($entity_type, $this->ls_entity_type);
    if (empty($ls_sid) && in_array($this->ls_entity_type, $with_suffix)) {
      throw new Exception('Cannot create an instance of ' . $this->ls_entity_type . ' entity without a specified survey id.');
    }
    else {
      $this->ls_sid = $ls_sid;
    }
    parent::__construct($entity_type, $entity_info);
  }

  /**
   * Provides definitions for implementing hook_menu().
   *
   * Manage entity view page, adding, synchronize (one and all) pages, revisions,
   * import, clone, devel pages.
   */
  public function hook_menu() {

    // In order to works with the menu router, the modules must be called in
    // this order : entity, limesurvey_sync, limesurvey_sync_survey, the answer
    // and token, depending on how the hook_entity_info() functions are called.
    $items = parent::hook_menu();

    // Edit result from EntityDefaultUIController::hook_menu().
    // In order to set survey view tab as a tree with manage fields : use the wildcard.
    $survey_machine_name = limesurvey_sync_api_typename('survey', $this->ls_sid);
    $wildcard = isset($this->entityInfo['admin ui']['menu wildcard']) ? $this->entityInfo['admin ui']['menu wildcard'] : '%entity_object';
    $path_admin = $this->path;

    // Fix the add page : from the admin menu to general menu; use
    // hook_menu_local_tasks_alter() to add it to admin menu and
    // hook_admin_paths() pour admin theme.
    // The add page for response, is handled into the response page.

    // Set the entity view page.
    $path_view = limesurvey_sync_api_root_uri($this->ls_entity_type, $this->ls_sid, FALSE) . "/{$wildcard}";
    $id_count = count(explode('/', $path_view));
    $placeholder_view = $id_count - 1;

    // Set a synchronization page for all entities.
    $items[$path_admin . '/synchronize-all'] = array(
      'title' => 'Synchronize all ' . $this->ls_entity_type,
      'page callback' => 'drupal_get_form',
      'page arguments' => array(
        'limesurvey_sync_api_synchronize_page',
        $this->entityType,
        FALSE,
      ),
      'access arguments' => array("administer limesurvey sync {$this->ls_entity_type}"),
      'weight' => -5,
      'type' => MENU_LOCAL_ACTION,
      'file' => 'limesurvey_sync_api.pages.inc',
      'file path' => drupal_get_path('module', 'limesurvey_sync') . '/api',
    );

    // Set a synchronization page for 1 entity.
    $items[$path_view . '/synchronize'] = array(
      'title' => 'Synchronize',
      // 'title callback' => 'entity_label',
      // 'title arguments' => array($this->entityType, $id_count + 1),
      'page callback' => 'drupal_get_form',
      'page arguments' => array(
        'limesurvey_sync_api_synchronize_page',
        $this->entityType,
        $placeholder_view,
      ),
      'load arguments' => array($this->entityType),
      'access callback' => 'entity_access',
      'access arguments' => array(
        'update',
        $this->entityType,
        $placeholder_view,
      ),
      'type' => MENU_LOCAL_TASK,
      'file' => 'limesurvey_sync_api.pages.inc',
      'file path' => drupal_get_path('module', 'limesurvey_sync') . '/api',
    );

    // Hide "/import" into the menu.
    if (isset($items["{$this->path}/import"])) {
      // @TODO : it is still present in the menu...
      $items["{$this->path}/import"]['type'] = MENU_CALLBACK;
    }

    // The survey view is handled into the survey class.
    if ($this->ls_entity_type == 'survey') {
      return $items;
    }


    // The add page for response, is handled into the response page.
    //$path_add = limesurvey_sync_api_root_uri($this->ls_entity_type, $this->ls_sid, FALSE) . "/add";
    $path_add = $this->path . '/add';
    if (isset($items[$path_add]) && $this->disallow_add) {
      // @TODO : replace survey type by wildcard for sub entities.
      unset($items[$path_add]);
    }



    if (!isset($items[$path_view])) {
      $items[$path_view] = array(
        'title callback' => 'entity_label',
        'title arguments' => array(
          $this->entityType,
          $placeholder_view,
        ),
        'access callback' => 'entity_access',
        'access arguments' => array(
          'view',
          $this->entityType,
          $placeholder_view,
        ),
        'page callback' => 'limesurvey_sync_api_view_page',
        'page arguments' => array(
          $this->entityType,
          $placeholder_view,
        ),
        'load arguments' => array($this->entityType),
        'description' => 'View LimeSurvey ' . $this->entityInfo['label'] . ' page',
        'type' => MENU_CALLBACK,
      );

      $items["$path_view/view"] = array(
        'title' => 'view', // ($this->ls_entity_type == 'survey') ? $survey->label : 'View',
        'weight' => -10,
        'type' => MENU_DEFAULT_LOCAL_TASK,
      );
    }

    // Set token and answer overview page as local tab from the corresponding survey.
    if ($this->ls_sub_entity) {
      // 'admin/structure/limesurvey/surveys/manage/%limesurvey_sync_survey/ls_token_79447'

      if (isset($items[$path_admin])) {

        // Set them as local tab.
        $items[$path_admin]['type'] = MENU_LOCAL_TASK; //MENU_NORMAL_ITEM | MENU_LOCAL_TASK;
        $items[$path_admin]['weight'] = $this->menu_weight;
        // Display only the survey or token table corresponding to the survey.
        $survey_get_info = entity_get_info('limesurvey_sync_survey');
        $path_root_survey = $survey_get_info['admin ui']['path']; // limesurvey_sync_api_root_uri('survey');
        $survey_placeholder_pos = count(explode('/', $path_root_survey)) + 1;
        $items[$path_admin]['access callback'] = 'limesurvey_sync_api_subentity_tab_access';
        $items[$path_admin]['access arguments'] = array('update', $this->entityType, $survey_placeholder_pos);
      }
      // fix import.
      if (isset($items[$this->path . '/import'])) {
        $items[$this->path . '/import']['access arguments'][] = $survey_placeholder_pos;
      }
    }


    // Move edit, clone, delete, export from admin uri to view uri.
    $old_path = $this->path . '/manage/' . $wildcard;
    $new_path = $path_view;
    if (isset($items[$old_path])) { //Edit
      $items[$new_path . '/edit'] = $items[$old_path];
      // Fix wildcard place.
      $page_arguments = $items[$new_path . '/edit']['page arguments'];
      $page_arguments[1] = $placeholder_view;
      $items[$new_path . '/edit']['page arguments'] = $page_arguments;

      $access_arguments = $items[$new_path . '/edit']['access arguments'];
      $access_arguments[2] = $placeholder_view;
      $items[$new_path . '/edit']['access arguments'] = $access_arguments;

      $items[$new_path . '/edit']['type'] = MENU_LOCAL_TASK;
      unset($items[$new_path . '/edit']['title arguments'], $items[$new_path . '/edit']['title callback']);
      unset($items[$old_path], $items[$old_path . '/edit']);
      // Delete the edit path, depending of the options.
      if ($this->ls_sub_entity && $this->disallow_edit) {
        unset($items[$new_path . '/edit']);
      }
    }
    if (isset($items[$old_path . '/clone'])) { // clone
      $items[$new_path . '/clone'] = $items[$old_path . '/clone'];
      // Fix wildcard place.
      $page_arguments = $items[$new_path . '/clone']['page arguments'];
      $page_arguments[1] = $placeholder_view;
      $items[$new_path . '/clone']['page arguments'] = $page_arguments;
      unset($items[$old_path . '/clone']);
      // Cloning a entry may be a problem for linking drupal and limesurvey.
      // Disalow it !
      unset($items[$new_path . '/clone']);
    }
    if (isset($items[$old_path . '/%'])) { // revert, delete.
      $items[$new_path . '/%'] = $items[$old_path . '/%'];
      // Fix wildcard place.
      $page_arguments = $items[$new_path . '/%']['page arguments'];
      $page_arguments[2] = $placeholder_view;
      $page_arguments[3] = $placeholder_view + 1;
      $items[$new_path . '/%']['page arguments'] = $page_arguments;
      unset($items[$old_path . '/%']);
    }

    // Handle revision stuff.
    if (!empty($this->entityInfo['entity keys']['revision'])) {
      $items[$path_view . '/revisions'] = array(
        'title' => 'Revisions',
        'page callback' => 'limesurvey_sync_api_revision_overview',
        'page arguments' => array(
          $this->entityType,
          $placeholder_view,
        ),
        'load arguments' => array($this->entityType),
        'access callback' => '_limesurvey_sync_api_revision_access',
        'access arguments' => array(
          $this->entityType,
          $placeholder_view,
        ),
        'weight' => 2,
        'type' => MENU_LOCAL_TASK,
        'file path' => drupal_get_path('module', 'limesurvey_sync') . '/api/',
        'file' => 'limesurvey_sync_api.pages.inc',
      );

      // Too bad the %entity_revision wildcard doesn't work. It may be because
      // of the previous %entity_object wildcard. I manage it in the
      // _limesurvey_sync_api_revision_access() function.
      $items[$path_view . '/revisions/%/view'] = array(
        'title' => 'Revisions',
        'load arguments' => array(
          $this->entityType,
          $placeholder_view + 2,
        ),
        'page callback' => 'limesurvey_sync_api_show',
        'page arguments' => array(
          $this->entityType,
          $placeholder_view + 2,
          TRUE,
        ),
        'access callback' => '_limesurvey_sync_api_revision_access',
        'access arguments' => array(
          $this->entityType,
          $placeholder_view + 2,
        ),
      );
      $items[$path_view . '/revisions/%/revert'] = array(
        'title' => 'Revert to earlier revision',
        'load arguments' => array(
          $this->entityType,
          $placeholder_view + 2,
        ),
        'page callback' => 'drupal_get_form',
        'page arguments' => array(
          'limesurvey_sync_api_revision_revert_confirm',
          $this->entityType,
          $placeholder_view + 2,
        ),
        'access callback' => '_limesurvey_sync_api_revision_access',
        'access arguments' => array(
          $this->entityType,
          $placeholder_view + 2,
          'update',
        ),
        'file path' => drupal_get_path('module', 'limesurvey_sync') . '/api/',
        'file' => 'limesurvey_sync_api.pages.inc',
      );
      $items[$path_view . '/revisions/%/delete'] = array(
        'title' => 'Delete earlier revision',
        'load arguments' => array(
          $this->entityType,
          $placeholder_view + 2,
        ),
        'page callback' => 'drupal_get_form',
        'page arguments' => array(
          'limesurvey_sync_api_revision_delete_confirm',
          $this->entityType,
          $placeholder_view + 2,
        ),
        'access callback' => '_limesurvey_sync_api_revision_access',
        'access arguments' => array(
          $this->entityType,
          $placeholder_view + 2,
          'delete',
        ),
        'file path' => drupal_get_path('module', 'limesurvey_sync') . '/api/',
        'file' => 'limesurvey_sync_api.pages.inc',
      );
    }

    // Set devel module with wildcard.
    if (module_exists('devel')) {
      $devel_path = drupal_get_path('module', 'devel');
      $items[$path_view . '/devel'] = array(
        'title' => 'Devel',
        'page callback' => 'devel_load_object',
        'page arguments' => array(
          $this->entityType,
          $placeholder_view,
        ),
        'access arguments' => array('access devel information'),
        'load arguments' => array($this->entityType),
        'type' => MENU_LOCAL_TASK,
        'file' => 'devel.pages.inc',
        'weight' => 100,
        'file path' => $devel_path,
      );
      $items[$path_view . '/devel/load'] = array(
        'title' => 'Load',
        'type' => MENU_DEFAULT_LOCAL_TASK,
      );
      // @TODO : render page fails because it calls limesurvey_sync_token_view()
      // function that not exists (fata error).
      $items[$path_view . '/devel/render'] = array(
        'title' => 'Render',
        'page callback' => 'devel_render_object',
        'page arguments' => array(
          // For answer and token, provide 'limesurvey_sync_token' instead of
          // 'limesurvey_sync_token_xxxxxx' for calling limesurvey_sync_token_view().
          rtrim(limesurvey_sync_api_entityname($this->ls_entity_type, 'prefix'), '_'),
          $placeholder_view,
          $this->entityType,
        ),
        'access arguments' => array('access devel information'),
        'load arguments' => array($this->entityType),
        'file' => 'devel.pages.inc',
        'type' => MENU_LOCAL_TASK,
        'file path' => $devel_path,
        'weight' => 100,
      );
    }
    return $items;
  }

  /**
   * Provides definitions for implementing hook_forms().
   *
   * Use per bundle form ids if possible, such that easy per bundle alterations
   * are supported too.
   *
   * Manage editing form for answers and tokens entities. Note that for
   * performance reasons, this method is only invoked for forms, which receive
   * the entity_type as first argument. Thus any forms added, must follow that
   * pattern.
   *
   * @see entity_forms()
   */
  public function hook_forms() {

    $forms = parent::hook_forms();
    if ($this->ls_sub_entity) {
      foreach (limesurvey_sync_survey_get_surveys() as $type) {
        $entity_type = limesurvey_sync_api_entityname($this->ls_entity_type, $type->ls_sid);
        $form_id_value = $entity_type . '_form';
        $forms[$form_id_value] = array(
          'callback' => "limesurvey_sync_{$this->ls_entity_type}_form",
          'wrapper_callback' => 'entity_ui_form_defaults',
        );
      }
    }
    return $forms;
  }

  /**
   * Generates the table headers for the overview table.
   */
  protected function overviewTableHeaders($conditions, $rows, $additional_header = array()) {

    $header = $additional_header;
    $header[] = t('synchronization status');
    $header[] = t('LS status');

    array_unshift($header, t('Label'));
    if (!empty($this->entityInfo['exportable']) && !$this->hide_export_status) {
      $header[] = t('Status');
    }
    // Add operations with the right colspan.
    $header[] = array(
      'data' => t('Operations'),
      'colspan' => $this->operationCount(),
    );
    return $header;
  }

  /**
   * Generates the row for the passed entity and may be overridden in order to
   * customize the rows.
   *
   * Add the synchronization status and LimeSurvey status colomns and the
   * synchronize operation. Delete export status colomn and edit operation
   * depending on settings ($hide_export_status and $disallow_edit).
   *
   * @param $additional_cols
   *   Additional columns to be added after the entity label column.
   */
  protected function overviewTableRow($conditions, $id, $entity, $additional_cols = array()) {

    // Disable stuff.
    //return parent::overviewTableRow($conditions, $id, $entity, $additional_cols);
    // Rewrite te whole method, since all links changes.
    static $synchronize_set = array();
    if (empty($synchronize_set)) {
      // HERE : retrieve the synchronization set. Call it only one time.
      $entity_type = $this->entityType;
      $all = 'all_skip_import'; //'all'
      $synchronize_set = entity_get_controller($entity_type)->ls_synchronize($all);
    }
    $dp_key = $this->entityInfo['entity keys']['id'];
    $css = array(
      'css' => array(
        drupal_get_path('module', 'limesurvey_sync') . '/misc/limesurvey_sync.css',
      ),
    );

    $entity_uri = entity_uri($this->entityType, $entity); //($this->ls_sub_entity) ? entity_uri($this->entityType, $entity) : array('path' => limesurvey_sync_api_root_uri($this->ls_entity_type, $entity->ls_sid = FALSE, TRUE) . '/' . $entity->type, 'options' => array());

    $row[] = array(
      'data' => array(
        '#theme' => 'entity_ui_overview_item',
        '#label' => entity_label($this->entityType, $entity),
        '#name' => !empty($this->entityInfo['exportable']) ? entity_id($this->entityType, $entity) : FALSE,
        '#url' => $entity_uri ? $entity_uri : FALSE,
        '#entity_type' => $this->entityType,
      ),
    );

    $sync_entity = $entity;
    $need_sync = FALSE;
    $need_sync_entity = (!empty($synchronize_set['need_sync']) && array_key_exists($entity->$dp_key, $synchronize_set['need_sync']));
    $need_sync_ref = (!empty($synchronize_set['need_sync_ref']) && array_key_exists($entity->$dp_key, $synchronize_set['need_sync_ref']));
    if ($need_sync_entity || $need_sync_ref) {
      $need_sync = TRUE;
      // Build the synchronized entity.
      if ($need_sync_entity) {
        $sync_entity = $synchronize_set['need_sync'][$entity->$dp_key]['dp'];
        foreach ($synchronize_set['need_sync'][$entity->$dp_key]['diff'] as $key => $value) {
          $sync_entity->$key = $value;
        }
      }
      else {
        $sync_entity = $synchronize_set['need_sync_ref'][$entity->$dp_key]['dp'];
      }
    }
    // lss_sync col. Display the sync status or 'To synchronize' if sync needed.
    $additional_cols[] = array(
      'data' => array(
        '#theme' => 'limesurvey_sync_field_label',
        '#field' => 'lss_sync',
        '#to_display' => $sync_entity->lss_sync,
        '#ls_entity_type' => $sync_entity->ls_entity_type,
        '#ls_sid' => $sync_entity->ls_sid,
        '#need_sync' => $need_sync,
        '#attached' => $css,
      ),
    );
    // lss_status_col.
    $additional_cols[] = array(
      'data' => array(
        '#theme' => 'limesurvey_sync_field_label',
        '#field' => 'lss_status',
        '#to_display' => $sync_entity->lss_status,
        '#ls_entity_type' => $sync_entity->ls_entity_type,
        '#ls_sid' => $sync_entity->ls_sid,
        '#attached' => $css,
      ),
    );

    // Add in any passed additional cols.
    foreach ($additional_cols as $col) {
      $row[] = $col;
    }

    // Add a row for the exportable status.
    if (!empty($this->entityInfo['exportable']) && !$this->hide_export_status) {
      $row[] = array('data' => array(
          '#theme' => 'entity_status',
          '#status' => $entity->{$this->statusKey},
        ));
    }
    // In case this is a bundle, we add links to the field ui tabs.
    $field_ui = !empty($this->entityInfo['bundle of']) && entity_type_is_fieldable($this->entityInfo['bundle of']) && module_exists('field_ui');
    // For exportable entities we add an export link.
    $exportable = !empty($this->entityInfo['exportable']);
    // If i18n integration is enabled, add a link to the translate tab.
    $i18n = !empty($this->entityInfo['i18n controller class']);

    // Set a view path instead of admin path:
    $path_view = limesurvey_sync_api_root_uri($this->ls_entity_type, $entity->ls_sid, FALSE);

    // Add operations depending on the status.
    if (entity_has_status($this->entityType, $entity, ENTITY_FIXED)) {
      $row[] = array(
        'data' => l(t('clone'), $path_view . '/' . $id . '/clone'),
        'colspan' => $this->operationCount(),
      );
    }
    else {
      // Delete edit link.
      if (!$this->disallow_edit) {
        $row[] = l(t('edit'), $path_view . '/' . $id . '/edit');
      }
      if ($field_ui) {
        $row[] = l(t('manage fields'), $this->path . '/manage/' . $id . '/fields');
        $row[] = l(t('manage display'), $this->path . '/manage/' . $id . '/display');
      }
      if ($i18n) {
        $row[] = l(t('translate'), $path_view . '/' . $id . '/translate');
      }
      // Cloning a entry may be a problem for linking drupal and limesurvey...
      if ($exportable) {
        $row[] = l(t('clone'), $path_view . '/' . $id . '/clone');
      }

      if (empty($this->entityInfo['exportable']) || !entity_has_status($this->entityType, $entity, ENTITY_IN_CODE)) {
        $row[] = l(t('delete'), $path_view . '/' . $id . '/delete', array('query' => drupal_get_destination()));
      }
      elseif (entity_has_status($this->entityType, $entity, ENTITY_OVERRIDDEN)) {
        $row[] = l(t('revert'), $path_view . '/' . $id . '/revert', array('query' => drupal_get_destination()));
      }
      else {
        $row[] = '';
      }
    }
    if ($exportable) {
      $row[] = l(t('export'), $path_view . '/' . $id . '/export');
    }

    // Synchronize link col : it will be included into the operation header.
    $row[] = l(t('synchronize'), $path_view . '/' . $id . '/synchronize');
    return $row;
  }

  /**
   * Returns the operation count for calculating colspans.
   */
  protected function operationCount() {

    $count = parent::operationCount();
    $count += 1; // The synchronize operation.
    if ($this->disallow_edit) {
      $count -= 1; // The deleted edit link.
    }

    return $count;
  }
}

/**
 * Sets the property to the given value. May be used as 'setter callback'.
 */
function limesurvey_sync_api_setter_callback($data, array $options, $name, $type, $info) {
  return entity_property_verbatim_set($data, $options, $name, $type, $info);
}


/**
 * return entity_view result, if the entity is provided as an object either as
 * primary numeric key.
 */
function limesurvey_sync_api_view_page($entity_type, $entity, $view_mode = 'full', $langcode = NULL, $page = NULL) {

  // This function exists because the entity object must be keyed.
  // Furthermore, the survey is provided as its identifier instead of the full
  // object, because I don't want to use wildcard to preserve the menu tree.
  // @TODO : I may not need this function anymore.
  $entities = array($entity);
  if (!is_object($entity) && !empty($entity)) {
    $entities = entity_load($entity_type, $entities);
  }
  return entity_view($entity_type, $entities, $view_mode, $langcode, $page);
}

/**
 * Determines whether the given user has access to an entity.
 *
 * @param $op
 *   The operation being performed. One of 'view', 'update', 'create' or
 *   'delete'.
 * @param $entity_type
 *   The entity type of the entity to check for.
 * @param $entity
 *   Optionally an entity to check access for. If no entity is given, it will be
 *   determined whether access is allowed for all entities of the given type.
 * @param $account
 *   The user to check for. Leave it to NULL to check for the global user.
 *
 * @return boolean
 *   Whether access is allowed or not. If the entity type does not specify any
 *   access information, NULL is returned.
 *
 * @see entity_type_supports()
 */
function limesurvey_sync_api_entity_access($op, $entity = NULL, $account = NULL, $entity_type = NULL) {
  // Map the op value. EntityDefaultUIController:hook_menu() use 'add' and
  // 'update'. entity_access() will work even if 'create' or 'edit' are provided.
  switch ($op) {
    case 'edit':
      $op = 'update';
      break;
    case 'create':
      $op = 'add';
      break;
  }

  // Now op and entity_type are set : check them.
  // First retreive the ls_entity_type from the entity type name, as $entity may be null.
  if ($entity_type && !($ls_entity_type = limesurvey_sync_api_retrieve_ls_entity_type_from_entityname($entity_type))) {
    // In the menu tree, make sure the answers and token entities are displayed
    // into the same menu tree that the corresponding survey.
    return FALSE;
  }

  // @TODO: add 'synchronize' op.
  if ((empty($entity) && empty($entity_type)) || !in_array($op, array('view', 'update', 'delete', 'add'), TRUE)) {
    // If there was no entity to check against, or the $op was not one of the
    // supported ones, we return access denied.
    return FALSE;
  }

  // If no user object is supplied, the access check is for the current user.
  if (empty($account)) {
    $account = $GLOBALS['user'];
  }

  // Check for high level permission.
  if (user_access("administer limesurvey sync {$ls_entity_type}", $account)) {
    return TRUE;
  }

  if ($ls_entity_type == 'response' && $op == 'add' && (empty($entity) || empty($entity->ls_sid))) {
    // This situation should not exist : for checking response creation
    // permission, the $entity must be provided as an object. That why I create
    // the _limesurvey_sync_response_add_access() function.
    return FALSE;
  }

  // Prepare the user_access() string.
  $no_bundles = (in_array($ls_entity_type, array('survey', 'participant')));
  if (!$no_bundles) {
    // ls_sid needed.
    $ls_sid = NULL;
    $ls_sid_from_entitytype = limesurvey_sync_api_retrieve_ls_sid_from_entityname($entity_type, $ls_entity_type);
    if (!empty($entity->ls_sid)) {
      $ls_sid = $entity->ls_sid;
    }
    elseif ($ls_sid_from_entitytype) {
      $ls_sid = $ls_sid_from_entitytype;
    }
    else {
      $entity = ($ls_entity_type != 'response') ? entity_create($entity_type, array()) : NULL;
      if (!empty($entity->ls_sid)) {
        $ls_sid = $entity->ls_sid;
      }
    }

    if ($ls_sid) {
      $survey_type = check_plain(limesurvey_sync_api_typename('survey', $ls_sid));
      if ($survey_type) {
        $perm_machine_suffix = "{$survey_type} {$ls_entity_type}";
      }
      else {
        return FALSE;
      }
    }
    else {
      return FALSE;
    }
  }
  else {
    $perm_machine_suffix = $ls_entity_type;
  }

  // Handle operations.
  // Create entity.
  if ($op == 'add') {
    if (user_access("$op {$perm_machine_suffix}", $account)) {
      return TRUE;
    }
    return FALSE;
  }
  // View, delete, update entity.
  if (user_access("$op any {$perm_machine_suffix}", $account)) {
    return TRUE;
  }
  if (user_access("$op own {$perm_machine_suffix}", $account) && isset($entity->uid) && $entity->uid == $account->uid) {
    return TRUE;
  }

  // At last.
  return FALSE;
}

/**
 * Determines whether the given user has access to an entity.
 *
 * @param $op
 *   The operation being performed. One of 'view', 'update', 'create' or
 *   'delete'.
 * @param $entity_type
 *   The entity type of the entity to check for.
 * @param $entity
 *   Optionally an entity to check access for. If no entity is given, it will be
 *   determined whether access is allowed for all entities of the given type.
 * @param $account
 *   The user to check for. Leave it to NULL to check for the global user.
 *
 * @return boolean
 *   Whether access is allowed or not. If the entity type does not specify any
 *   access information, NULL is returned.
 *
 * @see entity_type_supports()
 */
function limesurvey_sync_api_subentity_tab_access($op, $entity_type, $survey, $entity = NULL, $account = NULL ) {
  if (!is_object($survey)) {
    return FALSE;
  }
  // Check if the survey and the entity type match.
  $sub_entity_ls_entity_type = limesurvey_sync_api_retrieve_ls_entity_type_from_entityname($entity_type);
  $sub_entity_ls_sid = limesurvey_sync_api_retrieve_ls_sid_from_entityname($entity_type, $sub_entity_ls_entity_type);
  if ($sub_entity_ls_sid != $survey->ls_sid) {
    // In the menu tree, make sure the answers and token entities are displayed
    // into the same menu tree that the corresponding survey.
    return FALSE;
  }
  else {
    return entity_access($op, $entity_type, $entity, $account);
  }
}

/**
 * Access callback: Checks entity revision access.
 *
 * Inspired from _node_revision_access().
 *
 * @param $entity_type
 *   The entity type of the entity to check for.
 * @param $entity
 *   Optionally an entity to check access for. If no entity is given, it will be
 *   determined whether access is allowed for all entities of the given type.
 * @param $op
 *   (optional) The specific operation being checked. Defaults to 'view.'
 * @param object $account
 *   (optional) A user object representing the user for whom the operation is
 *   to be performed. Determines access for a user other than the current user.
 *
 * @return
 *   TRUE if the operation may be performed, FALSE otherwise.
 */
function _limesurvey_sync_api_revision_access($entity_type, $entity = NULL, $op = 'view', $account = NULL) {

  if (is_numeric($entity)) {
    $entity = entity_revision_load($entity_type, $entity);
  }

  if (!($ls_entity_type = limesurvey_sync_api_retrieve_ls_entity_type_from_entityname($entity_type))) {
    return FALSE;
  }
  if (!($ls_sid = limesurvey_sync_api_retrieve_ls_sid_from_entityname($entity_type, $ls_entity_type))) {
    return FALSE;
  }

  // Find survey.
  $survey_type = limesurvey_sync_api_typename('survey', $ls_sid);
  if (!($survey = limesurvey_sync_survey_get_surveys($survey_type))) {
    return FALSE;
  }

  $access = &drupal_static(__FUNCTION__, array());

  $map = array(
    'view' => "view {$survey_type} {$ls_entity_type} revisions",
    'update' => "revert {$survey_type} {$ls_entity_type} revisions",
    'delete' => "delete {$survey_type} {$ls_entity_type} revisions",
  );

  if (!$entity || !isset($map[$op])) {
    // If there was no entity to check against, or the $op was not one of the
    // supported ones, we return access denied.
    return FALSE;
  }

  if (!isset($account)) {
    $account = $GLOBALS['user'];
  }

  // Statically cache access by revision ID, user account ID, and operation.
  $cid = $entity->vid . ':' . $account->uid . ':' . $op;

  if (!isset($access[$cid])) {
    // Perform basic permission checks first.
    if (!user_access($map[$op], $account) && !user_access("administer limesurvey sync {$ls_entity_type}", $account)) {
      return $access[$cid] = FALSE;
    }

    $entity_info = entity_get_info($entity_type);
    $id_field = $entity_info['entity keys']['id'];
    $entity_current_revisions = entity_load($entity_type, array($entity->$id_field));
    $entity_current_revision = $entity_current_revisions[$entity->$id_field];
    $is_current_revision = entity_revision_is_default($entity_type, $entity);

    // There should be at least two revisions. If the vid of the given entity and
    // the vid of the current revision differ, then we already have two
    // different revisions so there is no need for a separate database check.
    // Also, if you try to revert to or delete the current revision, that's not
    // good.

    if ($is_current_revision && (db_query('SELECT COUNT(' . $entity_info['entity keys']['revision'] . ') FROM {' . $entity_info['revision table'] . '} WHERE ' . $id_field . ' = :' . $id_field, array(
      ":{$id_field}" => $entity->$id_field,
    ))->fetchField() == 1 || $op == 'update' || $op == 'delete')) {
      $access[$cid] = FALSE;
    }
    elseif (user_access("administer limesurvey sync {$ls_entity_type}", $account)) {
      $access[$cid] = TRUE;
    }
    else {
      // First check the access to the current revision and finally, if the entity
      // passed in is not the current revision then access to that, too.
      $access[$cid] = entity_access($op, $entity_type, $entity_current_revision, $account) && ($is_current_revision || entity_access($op, $entity_type, $entity, $account));
    }

    return $access[$cid];
  }
}

/**
 * Generates an array which displays a node detail page.
 *
 * Inspired from node_show().
 *
 * @param $entity_type
 *   The entity type of the entity to check for.
 * @param $entity
 *   Optionally an entity to check access for. If no entity is given, it will be
 *   determined whether access is allowed for all entities of the given type.
 * @param $message
 *   A flag which sets a page title relevant to the revision being viewed.
 *
 * @return
 *   A $page element suitable for use by drupal_render().
 */
function limesurvey_sync_api_show($entity_type, $entity, $message = FALSE) {

  if (is_numeric($entity)) {
    $entity = entity_revision_load($entity_type, $entity);
  }

  $entity_info = entity_get_info($entity_type);
  $id_field = $entity_info['entity keys']['id'];
  if ($message) {
    drupal_set_title(t('Revision of %title from %date', array(
      '%title' => $entity->title,
      '%date' => format_date($entity->timestamp),
    )), PASS_THROUGH);
  }

  // For markup consistency with other pages, use node_view_multiple() rather than node_view().
  $entities = entity_view($entity_type, array(
    $entity->$id_field => $entity,
  ), 'full');

  // Update the history table, stating that this user viewed this entity.
  // entity_tag_new($entity);

  return $entities;
}

/**
 * Access check for multiple permissions, return TRUE if at least one is TRUE.
 *
 * If return FALSE, the user will not have access to any of the LimeSurvey sync
 * administration menus ! It corresponds to the 'admin/structure/limesurvey'
 * menu root.
 *
 * @param $list
 *   An array of permissions to pass to the user_access() function. If empty,
 *   check all administration permissions on the LimeSurvey_sync module. Default
 *   is empty.
 * @param $account
 *   A user object. If NULL the connected user is used. Default is NULL.
 *
 * @return
 *   A Boolean. TRUE for access.
 */
function limesurvey_sync_api_multiple_access_check($list = array(), $account = NULL) {

  if ($list === array()) {
    // Load all main LimeSurvey module access.
    $list = array(
      'administer limesurvey sync survey',
      'administer limesurvey sync answer',
      'administer limesurvey sync token',
    );
  }
  foreach ($list as $right) {
    if (user_access($right, $account)) {
      return TRUE;
    }
  }
  return FALSE;
}

function limesurvey_sync_api_retreive_response_from_ref_entity($ref_entity) {
  if (empty($ref_entity->ls_sid)) {
    return array();
  }
  $response_entityname = limesurvey_sync_api_entityname('response');
  $ref_entityname = limesurvey_sync_api_entityname($ref_entity->ls_entity_type, $ref_entity->ls_sid);
  $ref_field_name = limesurvey_sync_api_et_fieldname($ref_entity->ls_entity_type, $ref_entity->ls_sid);
  $survey_typename = limesurvey_sync_api_typename('survey', $ref_entity->ls_sid);
  $ref_entity_info = entity_get_info($ref_entityname);
  $dp_key = $ref_entity_info['entity keys']['id'];
  $dp_key_value = $ref_entity->$dp_key;
  $query = new EntityFieldQuery();
  $query->entityCondition('entity_type', $response_entityname)
  ->entityCondition('bundle', $survey_typename)
  ->fieldCondition($ref_field_name, 'target_id', $dp_key_value, '=' )
  //->range(0, 10)
  ->propertyOrderBy('dp_rid', 'DESC')
  ->addMetaData('account', user_load(1)); // Run the query as user 1.

  $result = $query->execute();
  $all_responses = array();
  if (isset($result[$response_entityname])) {
    $all_responses = entity_load($response_entityname, array_keys($result[$response_entityname]));
  }

  return $all_responses;
}

function limesurvey_sync_api_retreive_uid_from_ref_entity($ref_entity) {
  // @TODO try from the participant entity if token persistance enabled.
  $all_responses = limesurvey_sync_api_retreive_response_from_ref_entity($ref_entity);
  $all_uids = array();
  foreach ($all_responses as $dp_rid => $response_entity) {
    $all_uids[$dp_rid] = $response_entity->uid;
  }
  $unique_uids = array_unique($all_uids);
  if (count($unique_uids) == 1) {
    $uid = current($unique_uids);
  }
  elseif (empty($unique_uids)) {
    $uid = 0; // set as anonymous.
  }
  else {
    // Many uids associated to this reference, return an array.
    $uid = array_values($unique_uids);
  }
  return $uid;
}

/**
 * Build root uri for entities.
 *
 * Depending on it is admin or non admin uri and for specific ls_type and ls
 * survey ID. Use root_ui and root_admin_ui provided into the LimesurveySyncInfos
 * class.
 *
 * @param $ls_entity_type
 *   A string, 'response', or 'answer', or 'token', or 'survey'.
 * @param $ls_sid
 *   A numeric, the LS survey ID. Default is NULL.
 * @param $admin
 *   A boolean, TRUE for admin uri. Default is FALSE.
 * @param $real_path
 *   A boolean, if TRUE converting wilcard loader (example :
 *   '%limesurvey_sync_survey') to its corresponding value ('ls-survey-xxxxxx).
 *   Default is FALSE.
 *
 * @return
 *   A string, the corresponding uri (example :
 *   admin/structure/limesurvey/surveys/ls-survey-xxxxxx/ls-answer-xxxxxx).
 */
function limesurvey_sync_api_root_uri($ls_entity_type, $ls_sid = NULL, $admin = FALSE, $real_path = FALSE) {

  $object = limesurvey_sync_api_info_class_object($ls_entity_type);
  $sub_entity = array('answer', 'token');
  $machine_name = limesurvey_sync_api_typename($ls_entity_type, $ls_sid);
  if ($real_path) {
    $machine_name = str_replace('_', '-', $machine_name);
  }
  if (!$admin) {
    $root = (empty($object->root_ui)) ? '' : $object->root_ui . '/';
    // 'limesurvey/survey' ; 'limesurvey/ls_answer_1234'...
    $path = (in_array($ls_entity_type, $sub_entity)) ? "{$root}{$machine_name}" : (($ls_entity_type != 'survey') ? "{$root}{$ls_entity_type}" : $root);
  }
  else {
    // Admin page, for adding fields and display overview page.
    if (!in_array($ls_entity_type, $sub_entity)) {
      $path = $object->root_admin_ui;
    }
    else {
      $root = (empty($object->root_admin_ui)) ? '' : $object->root_admin_ui . '/';
      $survey_name = limesurvey_sync_api_typename('survey', $ls_sid);
      // 'admin/structure/limesurvey/surveys/manage/ls_survey_1234/answer_1234', etc...
      $path = "{$root}manage/{$survey_name}/{$machine_name}";

    }
  }
  // Delete the last '/' if any.
  return rtrim($path, '/');
}

/**
 * Return the LS survey ID from the entity type name.
 *
 * As an example, for 'limsurvey_sync_answer_123456' and 'answer', returns
 * '123456'.
 *
 * @param $entity_type
 *   The entity type of the entity.
 * @param $ls_entity_type
 *   A string, 'response', or 'answer', or 'token', or 'survey'.
 *
 * @return
 *   A numeric, the LS survey ID or FALSE if error or NULL if invalid
 *   parametters.
 */
function limesurvey_sync_api_retrieve_ls_sid_from_entityname($entity_type, $ls_entity_type) {

  $with_suffix = array('token', 'answer');
  $without_suffix = array('survey', 'response');
  if (!empty($entity_type) && in_array($ls_entity_type, $with_suffix)) {
    $ls_sid = str_replace(limesurvey_sync_api_entityname($ls_entity_type, 'prefix'), '', $entity_type);
    if (empty($ls_sid) || !is_numeric($ls_sid)) {
      return FALSE;
    }
    return $ls_sid;
  }
  return NULL;
}

/**
 * Return the survey ID from the type name.
 *
 * Example : 'ls_survey_123456' will return '123456'.
 *
 * @param $typename
 *   An string. Corresponding to the variable name.
 * @param $ls_entity_type
 *   A string, 'response', or 'answer', or 'token', or 'survey'.
 *
 * @return
 *   A numeric, the LS survey ID or FALSE if error or NULL if invalid
 *   parametters.
 */
function limesurvey_sync_api_retrieve_ls_sid_from_typename($typename, $ls_entity_type) {

  $entity_types = array('token', 'answer', 'survey');
  if (!empty($typename) && in_array($ls_entity_type, $entity_types)) {
    $ls_sid = str_replace(limesurvey_sync_api_typename($ls_entity_type, 'prefix'), '', $typename);
    if (empty($ls_sid) || !is_numeric($ls_sid)) {
      return FALSE;
    }
    return $ls_sid;
  }
  return NULL;
}

/**
 * Return the LS entity type from the entity type name.
 *
 * Example : 'limesurvey_sync_answer_123456' will return 'answer'.
 *
 * @param $entity_type
 *   The entity type of the entity.
 *
 * @return
 *   A string, the LS entity type or FALSE if error or NULL if invalid
 *   parametters.
 */
function limesurvey_sync_api_retrieve_ls_entity_type_from_entityname($entity_type) {

  if (empty($entity_type) || !is_string($entity_type)) {
    return FALSE;
  }
  $plit = explode('_', $entity_type);
  $count = count($plit);
  if ($count < 3 || strpos($entity_type, 'limesurvey_sync_') !== 0) {
    return FALSE;
  }
  $with_suffix = array('token', 'answer');
  $without_suffix = array('survey', 'response');
  $ls_entity_type = $plit[2];
  if (!in_array($ls_entity_type, array_merge($with_suffix, $without_suffix)) || (in_array($ls_entity_type, $without_suffix) && $count > 3) || (in_array($ls_entity_type, $with_suffix) && ($count > 4 || !is_numeric($plit[3])))) {
    return FALSE;
  }
  return $ls_entity_type;
}

/**
 * Return a dynamic title for answers and tokens.
 *
 * @param $entity
 *   The answer or token entity object.
 * @param $ls_key
 *   A string, The LS primary key fieldname on DP. It is provided into the
 *   entity_get_info, into 'entity keys' key as 'ls key'.
 *
 * @return
 *   A string, the tranlated title including the primary ls key value.
 */
function limesurvey_sync_api_default_label($entity, $ls_key) {

  $id = $entity->$ls_key;
  $arg = array(
    '@id' => $id,
    '@type' => drupal_ucfirst($entity->ls_entity_type),
    '@sid' => $entity->ls_sid,
  );
  return t('@type id #@id from the survey #@sid', $arg);
}

/**
 * List of the lss_status possible values.
 *
 * Used for compatibility with Views and Rules modules, throught
 * hook_entity_property_info().
 *
 * @return
 *   An array, the keys are the lss_status value and values are the human
 *   corresponding display message.
 */
function limesurvey_sync_api_lss_sync_status_list() {
  $status = array(
    LIMESURVEY_SYNC_SYNCHRONIZED,
    LIMESURVEY_SYNC_DESYNCHRONIZED,
    LIMESURVEY_SYNC_UNSYNCHRONIZED,
  );
  $list = array();
  module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
  foreach ($status as $value) {
    $list[$value] = limesurvey_sync_api_display_field('survey', 'lss_sync', $value);
  }
  return $list;
}

/**
 * Return the object info for a LS entity type .
 *
 * From the LimesurveySyncInfos class, for building entty info or schema or root
 * uri.
 *
 * @param $ls_entity_type
 *   A string, 'response', or 'answer', or 'token', or 'survey'.
 *
 * @return
 *   The entity info object.
 */
function limesurvey_sync_api_info_class_object($ls_entity_type = '') {

  $class = 'LimesurveySyncInfos' . drupal_ucfirst(str_replace('_', '', $ls_entity_type));
  $object = new $class();
  return $object;
}


/**
 * Generates an array which displays a node detail page.
 *
 * Inspired from entity_class_label() but can handle empty entity (for '/import'
 * menu as an example).
 *
 * @param $entity
 *   an entity object.
 * @param $entity_type
 *   The entity type of the entity.
 *
 * @return
 *   A $page element suitable for use by drupal_render().
 */
function limesurvey_sync_api_entity_class_label($entity, $entity_type) {
  // @TODO : Delete this function, replacing it by an appropriate title callback
  // into hook_menu().
  $label = '';
  if (!empty($entity)) {
    $label = $entity->label();
  }
  elseif ($info = entity_get_info($entity_type) && !empty($info['entity keys']['label']) && isset($entity->{$info['entity keys']['label']})) {
    $label = $entity->{$info['entity keys']['label']};
  }
  return $label;
}


/**
 * Create a new entity object for a LS entity type and a LS survey ID.
 *
 * @param $ls_entity_type
 *   A string, 'response', or 'answer', or 'token', or 'survey'.
 * @param $ls_sid
 *   A numeric, the LS survey ID.
 *
 * @return
 *   An entity object.
 */
function limesurvey_sync_api_entity_class_object($ls_entity_type, $ls_sid = FALSE) {

  $class = 'LimesurveySync' . drupal_ucfirst(str_replace('_', '', $ls_entity_type));
  $values = array(
    'ls_sid' => $ls_sid,
  );
  $entity_type = limesurvey_sync_api_entityname($ls_entity_type, $ls_sid);
  // $object = (!empty($sid)) ? new $class(array('ls_sid' => $sid)) : new $class;
  $object = entity_create($entity_type, $values);
  return $object;
}

/**
 * Return a type name for a LS entity type and a LS survey ID.
 *
 * As an example : return 'ls_answer_123456' for 'answer' and '123456'.
 *
 * @param $ls_entity_type
 *   A string, 'response', or 'answer', or 'token', or 'survey'.
 * @param $ls_sid
 *   A numeric, the LS survey ID or the string 'prefix' for returning the prefix
 *   ('ls_answer_' in the example). Or FALSE if an error occures.
 *
 * @return
 *   An entity object.
 */
function limesurvey_sync_api_typename($ls_entity_type, $ls_sid = FALSE) {

  if (empty($ls_sid)) {
    return FALSE;
  }
  $prefix = "ls_{$ls_entity_type}_";
  if ($ls_sid == 'prefix') {
    $ls_sid = '';
  }
  switch ($ls_entity_type) {
    case 'prefix':
    case 'survey':
    case 'answer':
    case 'token':
    case 'response':
      // Return 'ls_survey_123456' or 'ls_answer_123456', or 'ls_token_123456'.
      return "ls_{$ls_entity_type}_{$ls_sid}";
  }
  return FALSE;
}

/**
 * Return field name for a LS entity type and a LS survey ID.
 *
 * As an example : return 'field_ls_answer_123456' for 'answer' and '123456'. It
 * is used by the Field core module.
 *
 * @param $ls_entity_type
 *   A string, 'response', or 'answer', or 'token', or 'survey'.
 * @param $ls_sid
 *   A numeric, the LS survey ID. Default is FALSE.
 *
 * @return
 *   Astring, the field name.
 */
function limesurvey_sync_api_et_fieldname($ls_entity_type, $ls_sid = FALSE) {
  return 'field_' . limesurvey_sync_api_typename($ls_entity_type, $ls_sid);
}

/**
 * Return entity name for a LS entity type and a LS survey ID.
 *
 * As an example : return 'limesurvey_sync_answer_123456' for 'answer' and
 * '123456'.
 *
 * @param $ls_entity_type
 *   A string, 'response', or 'answer', or 'token', or 'survey'.
 * @param $ls_sid
 *   A numeric, the LS survey ID. Default is FALSE.
 *
 * @return
 *   Astring, the field name.
 */
function limesurvey_sync_api_entityname($ls_entity_type, $ls_sid = FALSE) {

  // If $ls_sid value is 'prefix : return the prefix entity name : as an example :
  // 'limesurvey_sync_answer_'.
  $with_suffix = array('token', 'answer');
  $without_suffix = array('survey', 'response');

  if (in_array($ls_entity_type, $with_suffix) && (empty($ls_sid) || (!is_numeric($ls_sid) && $ls_sid != 'prefix'))) {
    return FALSE;
  }
  if ($ls_sid == 'prefix') {
    $ls_sid = '';
  }
  $sid_suffix = (in_array($ls_entity_type, $with_suffix)) ? "_{$ls_sid}" : '';
  if (in_array($ls_entity_type, array_merge($with_suffix, $without_suffix))) {
    // Return 'limesurvey_sync_survey' or 'limesurvey_sync_answer_123456', or
    // or 'limesurvey_sync_token_123456'.
    return "limesurvey_sync_{$ls_entity_type}{$sid_suffix}";
  }
  return FALSE;
}

/**
 * Return database table name for a LS entity type and a LS survey ID.
 *
 * As an example : return 'limesurvey_sync_answer_123456_revision' for 'answer'
 * and '123456' and TRUE.
 *
 * @param $ls_entity_type
 *   A string, 'response', or 'answer', or 'token', or 'survey'.
 * @param $ls_sid
 *   A numeric, the LS survey ID. Default is FALSE.
 * @param $revision
 *   A boolean, if TRUE, return the revision table name. Default is FALSE.
 *
 * @return
 *   A string, the database table name.
 */
function limesurvey_sync_api_tablename($ls_entity_type, $ls_sid = FALSE, $revision = FALSE) {
  $entity_name = limesurvey_sync_api_entityname($ls_entity_type, $ls_sid);
  $revision_suffix = ($revision && $entity_name) ? '_revision' : '';
  return $entity_name . $revision_suffix;
}
