<?php

/**
 * @file
 * Functions and classes for the response entities.
 */

define('LIMESURVEY_SYNC_RESPONSE_END', 1);
define('LIMESURVEY_SYNC_RESPONSE_START', 2);
define('LIMESURVEY_SYNC_RESPONSE_OUTSIDE_DP', 3);

/**
 * Class managing building response entities.
 *
 * Building entity_info, entity_property_info, schema, entity default values and
 * updating entity table if needed. Take care of Drupal fields, LimeSurvey fixed
 * fields and Limesurver variables fields.
 */
class LimesurveySyncResponse extends LimesurveySync {
  public $ls_sid;
  public $ls_entity_type = 'response';

  public function __construct(array $values = array(), $entityType = NULL) {
    // The type should alwyes be provided into the $values variable.
    if (!empty($values['type'])) {
      // Provide ls_sid from the survey.
      $survey = limesurvey_sync_survey_get_surveys($values['type']);
      $values['ls_sid'] = ($survey) ? $survey->ls_sid : '';
    }
    parent::__construct($values, $entityType);
  }

  public function ls_synchronize_references_ls2dp($all = FALSE) {
    $arg = ($all != 'all') ? $this : 'all';
    return entity_get_controller($this->entityType)->ls_synchronize_references_ls2dp($arg);
  }

  protected function defaultLabel() {
    return $this->title;
  }
}

/**
 * The Controller for response entities
 */
class LimesurveySyncResponseController extends LimesurveySyncController {
  public $ls_entity_type = 'response';
  public $ls_key = 'ls_sid';

  /**
   * Overridden.
   *
   * Set the ls_sid value from the type value.
   */
  public function create(array $values = array()) {
    // Add values that are specific to our LimesurveySyncSurvey
    if (!empty($values['type'])) {
      $values['ls_sid'] = limesurvey_sync_api_retrieve_ls_sid_from_typename($values['type'], 'survey');
    }
    $entity = parent::create($values);
    return $entity;
  }

  /**
   * Provide the LimeSurvey Sync module entity synchronzation status.
   *
   * Status depending on the references to the survey and answer and token
   * associated entities.
   */
  protected function lss_sync($entity, $existing_ls_enties_keyed_by_ls_key = array()) {

    $answer_need_sync = $token_need_sync = $survey_need_sync = FALSE;

    $answer_field_name = limesurvey_sync_api_et_fieldname('answer', $entity->ls_sid);
    $answer_field = (!empty($entity->$answer_field_name)) ? $entity->$answer_field_name : array();
    if (!empty($answer_field[LANGUAGE_NONE][0]['target_id'])) {
      $dp_aid = $answer_field[LANGUAGE_NONE][0]['target_id'];
      $answer_entityname = limesurvey_sync_api_entityname('answer', $entity->ls_sid);
      $answer_entities = entity_load($answer_entityname, array($dp_aid));
      if (!empty($answer_entities)) {
        $answer_entity = $answer_entities[$dp_aid];
        $answer_lss_sync = LIMESURVEY_SYNC_SYNCHRONIZED;
      }
      else {
        $answer_lss_sync = LIMESURVEY_SYNC_DESYNCHRONIZED;
      }
    }
    else {
      $answer_lss_sync = LIMESURVEY_SYNC_UNSYNCHRONIZED;
    }

    $token_field_name = limesurvey_sync_api_et_fieldname('token', $entity->ls_sid);
    $token_field = (!empty($entity->$token_field_name)) ? $entity->$token_field_name : array();
    if (!empty($token_field[LANGUAGE_NONE][0]['target_id'])) {
      $dp_tid = $token_field[LANGUAGE_NONE][0]['target_id'];
      $token_entityname = limesurvey_sync_api_entityname('token', $entity->ls_sid);
      $token_entities = entity_load($token_entityname, array($dp_tid));
      if (!empty($token_entities)) {
        $token_entity = $token_entities[$dp_tid];
        $token_lss_sync = LIMESURVEY_SYNC_SYNCHRONIZED;
      }
      else {
        $token_lss_sync = LIMESURVEY_SYNC_DESYNCHRONIZED;
      }
    }
    else {
      $token_lss_sync = LIMESURVEY_SYNC_UNSYNCHRONIZED;
    }

    $survey_type = $entity->type;
    if (!empty($survey_type)) {
      $survey_entity = limesurvey_sync_survey_get_surveys($entity->type);
      if ($survey_entity) {
        $survey_lss_sync = LIMESURVEY_SYNC_SYNCHRONIZED;
      }
      else {
        $survey_lss_sync = LIMESURVEY_SYNC_DESYNCHRONIZED;
      }
    }
    else {
      $survey_lss_sync = LIMESURVEY_SYNC_UNSYNCHRONIZED;
    }
    // Retrieve token response if exists.
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
    $survey_properties = limesurvey_sync_survey_properties($entity->ls_sid);

    if ($answer_lss_sync == LIMESURVEY_SYNC_DESYNCHRONIZED || ($token_lss_sync == LIMESURVEY_SYNC_DESYNCHRONIZED && $survey_properties['tokens_table']) || $survey_lss_sync == LIMESURVEY_SYNC_DESYNCHRONIZED) {
      return LIMESURVEY_SYNC_DESYNCHRONIZED;
    }
    elseif ($survey_lss_sync == LIMESURVEY_SYNC_UNSYNCHRONIZED) {
      return LIMESURVEY_SYNC_UNSYNCHRONIZED;
    }
    // There is only Token and answer (synchronized or unsynchronized).
    elseif ($answer_lss_sync == LIMESURVEY_SYNC_UNSYNCHRONIZED && (!$survey_properties['tokens_table'] || $token_lss_sync == LIMESURVEY_SYNC_UNSYNCHRONIZED)) {
      return LIMESURVEY_SYNC_UNSYNCHRONIZED;
    }
    elseif ($answer_lss_sync == LIMESURVEY_SYNC_SYNCHRONIZED && (!$survey_properties['tokens_table'] || $token_lss_sync == LIMESURVEY_SYNC_SYNCHRONIZED)) {
      return LIMESURVEY_SYNC_SYNCHRONIZED;
    }
    else {
      return LIMESURVEY_SYNC_UNSYNCHRONIZED;
    }
  }

  /**
   * Provide the response entity status.
   *
   * Status are 'response ended from DP', 'response started from DP', and
   * 'respone ouside from DP'.
   */
  protected function lss_status($entity) {
    if (!empty($entity->dp_back)) {
      return LIMESURVEY_SYNC_RESPONSE_END;
    }
    elseif (!empty($entity->dp_editing)) {
      return LIMESURVEY_SYNC_RESPONSE_START;
    }
    else {
      return LIMESURVEY_SYNC_RESPONSE_OUTSIDE_DP;
    }
  }

  /**
   * Overridden.
   *
   * Add response ls_sid from the survey entity on loading entities.
   */
  public function load($ids = array(), $conditions = array()) {

    $entities = parent::load($ids, $conditions);
    foreach ($entities as $entity) {
      // Provide ls_sid from the survey.
      $survey = limesurvey_sync_survey_get_surveys($entity->type);
      $entity->ls_sid = $survey->ls_sid;
    }
    return $entities;
  }


  /**
   * Builds a structured array representing the entity's content.
   *
   */
  public function buildContent($entity, $view_mode = 'full', $langcode = NULL, $content = array()) {

    module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
    $render_admin = limesurvey_sync_api_fields_render($this->ls_entity_type, $entity, $this->ls_sid);
    $content['response_admin_datas'] = $render_admin + array(
      '#theme' => 'limesurvey_sync_admin_data_inline',
    );

    $build = parent::buildContent($entity, $view_mode, $langcode, $content);
    return $build;
  }

  /**
   * Save response entity.
   *
   * Look for 'dp_start_answer' and 'dp_backfromls' flags. Depending on thoses
   * flags or if there is token persistance with answers, bound answer and or
   * token to the response. On saving, synchronize (and update if needed)
   * associated answer and token to.
   */
  public function save($entity, DatabaseTransaction $transaction = NULL) {

    // set a title.
    if (empty($entity->title)) {
      $entity->title = limesurvey_sync_api_response_create_title($entity->ls_sid);
    }

    // Flag fired from lsback page (switch back to DP from the LS filling page).
    if (!empty($entity->dp_start_answer)) {
      $entity->dp_editing = REQUEST_TIME;
      $entity->dp_back = 0;
    }

    if (!empty($entity->dp_backfromls)) {
      $entity->dp_back = REQUEST_TIME;
    }

    // Handle responses to token and answer.

    $token_field_name = limesurvey_sync_api_et_fieldname('token', $entity->ls_sid);
    $token_field = (isset($entity->$token_field_name)) ? $entity->$token_field_name : array();
    $token_entityname = limesurvey_sync_api_entityname('token', $entity->ls_sid);

    // Retrieve token response if exists.
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
    $survey_properties = limesurvey_sync_survey_properties($entity->ls_sid);
    $token_entity = NULL;
    if ($survey_properties['tokens_table']) {
      if (!isset($token_field[LANGUAGE_NONE][0]['target_id']) || empty($token_field[LANGUAGE_NONE][0]['target_id'])) {
        // No token invitation
        if (!empty($entity->dp_start_answer)) {
          // Create it if asked.
          $token_entity = entity_create($token_entityname, array());
        }
      }
      else {
        // A token entity exists. Load it.
        $dp_tid = $token_field[LANGUAGE_NONE][0]['target_id'];
        $token_entities = entity_load($token_entityname, array($dp_tid));
        $token_entity = (!empty($token_entities)) ? $token_entities[$dp_tid] : NULL;
        if (empty($token_entity) && !empty($entity->dp_start_answer)) {
          // The token entry had existed but had been deleted.
          $token_entity = entity_create($token_entityname, array());
          // update the entity response value
          $entity->$token_field[LANGUAGE_NONE][0]['target_id'] = $token_entity->dp_tid;
        }
      }
    }
    // Handle Flag.
    $token_saved = FALSE;
    if ($token_entity && !empty($entity->dp_start_answer)) {
      // The "edit answer" checkbox is checked.
      // Edit the token entity accordingly to the entity response.
      $token_entity->ls_language = $entity->ls_language;
      // Set a flag for activating the token during the save token process.
      $token_entity->activate_token = TRUE;
      // Save the token. The LS token entry will be modified and the DP entry to.
      entity_save($token_entityname, $token_entity);
      $token_saved = TRUE;

      // Retrieve and set the DP token ID.
      if (empty($entity->{$token_field_name}[LANGUAGE_NONE][0]['target_id'])) {
        $dp_tid = $token_entity->dp_tid;
        // And set it to the response entity.
        $entity->{$token_field_name}[LANGUAGE_NONE][0]['target_id'] = $token_entity->dp_tid;
        $entity->{$token_field_name}[LANGUAGE_NONE][0]['target_type'] = $token_entityname;
      }
      $entity_for_url = $token_entity;
    }

    // Handle Answer entity
    $answer_field_name = limesurvey_sync_api_et_fieldname('answer', $entity->ls_sid);
    $answer_field = $entity->$answer_field_name;
    $answer_entityname = limesurvey_sync_api_entityname('answer', $entity->ls_sid);
    if (!isset($answer_field[LANGUAGE_NONE][0]['target_id']) || empty($answer_field[LANGUAGE_NONE][0]['target_id'])) {

      if ($token_entity && !empty($token_entity->ls_token)) {
        $find_dp_aid = limesurvey_sync_api_response_retrieve_dp_aid_from_token($entity->ls_sid, $token_entity->ls_token);
        if ($find_dp_aid['dp_aid']) {
          $entity->{$answer_field_name}[LANGUAGE_NONE][0]['target_id'] = $find_dp_aid['dp_aid'];
          $entity->{$answer_field_name}[LANGUAGE_NONE][0]['target_type'] = $answer_entityname;
        }
      }
    }

    // provide url for filling the survey : with or without the token.
    $entity_for_url = ($token_entity) ? $token_entity : $entity;
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_answer');
    $entity->filling_url = limesurvey_sync_ls_api_answer_url($entity_for_url);

    // On saving a response, make sur to synchronize associated token and answer.
    $synchronize_ref = $this->ls_synchronize_references_ls2dp($entity, TRUE);

    parent::save($entity, $transaction);
  }

  /**
   * Test the synchronization for response associated entities.
   *
   * When the response does not need update, look For associated token or answer
   * entities that may need update. Add a 'need_sync_ref' key to the result for
   * updating assocated entities if needed.
   */
  public function ls_synchronize(&$entity, $way = 'ls2dp') {
    // For responses : no dp2ls synchronization.
    $synchronize_set = parent::ls_synchronize($entity, 'ls2dp');

    if (!empty($synchronize_set['ok'])) {
      $response_entityname = limesurvey_sync_api_entityname('response');
      foreach ($synchronize_set['ok'] as $dp_rid => $data) {
        $response_entities = entity_load($response_entityname, array($dp_rid));
        $response_entity = $response_entities[$dp_rid];
        $synchronize_ref = $this->ls_synchronize_references_ls2dp($response_entity);
        if (!empty($synchronize_ref['need_sync_ref'])) {
          $synchronize_set['need_sync_ref'][$dp_rid] = $synchronize_ref['need_sync_ref'][$dp_rid];
        }
      }

    }
    return $synchronize_set;
  }

  /**
   * Test the synchronization for response associated entities.
   *
   * When the response does not need update, look for associated token or answer
   * entities that may need update (an update it depending on the $save
   * parametter).
   *
   * @param $entity
   *  The response entity object.
   * @param $save
   *  A boolean, If TRUE save the associated reference that need saving. Default
   *  is FALSE.
   *
   * @return
   *   An array. First keys are 'ok_ref' and 'need_sync_ref' if associated
   *   entities need update. Then it is the dp_rid_value, the 'token' or
   *   'answer', then ls premary key, then the entity object.
   */
  public function ls_synchronize_references_ls2dp($entity, $save = FALSE) {

    $dp_key = $this->idKey; // entityInfo['entity keys']['id'];
    $this_dp_id = FALSE;

    if (!empty($entity->$dp_key)) {
      $this_dp_id = $entity->$dp_key;
    }
    else {
      // Only one entity : Use 'this' index.
      $this_dp_id = 'this';
    }

    $token_field_name = limesurvey_sync_api_et_fieldname('token', $entity->ls_sid);
    $token_entityname = limesurvey_sync_api_entityname('token', $entity->ls_sid);
    $answer_field_name = limesurvey_sync_api_et_fieldname('answer', $entity->ls_sid);
    $answer_entityname = limesurvey_sync_api_entityname('answer', $entity->ls_sid);


    $dp_key_value = ($this_dp_id != 'this') ? $entity->$dp_key : 'this';

    $token_field = (isset($entity->$token_field_name)) ? $entity->$token_field_name : array();
    $answer_field = (isset($entity->$answer_field_name)) ? $entity->$answer_field_name : array();

    $synchronize_ref = array();
    // On saving a response, make sur to synchronize associated token and answer.
    if (!empty($token_field[LANGUAGE_NONE][0]['target_id']) && $token_entities = entity_load($token_entityname, array($token_field[LANGUAGE_NONE][0]['target_id']))) {
      $dp_tid = $token_field[LANGUAGE_NONE][0]['target_id'];
      $token_entity = $token_entities[$dp_tid];
      $synchronize_set = $token_entity->ls_synchronize();
      if (!empty($token_entity->need_save_ls2dp)) {
        $synchronize_ref['need_sync_ref'][$dp_key_value]['dp'] = $entity;
        $synchronize_ref['need_sync_ref'][$dp_key_value]['token'][$dp_tid] = $token_entity;
        if ($save) {
          entity_save($token_entityname, $token_entity);
        }
      }
      else {
        $synchronize_ref['ok_ref'][$dp_key_value]['token'][$dp_tid] = TRUE;
      }
    }

    if (!empty($answer_field[LANGUAGE_NONE][0]['target_id']) && $answer_entities = entity_load($answer_entityname, array($answer_field[LANGUAGE_NONE][0]['target_id']))) {
      $dp_aid = $answer_field[LANGUAGE_NONE][0]['target_id'];
      $answer_entity = $answer_entities[$dp_aid];
      $synchronize_set = $answer_entity->ls_synchronize();
      if (!empty($answer_entity->need_save_ls2dp)) {
        $synchronize_ref['need_sync_ref'][$dp_key_value]['dp'] = $entity;
        $synchronize_ref['need_sync_ref'][$dp_key_value]['answer'][$dp_aid] = $answer_entity;
        if ($save) {
          entity_save($answer_entityname, $answer_entity);
        }
      }
      else {
        $synchronize_ref['ok_ref'][$dp_key_value]['token'][$dp_aid] = TRUE;
      }
    }
    return $synchronize_ref;
  }

  /**
   * Delete.
   *
   * On deleting a response : delete the associated answer and token refences on
   * DP.
   */
  public function delete($ids, DatabaseTransaction $transaction = NULL) {
    $entities = $ids ? $this->load($ids) : FALSE;
    if ($entities) {
      // Delete associated answer and token entities and fields.
      // @TODO : It may be optional to delete references associated to response.
      $dp_key_value = ($this_dp_id != 'this') ? $entity->$dp_key : 'this';
      foreach ($entities as $id => $entity) {
        foreach (array('answer', 'token') as $ls_type) {
          $ref_field_name = limesurvey_sync_api_et_fieldname($ls_type, $entity->ls_sid);
          $ref_entityname = limesurvey_sync_api_entityname($ls_type, $entity->ls_sid);
          $ref_field = (isset($entity->$ref_field_name)) ? $entity->$ref_field_name : array();
          if (!empty($ref_field[LANGUAGE_NONE][0]['target_id'])) {
            // This will delete the associated field
            entity_delete($ref_entityname, $ref_field[LANGUAGE_NONE][0]['target_id']);
          }
        }
      }
      parent::delete($ids, $transaction);
    }
  }

}

/**
 * The UI Controller for response entities
 */
class LimesurveySyncResponseUIController extends LimesurveySyncUIController {
  public $ls_entity_type = 'response';
  // Survey are added by a user.
  protected $disallow_add = FALSE;
  protected $disallow_edit = FALSE;
  protected $move_to_content_tab = TRUE;

  public function hook_menu() {
    // use the basc system
    // $items = EntityDefaultUIController::hook_menu();
    $items = parent::hook_menu();

    $path_add = $this->path . '/add';
    if (isset($path_add)) {
      unset($items[$this->path . '/add']);
      $new_add_path = limesurvey_sync_api_root_uri($this->ls_entity_type, '', FALSE) . '/add';
      $items[$new_add_path] = array(
        'title' => 'Add response',
        'page callback' => 'limesurvey_sync_response_add_page',
        'access callback' => '_limesurvey_sync_response_add_access',
      );
      foreach (limesurvey_sync_survey_get_surveys() as $survey) {
        $survey_url_str = str_replace('_', '-', $survey->type);
        $menu_item = array(
          // No t() for title menu item.
          'title' => format_string('Add @type', array('@type' => $survey->label)),
          'title callback' => 'check_plain',
          'page callback' => 'limesurvey_sync_response_add',
          'page arguments' => array($survey->type),
          'access callback' => '_limesurvey_sync_response_add_access',
          'access arguments' => array($survey->type),
          'description' => $survey->description,
        );

        $items[$new_add_path . '/' . $survey_url_str] = $menu_item;
        // Add this for the menu navigation form admin>content>limesurvey.
        $items[$this->path . '/add/' . $survey_url_str] = $menu_item;
        $items[$this->path . '/add/' . $survey_url_str]['page callback'] = 'drupal_goto';
        $items[$this->path . '/add/' . $survey_url_str]['page arguments'] = array($new_add_path . '/' . $survey_url_str);
      }
    }

    // Change the overview page : becomming a tab on admin/content.
    $items[$this->path]['type'] = MENU_LOCAL_TASK | MENU_NORMAL_ITEM;

    // Handle the LS form page :
    $items['lsform/%'] = array(
      'title' => 'Form',
      'page callback' => 'limesurvey_sync_invitation_fill_survey',
      'page arguments' => array(1),
      'access callback' => 'limesurvey_sync_invitation_fill_survey_access',
      'access arguments' => array(1),
      // @TODO
      'type' => MENU_CALLBACK,
      'file path' => drupal_get_path('module', 'limesurvey_sync') . '/api',
      'file' => 'limesurvey_sync_invitation.form.inc',
    );

    // Back url from LS.
    $ls_back = rtrim(limesurvey_sync_api_response_lsback_path(), '/');
    $items["$ls_back/%limesurvey_sync_survey"] = array(
      'title' => 'End of survey',
      'page callback' => 'limesurvey_sync_invitation_back_from_ls',
      'page arguments' => array(1, NULL),
      'access callback' => 'limesurvey_sync_api_response_back_from_ls_access',
      'access arguments' => array(1),
      'type' => MENU_CALLBACK,
      'weight' => 15,
      'file path' => drupal_get_path('module', 'limesurvey_sync') . '/api',
      'file' => 'limesurvey_sync_invitation.back.inc',
    );

    // Page for iframes, redirect to the 'response/%' page.
    $items["$ls_back/%limesurvey_sync_survey/%"] = array(
      'title' => 'End of survey',
      'page callback' => 'limesurvey_sync_invitation_back_from_ls',
      'page arguments' => array(1, 2),
      'access callback' => 'limesurvey_sync_api_response_back_from_ls_access',
      'access arguments' => array(1, 2),
      'type' => MENU_CALLBACK,
      'file path' => drupal_get_path('module', 'limesurvey_sync') . '/api',
      'file' => 'limesurvey_sync_invitation.back.inc',
    );

    return $items;
  }

  protected function overviewTableHeaders($conditions, $rows, $additional_header = array()) {

    $additional_header = array(
      t('Survey'),
      t('User'),
    );
    return parent::overviewTableHeaders($conditions, $rows, $additional_header);
  }

  protected function overviewTableRow($conditions, $id, $entity, $additional_cols = array()) {
    $survey = limesurvey_sync_survey_get_surveys($entity->type);
    $additional_cols[] = $survey->label;
    $account = user_load($entity->uid);
    $additional_cols[] = $account->name;

    return parent::overviewTableRow($conditions, $id, $entity, $additional_cols);
  }

}

/**
 * The entity info class for response entities.
 */
class LimesurveySyncInfosResponse extends LimesurveySyncInfos {
  public $ls_entity_type = 'response';
  public $root_admin_ui = 'admin/content/limesurvey';
  protected $entity_keys = array(
    'label' => 'title',
  );
  protected $revision = TRUE;
  protected $date_fields = TRUE;
  protected $language_field = TRUE;
  protected $ls_sync_field = TRUE;
  protected $user_field = TRUE;
  protected $primary_key = 'dp_rid';
  protected $table_create_during_install = TRUE;
  protected $fieldable = TRUE;
  protected $exportable = TRUE;
  protected $label = 'Response';
  protected $plural_label = 'Responses';
  protected $bundle = 'type';

  protected function schema_dp_table($sid = NULL) {

    return array(
      'description' => 'Stores responses datas for limeSurvey datas, survey as the bundle.',
      // @TODO: add some indexes.
      'foreign keys' => array(
        'type' => array('limesurvey_sync_survey' => 'type'),
      ),
    );
  }

  protected function schema_dp_fields($sid = NULL, $revision = FALSE) {

    $schema = parent::schema_dp_fields($sid, $revision);
    $schema['lss_status'] = array(
      'type' => 'int',
      'unsigned' => TRUE,
      'not NULL' => TRUE,
      'default' => 1,
      'size' => 'tiny',
      'description' => 'Status of this token from the sid #' . $sid . '(1 for ended from Drupal, 2 for started from Drupal, 3 for created outside from Drupal).',
    );

    $more_fields = array(
      'type' => array(
        'description' => 'The machine-readable name of this survey.',
        'type' => 'varchar',
        'length' => 255,
        'not null' => TRUE,
      ),
      'ls_language' => array(
        'type' => 'varchar',
        'length' => 25,
        'default' => NULL,
        'description' => 'The LimeSurvey language asked from Drupal.',
      ),
      'dp_editing' => array(
        'type' => 'int',
        'not null' => TRUE,
        'default' => 0,
        'description' => 'The date when the answer has been edited from the Drupal site.',
      ),
      'dp_back' => array(
        'type' => 'int',
        'not null' => TRUE,
        'default' => 0,
        'description' => 'The date when the answer came back to the Drupal site from the LimeSurvey site.',
      ),
      'title' => array(
        'description' => 'The title of this response, always treated as non-markup plain text.',
        'type' => 'varchar',
        'length' => 255,
        'not null' => TRUE,
        'default' => '',
      ),
      // Added for later developpement. @TODO.
      'published' => array(
        'description' => 'Boolean indicating whether the node is published (visible to non-administrators).',
        'type' => 'int',
        'not null' => TRUE,
        'default' => 1,
      ),
      'comment' => array(
        'description' => 'Whether comments are allowed on this response: 0 = no, 1 = closed (read only), 2 = open (read/write).',
        'type' => 'int',
        'not null' => TRUE,
        'default' => 0,
      ),
      'promote' => array(
        'description' => 'Boolean indicating whether the response should be displayed on the front page.',
        'type' => 'int',
        'not null' => TRUE,
        'default' => 0,
      ),
      'sticky' => array(
        'description' => 'Boolean indicating whether the response should be displayed at the top of lists in which it appears.',
        'type' => 'int',
        'not null' => TRUE,
        'default' => 0,
      ),
    );
    return array_merge($schema, $more_fields);
  }

  protected function entity_property_info_dp_fields($sid = NULL) {

    $info = parent::entity_property_info_dp_fields($sid);
    $info['lss_status'] = array(
      'label' => t('DP response status'),
      'description' => t('statut ID for the response (1 for ended from Drupal, 2 for started from Drupal, 3 for created outside from Drupal).'),
      'type' => 'list<int>',
      'options list' => 'limesurvey_sync_api_response_lss_status_list',
      'schema field' => 'lss_status',
    );
    $info['type'] = array(
      'label' => t('Survey type'),
      'description' => t('The survey ID into the Drupal software'),
      'type' => 'limesurvey_sync_survey',
      'schema field' => 'type',
      'required' => TRUE,
      'bundle' => 'limesurvey_sync_survey',
    );
    $info['ls_language'] = array(
      'label' => t('LimeSurvey language'),
      'description' => t('The LimeSurvey language asked from Drupal.'),
      'type' => 'token',
      'options list' => 'limesurvey_sync_survey_ls_language_list',
      'schema field' => 'ls_language',
    );
    $info['dp_editing'] = array(
      'label' => t('Start date editing LS from DP'),
      'description' => t('The date when the answer has been edited from the Drupal site.'),
      'type' => 'date',
      'schema field' => 'dp_editing',
    );
    $info['dp_back'] = array(
      'label' => t('End date editing LS from DP'),
      'description' => t('The date when the answer came back to the Drupal site from the LimeSurvey site.'),
      'type' => 'date',
      'schema field' => 'dp_back',
    );
    $info['title'] = array(
      'label' => t('Title'),
      'description' => t('The title of this response'),
      'type' => 'text',
      'schema field' => 'title',
    );
    // @TODO : add published, promote, comment, sticky.
    return $info;
  }

  protected function create_dp_fields_default_values($sid = NULL) {

    return array(
      'type' => '',
      'lss_status' => LIMESURVEY_SYNC_RESPONSE_OUTSIDE_DP,
      'ls_language' => '',
      'dp_editing' => 0,
      'dp_back' => 0,
      'title' => '',
      'published' => 1,
      'comment' => 0,
      'promote' => 0,
      'sticky' => 0,
    );
  }

  public function update_table() {
    // Create the entity response fields add attach them to the bundle.
    $surveys = db_select('limesurvey_sync_survey', 'lss')->fields('lss')->execute()->fetchAllAssoc('type');
    $rebuild_schema = FALSE;

    // Create one entity type per survey ID.
    foreach ($surveys as $survey) {
      foreach (array('answer', 'token') as $ls_entity_type) {
        $entity_type = limesurvey_sync_api_entityname($ls_entity_type, $survey->ls_sid);
        if ($entity_info = entity_get_info($entity_type)) {
          // The sub entity info exists. Create the field.
          $field_name = limesurvey_sync_api_et_fieldname($ls_entity_type, $survey->ls_sid);
          if (!field_info_field($field_name)) {
            // the field dos not exists : create it.
            $field = array(
              'field_name' => $field_name,
              'type' => 'entityreference',
              //'cardinality' => FIELD_CARDINALITY_UNLIMITED,
              'settings' => array(
                'target_type' => $entity_type,
                'handler_settings' => array('target_bundles' => NULL),
              ),
              'cardinality' => 1,
              'translatable' => FALSE,
            );
            field_create_field($field);
            $link = limesurvey_sync_api_root_uri('survey', $survey->ls_sid) . '/' . $survey->type . '/fields';
            watchdog('limesurvey_sync', 'The %fieldname field as been created', array('%fieldname' => $field_name), WATCHDOG_INFO, $link);
          }

          $bundle_name = $survey->type;
          $instance_entity_type = 'limesurvey_sync_response';
          if (!field_info_instance($instance_entity_type, $field_name, $bundle_name)) {
            $instance = array(
              'field_name' => $field_name,
              'entity_type' => $instance_entity_type,
              'bundle' => $bundle_name,
              'label' => t('@ls_entity_type', array('@ls_entity_type' => $ls_entity_type)),
              'widget' => array(
                'type' => 'options_select',
              ),
              'display' => array(
                'default' => array(
                  'label' => 'inline',
                  'type' => 'entityreference_label',
                  // set as link by default.
                  'settings' => array(
                    'link' => 1,
                  ),
                ),
              ),
            );
            field_create_instance($instance);
            $link = limesurvey_sync_api_root_uri('survey', $survey->ls_sid) . '/' . $bundle_name . '/fields';
            watchdog('limesurvey_sync', 'The instance for the %entity entity with the %fieldname field and %bundle bundle as been created', array('%fieldname' => $field_name, '%bundle' => $bundle_name, '%entity' => $entity_type), WATCHDOG_INFO, $link);
          }
        }
      }
    }
  }

}

/**
 * Loader for menu wildcard %limesurvey_response.
 *
 * @param $dp_rid
 *   A numeric, the response entity primary key.
 *
 * @return
 *   The loaded LimeSurvey response entity, or FALSE if not exists.
 */
function limesurvey_sync_response_load($dp_rid = NULL) {
  if (empty($dp_rid) || !is_numeric($dp_rid) || $dp_rid < 0) {
    return FALSE;
  }
  $entity = entity_load_multiple_by_name('limesurvey_sync_response', array($dp_rid));
  if (is_array($entity) && isset($entity[$dp_rid])) {
    return $entity[$dp_rid];
  }
  return FALSE;
}

/**
 * Build the iframe page uri.
 *
 * @param $entity
 *   A response entity object. Default is NULL.
 *
 * @return
 *   The iframe page uri, as 'lsform' or 'lsform/12' (as an example).
 */
function limesurvey_sync_api_response_form_url($entity = NULL) {
  $base = 'lsform';
  if (!empty($entity->dp_rid)) {
    return $base . '/' . $entity->dp_rid;
  }
  return $base;
}

/**
 * Build the uri of the page LS should redirect when completing an answer.
 *
 * @param $ls_sid
 *   A numeric, the LS survey ID. Default is NULL.
 * @param $entity
 *  A response entity object. Default is NULL.
 *
 * @return
 *   The page uri, as 'lsback/ls-survey-xxxxxx' or 'lsback/ls-survey-xxxxxx/12'
 *   (as an example).
 */
function limesurvey_sync_api_response_lsback_path($ls_sid = NULL, $entity = NULL) {
  $survey_type = (!empty($ls_sid)) ? limesurvey_sync_api_typename('survey', $ls_sid) : '';
  $base = ($survey_type) ? 'lsback/' . str_replace('_', '-', $survey_type) : 'lsback';
  if (!empty($entity->dp_rid)) {
    return $base . '/' . $entity->dp_rid;
  }
  return $base;
}

/**
 * Return the correct back url from LS for each language.
 *
 * @param $ls_sid
 *   A numeric, the LS survey ID. Default is NULL.
 * @param $absolute
 *  A boolean, TRUE for retruning absolute url. Default is TRUE.
 *
 * @return
 *   An array, the correct back urls keyed by there LS language.
 */
function limesurvey_sync_api_response_ls_back_urls($ls_sid = NULL, $absolute = TRUE) {
  module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api_helpers');
  $langs_ls2dp = limesurvey_sync_api_get_ls_langs($ls_sid); //'installed'
  $options = $absolute ? array('absolute' => TRUE) : array();
  $path = limesurvey_sync_api_response_lsback_path($ls_sid);

  $dp_default_lang = language_default();
  $languages = language_list('enabled');
  $dp_languages_object = $languages[1];

  module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api');
  $ls_back_urls = array();
  foreach ($langs_ls2dp as $ls_lang => $human_lang) {
    // mangage http://example.com/de/contact and http://de.example.com/contact
    // thanks to options['language'], converting language to Drupal.
    $dp_lang = limesurvey_sync_ls_api_fixlang_ls2dp($ls_lang);
    if (isset($dp_languages_object[$dp_lang])) {
      $options['language'] = $dp_languages_object[$dp_lang];
    }
    else {
      // The language exists en LS but is not enabled on Drupal : Use the
      // default site language.
      $options['language'] = $dp_languages_object[$dp_default_lang->language];
    }
    $ls_back_urls[$ls_lang] = url($path, $options);
  }
  return $ls_back_urls;
}

/**
 * Return TRUE if the back urls are correctly set on LS for all LS languages.
 *
 * @param $ls_sid
 *   A numeric, the LS survey ID. Default is NULL.
 * @param $absolute
 *  A boolean, TRUE for retruning absolute url. Default is TRUE.
 *
 * @return
 *   An array, the correct back urls keyed by there LS language.
 */
function limesurvey_sync_api_response_check_ls_back_urls($ls_sid = NULL, $absolute = TRUE) {
  $correct_ls_back_urls = limesurvey_sync_api_response_ls_back_urls($ls_sid, $absolute);
  module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
  $survey_properties = limesurvey_sync_survey_properties($ls_sid);
  $ls_path = variable_get('limesurvey_sync_ls_path', '');
  $actual_ls_back_urls = array();
  foreach ($survey_properties['#lang'] as $ls_lang => $datas_ls_lang) {
    $source_url = $datas_ls_lang['back_url'];
    // Handle back url from LS, comparing to the drupal url() version.
    $actual_back_url = limesurvey_sync_drupal_formated_url(limesurvey_sync_rel2abs($source_url, limesurvey_sync_rel2abs($ls_path . '/')));
    $actual_ls_back_urls[$ls_lang] = $actual_back_url;
  }
  // Compare both.
  $diff = array_diff_assoc($correct_ls_back_urls, $actual_ls_back_urls);
  if (empty($diff)) {
    return TRUE;
  }
  else {
    return FALSE;
  }
}

/**
 * Find the Drupal answer entity ID for a LS token.
 *
 * Save it if synchronization is required.
 *
 * @param $ls_sid
 *   A numeric, the LS survey ID. Default is NULL.
 * @param $ls_token
 *  A string, the LS token value.
 *
 * @return
 *   An array, keys are 'dp_aid' with a numeric result if finded (or FALSE),
 *   'need_sync' to FALSE or TRUE if the answer entity has been created/updated.
 */
function limesurvey_sync_api_response_retrieve_dp_aid_from_token($ls_sid, $ls_token) {
  // Look for an answer with this token
  module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_answer');
  $row_answer = limesurvey_sync_ls_api_answer_retrieve_row_from_token($ls_sid, $ls_token);
  $dp_aid = $dp_answer_need_sync = $exist_on_ls = $exist_on_dp = $ls_submitdate = FALSE;
  if (!empty($row_answer)) {
    // Answer with this token does exists on LS.
    $exist_on_ls = TRUE;
    $ls_id = $row_answer['id'];
    // Look for an answer entity with this id.
    $answer_entityname = limesurvey_sync_api_entityname('answer', $ls_sid);
    $answer_table = limesurvey_sync_api_tablename('answer', $ls_sid);
    $dp_aid = db_query('SELECT dp_aid FROM {' . $answer_table . '} WHERE ls_id = :ls_id ORDER BY changed DESC', array(':ls_id' => $ls_id))->FetchField();
    if (!empty($dp_aid)) {
      $exist_on_dp = TRUE;
      $answer_entities = entity_load($answer_entityname, array($dp_aid));
      $answer_entity = $answer_entities[$dp_aid];
    }
    else {
      // create the drupal answer entity.
      $exist_on_dp = FALSE;
      $answer_entity = entity_create($answer_entityname, array('ls_id' => $ls_id));
    }
    // Synchronize it without saving it.
    $synchronize_set = entity_get_controller($answer_entityname)->ls_synchronize($answer_entity);
    if (!empty($synchronize_set['need_sync']) || !empty($synchronize_set['need_import'])) {
      // The corresponding answer has changed on LS datas (need to sync or import).
      // Save/import it.
      entity_save($answer_entityname, $answer_entity);
      // Retrieve the Drupal answer entity ID value
      $dp_aid = $answer_entity->dp_aid;
      $dp_answer_need_sync = TRUE;
    }
    $ls_submitdate = $answer_entity->ls_submitdate;
  }
  return array(
    'dp_aid' => $dp_aid,
    'exist_on_ls' => $exist_on_ls, // TRUE or FALSE
    'exist_on_dp' => $exist_on_dp, // TRUE or FALSE
    'need_sync' => $dp_answer_need_sync, // TRUE or FALSE
    'ls_submitdate' => $ls_submitdate,
    //'answer_entity' => $answer_entity,
  );
}

/**
 * Back url from LS access function.
 *
 * Manage iframe child page ($dp_rid is NULL) and the DP page ($dp_rid is
 * provided).
 *
 * @param $survey
 *   A survey entity object.
 * @param $dp_rid
 *   A numeric, the response entity primary key. Default is NULL.
 *
 * @return
 *   A boolean, TRUE if access granted.
 */
function limesurvey_sync_api_response_back_from_ls_access($survey, $dp_rid = NULL) {
  if (!is_object($survey) || empty($survey->ls_entity_type) || $survey->ls_entity_type != 'survey') {
    return FALSE;
  }

  if (!empty($dp_rid)) {
    // Come back from the iframe.
    if (!is_numeric($dp_rid) || $dp_rid <= 0) {
      return FALSE;
    }
    // Check if the previous page was The drupal editing page.
    $http_referer = $_SERVER['HTTP_REFERER'];
    $url_extract = @parse_url($http_referer);

    if (!isset($url_extract['scheme']) || !isset($url_extract['host']) || !isset($url_extract['path'])) {
      return FALSE;
    }
    $port = (!empty($url_extract['port'])) ? ':' . $url_extract['port'] : '';
    $referrer_base_url = drupal_strtolower(trim($url_extract['scheme'] . '://' . $url_extract['host'] . $port . $url_extract['path']));
    $expected_url = drupal_strtolower(trim(url("lsform/$dp_rid", array('absolute' => TRUE))));
    $ls_referer = (strpos($referrer_base_url, $expected_url) === 0);
    if (!$ls_referer) {
      return FALSE;
    }

    $response_entityname = limesurvey_sync_api_entityname('response');
    $response = entity_load($response_entityname, array($dp_rid));
    if (!$response) {
      return FALSE;
    }
  }
  else {
    // Check if the previous page was LS.
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_answer');
    $extract = limesurvey_sync_ls_api_answer_extract_referer();
    if (!$extract) {
      return FALSE;
    }
  }
  // Don't check the permissions here.
  return TRUE;
}

/**
 * Iframe form page access function.
 *
 * @param $dp_rid
 *   A numeric, the response entity primary key. Default is NULL.
 *
 * @return
 *   A boolean, TRUE if access granted.
 */
function limesurvey_sync_invitation_fill_survey_access($dp_rid) {
  if (empty($dp_rid) || !is_numeric($dp_rid) || $dp_rid <= 0) {
    return FALSE;
  }
  $response_entityname = limesurvey_sync_api_entityname('response');
  $responses = entity_load($response_entityname, array($dp_rid));
  if (empty($responses)) {
    return FALSE;
  }
  $response = $responses[$dp_rid];
  //check about permissions
  if (!entity_access('add', $response_entityname, $response) && !entity_access('update', $response_entityname, $response)) {
    return FALSE;
  }
  // Check if the survey is open.
  $survey = limesurvey_sync_survey_get_surveys($response->type);
  if (!$survey) {
    return FALSE;
  }
  // Assume the survey is synchronzed.
  if ($survey->lss_status != LIMESURVEY_SYNC_SURVEY_OPEN) {
    return FALSE;
  }
  return TRUE;
}

function limesurvey_sync_api_response_tab($entity_type, $survey_type) {

  $survey = limesurvey_sync_survey_get_surveys($survey_type);
  if (entity_access('view', $entity_type, $survey)) {
    return entity_view($entity_type, array($survey));
  }
  else {
    return array(
      '#markup' => t('Administration for the survey @label', array(
        '@label' => $survey->label,
      )),
    );
  }
}

/**
 * Inspired by the _node_add_access() core function
 */
function _limesurvey_sync_response_add_access($survey_type = NULL) {
  if (empty($survey_type)) {
    $types = limesurvey_sync_survey_get_surveys();
    $iterate = $types;
  }
  else {
    // create a dummy object with $survey_type
    $fake_survey = (object) array('type' => $survey_type);
    $iterate = array($fake_survey);
  }
  $entity_type = 'limesurvey_sync_response';
  foreach ($iterate as $type) {
    $entity = entity_create($entity_type, array('type' => $type->type));
    if (entity_access('add', 'limesurvey_sync_response', $entity)) {
      return TRUE;
    }
  }
  if (user_access('administer limesurvey sync survey')) {
    // There are no content types defined that the user has permission to create,
    // but the user does have the permission to administer the content types, so
    // grant them access to the page anyway.
    return TRUE;
  }

  return FALSE;
}

/**
 * Inspired by the node_add() core function
 */
function limesurvey_sync_response_add($survey_type) {
  // I d'ont use directly entity_ui_get_form() because the 'type' need to be filled.
  $entity_type = 'limesurvey_sync_response';
  $op = 'add';
  $form_state = array();
  $entity = entity_create($entity_type, array('type' => $survey_type));
  return entity_ui_get_form($entity_type, $entity, $op, $form_state);
}

/**
 * Inspired by the node_add_page() core function
 */
function limesurvey_sync_response_add_page() {
  // Note : use the 'node_add_list' theme.
  module_load_include('inc', 'node', 'node.pages');
  return node_add_page();
}

/**
 * Generates the LimeSurvey response editing form.
 */
function limesurvey_sync_response_form($form, &$form_state, $entity, $op = 'edit') {

  if ($op == 'clone') {
    $entity->label .= ' (cloned)';
    $entity->type = '';
  }


  $form['#limesurvey_sync_response'] = $entity;

  $survey = limesurvey_sync_survey_get_surveys($entity->type);
  if (!empty($survey->help)) {
    // @TODO : add get_translation.
    $form['guidelines'] = array(
      '#type' => 'markup',
      '#markup' => filter_xss($survey->help),
      '#weight' => -10,
      '#prefix' => '<p>',
      '#suffix' => '</p>',
    );
  }

  $options = variable_get("limesurvey_sync_survey_{$entity->ls_sid}_options", array());
  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    //'#description' => $edit_descrition,
    '#default_value' => ((!empty($entity->title) ? $entity->title : limesurvey_sync_api_response_create_title($entity->ls_sid))),
    '#disabled' => ((!empty($options['ls_disabled_title'])) ? 1 : 0),
    // Displayed at the top of the entity edit form.
    '#weight' => -5,
  );

  global $user;
  $form['uid'] = array(
    '#type' => 'value',
    '#value' => empty($entity->is_new) ? $survey->uid : $user->uid,
  );


  $answer_field_name = limesurvey_sync_api_et_fieldname('answer', $entity->ls_sid);
  $token_field_name = limesurvey_sync_api_et_fieldname('token', $entity->ls_sid);
  // Load the survey type
  $survey = limesurvey_sync_survey_get_surveys($entity->type);
  if ($survey->lss_status != LIMESURVEY_SYNC_SURVEY_OPEN) {
    // the survey is not open.
    $check_edition = 0;
    $disabled_edition = 1;
    $edit_descrition = t('The survey is not available.');
  }
  else {
    // Check is the answer is editable.
    // Load the answer.
    if (!empty($entity->$answer_field_name) && !empty($entity->{$answer_field_name}[LANGUAGE_NONE][0]['target_id'])) {
      $answer_entity_name = limesurvey_sync_api_entityname('answer', $entity->ls_sid);
      $answer_id = $entity->{$answer_field_name}[LANGUAGE_NONE][0]['target_id'];
      $answer = entity_load($answer_entity_name, array($answer_id));
      if ($answer && $answer[$answer_id]->lss_status == LIMESURVEY_SYNC_ANSWER_COMPLETED) {
        // The answer is completed and not editable.
        $check_edition = 0;
        $disabled_edition = 1;
        $edit_descrition = t('The answer is completed and can not be edited.');
      }
      else {
        // @TODO : if 'editing' stage : check if someone else already editing it.
        $check_edition = !empty($entity->is_new) ? 1 : 0;
        $disabled_edition = 0;
        $edit_descrition = t('To edit the respond, check here and submit, you will be then redirected to the survey form.');
      }
    }
    else {
      // There is associated answer.
      $check_edition = !empty($entity->is_new) ? 1 : 0;
      $disabled_edition = 0;
      $edit_descrition = t('To edit the respond, check here and submit, you will be then redirected to the survey form.');
    }
  }

  $form['edit_answer'] = array(
    '#type' => 'checkbox',
    '#title' => t('Edit the survey respond'),
    '#description' => $edit_descrition,
    '#default_value' => $check_edition,
    '#return_value' => 1,
    '#disabled' => $disabled_edition,
    // Displayed at the top of the entity edit form.
    '#weight' => -5,
  );
  // Hide the checkbox for new answers : always go to the LS form.
  if (!empty($entity->is_new)) {
    $form['edit_answer']['#type'] = 'hidden';
    $form['edit_answer']['#value'] = $form['edit_answer']['#default_value'];
  }

  // Set the ls_language value.
  module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
  $options_ls_lang = limesurvey_sync_api_get_ls_langs($entity->ls_sid);
  $default_ls_lang = (!empty($entity->ls_language)) ? $entity->ls_language : limesurvey_sync_api_set_ls_lang_from_current_dp_lang($entity->ls_sid);
  if (count($options_ls_lang) <= 1) {
    // Only one language, don't display it.
    $form['ls_language'] = array(
      '#type' => 'value',
      '#value' => $default_ls_lang,
    );
  }
  else {
    // Propose to choose.
    $form['ls_language'] = array(
      '#type' => 'select',
      '#title' => t('Select a language for the form'),
      '#options' => $options_ls_lang,
      '#default_value' => $default_ls_lang,
      '#states' => array('visible' => array(':input[name="edit_answer"]' => array('value' => 1))),
      '#access' => ((empty($options['ls_hide_lang'])) ? 1 : 0),
    );
  }

  // Set the Drupal language
  global $language;
  $default_dp_language = (!empty($entity->language)) ? $entity->language : ((!empty($language->language)) ? $language->language : LANGUAGE_NONE);
  if (!drupal_multilingual() || !module_exists('locale')) {
    // Only one language, don't display it.
    $default = language_default();
    $form['language'] = array(
      '#type' => 'value',
      '#value' => $default_dp_language,
    );
  }
  else {
    $languages = language_list('enabled');
    $languages = $languages[1];
    $names = array();
    foreach ($languages as $langcode => $item) {
      $names[$langcode] = t($item->name);
    }
    $form['language'] = array(
      '#type' => 'select',
      '#title' => t('Select a displayed language'),
      '#options' => $names,
      '#default_value' => $default_dp_language,
      '#access' => ((empty($options['ls_hide_lang'])) ? 1 : 0),
    );
  }

  $form['actions'] = array(
    '#type' => 'actions',
  );
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => empty($entity->is_new) ? t('Update') : t('Save'),
    '#weight' => 40,
  );

  if (TRUE || !$entity->isLocked() && $op != 'add' && $op != 'clone') {
    $form['actions']['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
      '#weight' => 45,
      '#limit_validation_errors' => array(),
      '#submit' => array(
        'limesurvey_sync_response_form_submit_delete',
      ),
    );
  }
  // Load the entity_reference field (and maybe more custon fields).
  field_attach_form('limesurvey_sync_response', $entity, $form, $form_state);

  // Hide the entity reference field : their values will be set on saving.
  if (isset($form[$answer_field_name])) {
    $form[$answer_field_name]['#disabled'] = TRUE;
    $form[$answer_field_name]['#weight'] = 10;
    if (!empty($entity->is_new)) {
      $form[$answer_field_name]['#access'] = FALSE;
    }
  }
  if (isset($form[$token_field_name])) {
    $form[$token_field_name]['#disabled'] = TRUE;
    $form[$token_field_name]['#weight'] = 11;
    if (!empty($entity->is_new)) {
      $form[$token_field_name]['#access'] = FALSE;
    }
  }

  // @TODO : add vertical tab for log, revision, published, sticky, etc...

  return $form;
}

/**
 * Form API validate callback for the limesurvey_response form.
 */
function limesurvey_sync_response_form_validate($form, &$form_state) {

  // @TODO : check about the edit answer checkbox for server control.
  // @TODO : check if ls_lang has changed with edit answer to TRUE (disallow).
}

/**
 * Form API submit callback for the type form.
 */
function limesurvey_sync_response_form_submit(&$form, &$form_state) {

  $entity = entity_ui_form_submit_build_entity($form, $form_state);
  // Delete remaining garbage datas from entity_ui_form_submit_build_entity().
  $garbage = array(
    'submit',
    'form_build_id',
    'form_token',
    'form_id',
    'op',
    'delete',
  );

  foreach ($garbage as $garbage_field) {
    if (isset($entity->$garbage_field)) {
      unset($entity->$garbage_field);
    }
  }

  // The LS answer is going to be edited :Handle it here.
  if ($form_state['values']['edit_answer']) {
    // Set a flag : dp_start_answer
    $entity->dp_start_answer = TRUE;
  }

  // Save the entity response.
  $entity->save();
  drupal_set_message(t('The LimeSurvey response has been saved.'));
  $form_state['redirect'] = current(entity_uri($form_state['entity_type'], $entity));

  // Now the token entity should have been created or edited during the save process.
  // Retrieve the token value if any.
  if ($form_state['values']['edit_answer']) {
    // redirect to the LS page for filling the survey.
    $variable_name = "limesurvey_sync_survey_{$entity->ls_sid}_options";
    $ls_options = variable_get($variable_name, array());
    module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_invitation.options');
    $options = limesurvey_sync_invitation_options_default($entity->ls_sid, $ls_options);
    module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_invitation.form');
    $display_mode = limesurvey_sync_invitation_fix_display_mode($options['ls_redirect']);
    switch ($display_mode) {
      case 'iframe':
      case 'colorbox':
      case 'lightbox2':
      case 'modalframe':
        //case 'link':
        $redirect = "lsform/{$entity->dp_rid}";
        break;

      case 'redirect':
        //case 'form':
      default:
        // Set the invitation.
        $redirect = $entity->filling_url;
    }
    $form_state['redirect'] = $redirect;
  }
}

/**
 * Form API submit callback for the delete button.
 */
function limesurvey_sync_response_form_submit_delete(&$form, &$form_state) {

  $form_state['redirect'] = current(entity_uri('limesurvey_sync_response', $form_state['limesurvey_sync_response'])) . '/delete';
}

function limesurvey_sync_api_response_lss_status_list() {
  $status = array(
    LIMESURVEY_SYNC_RESPONSE_END,
    LIMESURVEY_SYNC_RESPONSE_START,
    LIMESURVEY_SYNC_RESPONSE_OUTSIDE_DP,
  );
  $list = array();
  module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
  foreach ($status as $value) {
    $list[$value] = limesurvey_sync_api_display_field('response', 'lss_status', $value);
  }
  return $list;
}

/**
 * Return a drupal LS field for display.
 *
 * As an example, return timestamp as formated date, LS language code as human
 * language, etc...
 *
 * @param $entity_type
 *   A string. The entity type (ex : 'limesurvey_sync_answer_xxxxx').
 * @param $field
 *   A string. The Drupal ls_field to display (ex : 'startlanguage').
 * @param $value
 *   A string or any. corresponding Drupal LS field to display (ex:
 *   'de-informal').
 *
 * @return
 *   A string. The value ready to be displayed.
 */
function limesurvey_sync_response_display_field($field, $value, $return = 'value', $ls_sid = FALSE) {
  if ($field == 'lss_status') {
    switch ($value) {

      case LIMESURVEY_SYNC_RESPONSE_END:
        switch ($return) {
          case 'help':
            return t("The response was ended from Drupal.");
          case 'css':
            return 'limesurvey-sync-field-lss-status-response-end';
          default:
            return t('Ended from Drupal');
        }

      case LIMESURVEY_SYNC_RESPONSE_START:
        switch ($return) {
          case 'help':
            return t('The response was started from Drupal.');
          case 'css':
            return 'limesurvey-sync-field-lss-status-response-start';
          default:
            return t('Started from Drupal');
        }

      case LIMESURVEY_SYNC_RESPONSE_OUTSIDE_DP:
        switch ($return) {
          case 'help':
            return t('The response was not started from Drupal.');
          case 'css':
            return 'limesurvey-sync-field-lss-status-response-outside-dp';
          default:
            return t('Not started from Drupal');
        }
    }
  }

  if (in_array($field, array(
    'dp_editing',
    'dp_back',
    'ls_language',
    'type',
    'title',
    'uid',
  ))) {
    if (in_array($return, array(
      'help',
      'css',
    ))) {
      return '';
      // No need to explicit css or help.
    }
    else {
      switch ($field) {
        case 'dp_editing':
          if ($value == '') {
            return t('Not started from Drupal');
          }
          return format_date($value);
          break;
        case 'dp_back':
          if ($value == '') {
            return t('Not ended to Drupal');
          }
          return format_date($value);
          break;
        case 'ls_language':
          return limesurvey_sync_api_display_field('token', 'ls_lang', $value, $return, $ls_sid);
          break;
        case 'type':
          $survey = limesurvey_sync_survey_get_surveys($value);
          if ($survey) {
            if (entity_access('view', 'limesurvey_sync_survey', $survey)) {
              $url = current(entity_uri('limesurvey_sync_survey', $survey));
              return l($survey->label, $url);
            }
            else {
              return $survey->label;
            }
            $link = current(entity_url('limesurvey_sync_survey', $survey));
            entity_access('view', 'limesurvey_sync_survey', $survey);
          }
          else {
            t('The survey has been deleted on Drupal.');
          }
          break;
        case 'title':
          return check_plain($value);
          break;
        case 'uid':
          $account = user_load($value);
          return check_plain(format_username($account));
          break;
      }
    }

  }
  return $value;
}

/**
 * Provide a automatic generated title for responses.
 *
 * This function generates unique titles, by generating a random number. Title
 * can be generate adding a number or a letter or comparing it with a other
 * title.
 *
 * @param $ls_sid
 *   A numeric. The LS survey id.
 * @param $method
 *   A string. Allow to choose how the title is going to be formated. Possible
 *   values are : '' (empty) by default, 'add_letter', 'change_prefix', 'no_title'. The
 *   ls_answ module only use the default method. Other method are for other
 *   LimeSurvey synchronization modules (ls_diff as a example).
 * @param $ref_title
 *   A string. Title to be compared with the generated title. Default is empty.
 * @param $ref_prefix
 *   A string. Prefix of the $ref_title. Default is empty.
 * @param $prefix
 *   A string. Prefix of the generated title. Default is empty.
 *
 * @return
 *   A string. The unique generated title or FALSE if the option variable is not
 *   set.
 */
function limesurvey_sync_api_response_create_title($ls_sid, $method = '', $ref_title = '', $ref_prefix = '', $prefix = '') {

  $thousands_separator = '&#8239;';
  // "narrow non breaking space", cf http://david.latapie.name/?p=4474
  // @TODO it may be interesting to use "narrow non breaking space" instead of
  // a simple space when formating numbers.

  if (!empty($ls_sid)) {
    $type = limesurvey_sync_api_typename('survey', $ls_sid);
    $variable_name = "limesurvey_sync_survey_{$ls_sid}_options";
  }
  else {
    $type = FALSE;
    $variable_name = "limesurvey_sync_survey_default_options";
  }
  $options = variable_get($variable_name, array());
  if (empty($options)) {
    return FALSE;
  }
  $numbermin = $options['ls_suffix_min'];
  $numbermax = $options['ls_suffix_max'];
  if (empty($prefix)) {
    $prefix = $options['ls_prefix'];
  }
  $method_number = $options['ls_suffix_method'];
  if (empty($options['ls_prefix'])) {
    $method == 'no_title';
  }
  if ($method == 'no_title') {
    return '';
  }
  $title_list = array();
  $ref_title_number = '';
  $i = 0;
  $j = 0;
  if ($ref_title != '' && preg_match('/([ 0-9]+)$/', trim($ref_title), $matches)) {
    $ref_title_number = str_replace(' ', '', $matches[0]);
  }

  // Select the pattern for extrating the number or the letter included into nodes title.
  $pattern_title = '/([ 0-9]+)$/';
  if ($method == 'add_letter') {
    $add_letter_prefix_title = str_replace($ref_prefix, $prefix, $ref_title);
    $pattern_title = "/^" . $add_letter_prefix_title . "[ ]?([A-Za-z]+)$/";
  }

  // Extracts all existings titles.
  // Order by title may improve the search via the array_key_exist() function.
  $title_list = array();
  if ($type) {
    $response_tablename = limesurvey_sync_api_tablename('response');
    $query_title = db_query('SELECT title FROM {' . $response_tablename . '} WHERE type = :type ORDER BY title', array(':type' => $type));

    // Load all numbers|letters into the $title_list variable, as keys.
    while ($result_title = $query_title->fetchAssoc()) {
      if (preg_match($pattern_title, trim($result_title['title']), $matches)) {
        $title_list[str_replace(' ', '', $matches[1])] = 1;
        $j++;
      }
    }
  }
  switch ($method) {
    case 'add_letter':
      if ($ref_title_number != '') {
        // Generate a unused unique letter.
        // "i", "o" and "l" letters are ignored because they may be confusing.
        $pattern = "abcdefghjkmnpqrstuvwxyz";
        $length = 1;
        $isvalidletter = FALSE;
        $j = 0;
        while ($isvalidletter == FALSE) {
          $newletter = drupal_strtoupper(limesurvey_sync_api_response_randomkey($length, $pattern));
          if (!isset($title_list[$newletter])) {
            $isvalidletter = TRUE;
          }
          $j++;
          // The system can adapt itself if the 24 letters are sold out.
          $length = ($j <= 24) ? 1 : ceil(($j / 24));
        }
        return ($add_letter_prefix_title . " " . $newletter);
        break;
      }

    case 'change_prefix':
      if ($ref_title_number != '') {
        return (str_replace($ref_prefix, $prefix, $ref_title));
      }
      break;

    default:
      if ($method_number == 'random') {
        // Generate a unique random number.
        $isvalidnumber = FALSE;
        while ($isvalidnumber == FALSE) {
          $newnumber = mt_rand($numbermin, $numbermax);
          if (!isset($title_list[$newnumber])) {
            $isvalidnumber = TRUE;
          }
          $i++;
          // The system can adapt itself if 75% of the interval are sold out.
          if ($i > (($numbermax - $numbermin) * 0.75)) {
            $numbermin = $options['ls_suffix_min'] = $numbermax;
            $numbermax = $options['ls_suffix_max'] = (int) (str_repeat('9', (drupal_strlen($i) + 1)));
            // Update the answers content type options.
            variable_set($variable_name, $options);
          }
        }
      }
      else {
        // Increment number
        ksort($title_list);
        $all_numbers = array_keys($title_list);
        $newnumber = ((count($all_numbers) > 0) ? max($all_numbers) : 0) + 1;
      }
      // Generate the title as "Answer #1 254" (for example).
      return ($prefix . number_format($newnumber, 0, ',', ' '));
  }
}

/**
 * Generate a random value.
 *
 * This function generates unique titles, by generating a random number. Title
 * can be generate adding a number or a letter or comparing it with a other
 * title.
 *
 * @param $length
 *   A numeric. The lenth of the random value.
 * @param $pattern
 *   A string. The caractere to use for generating the random sting. Default
 *   is "23456789abcdefghijkmnpqrstuvwxyz".
 *
 * @return
 *   A string. The unique generated key.
 */
function limesurvey_sync_api_response_randomkey($length, $pattern = "23456789abcdefghijkmnpqrstuvwxyz") {
  $patternlength = drupal_strlen($pattern) - 1;
  for ($i = 0; $i < $length; $i++) {
    if (isset($key)) {
      $key .= $pattern{rand(0, $patternlength)};
    }
    else {
      $key = $pattern{rand(0, $patternlength)};
    }
  }
  return $key;
}

/**
 * Implements hook_menu_local_tasks_alter().
 *
 * Add the 'add entity' to the admin menu.
 */
function limesurvey_sync_menu_local_tasks_alter(&$data, $router_item, $root_path) {
  // Add action link to 'node/add' on 'admin/content' page.
  if ($root_path == 'admin/content/limesurvey') {
    $item = menu_get_item(limesurvey_sync_api_root_uri('response', '', FALSE) . '/add');
    if ($item['access']) {
      $data['actions']['output'][] = array(
        '#theme' => 'menu_local_action',
        '#link' => $item,
      );
    }
  }
}


/**
 * Implements hook_admin_menu_map().
 *
 * Provide expansion arguments for dynamic menu items, for each survey.
 */
function limesurvey_sync_admin_menu_map() {
  // Expand survey types below admin > Structure > LimeSurvey > Surveys.
  // The key denotes the dynamic path to expand to multiple menu items.
  // Below returns : 'admin/structure/limesurvey/surveys/manage/%limesurvey_sync_survey'.
  if (!user_access('administer limesurvey sync survey')) {
    return;
  }
  $survey_entity_info = entity_get_info('limesurvey_sync_survey');
  $root_path = $survey_entity_info['admin ui']['path'];
  $wilcard = isset($survey_entity_info['admin ui']['menu wildcard']) ? $survey_entity_info['admin ui']['menu wildcard'] : '%entity_object';
  $list_surveys_url = array();
  foreach (limesurvey_sync_survey_get_surveys() as $type => $survey) {
    $list_surveys_url[] = strtr($type, array('_' => '-'));
  }
  $map["{$root_path}/manage/{$wilcard}"] = array(
    // Link generated items directly to the "Surveys" item.
    'parent' => $root_path,
    // Create expansion arguments for the '%entity_object' placeholder.
    'arguments' => array(
      array(
        $wilcard => $list_surveys_url,
      ),
    ),
  );
  return $map;
}

