<?php

/**
 * @file
 * Functions and classes for the tokens entities.
 */

define('LIMESURVEY_SYNC_TOKEN_COMPLETED_CLOSED', 1);
define('LIMESURVEY_SYNC_TOKEN_COMPLETED_REUSABLE', 2);
define('LIMESURVEY_SYNC_TOKEN_OPEN', 3);
define('LIMESURVEY_SYNC_TOKEN_SOON', 4);
define('LIMESURVEY_SYNC_TOKEN_EXPIRED', 5);
define('LIMESURVEY_SYNC_TOKEN_CLOSED', 6);
define('LIMESURVEY_SYNC_TOKEN_UNKNOWN', 7);

/**
 * Class managing building token entities.
 *
 * Building entity_info, entity_property_info, schema, entity default values and
 * updating entity table if needed. Take care of Drupal fields, LimeSurvey fixed
 * fields and Limesurver variables fields.
 */
class LimesurveySyncToken extends LimesurveySync {
  public $ls_sid;
  public $ls_entity_type = 'token';
}

/**
 * The Controller for token entities
 */
class LimesurveySyncTokenController extends LimesurveySyncController {
  public $ls_entity_type = 'token';

  /**
   * Builds a structured array representing the entity's content.
   *
   * The content built for the entity will vary depending on the $view_mode
   * parameter.
   *
   * @param $entity
   *   An entity object.
   * @param $view_mode
   *   View mode, e.g. 'full', 'teaser'...
   * @param $langcode
   *   (optional) A language code to use for rendering. Defaults to the global
   *   content language of the current request.
   *
   * @return
   *   The renderable array.
   */
  public function buildContent($entity, $view_mode = 'full', $langcode = NULL, $content = array()) {

    module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
    $render_admin = limesurvey_sync_api_fields_render($this->ls_entity_type, $entity, $this->ls_sid);
    $content['token_admin_datas'] = $render_admin + array(
      '#theme' => 'limesurvey_sync_admin_data_inline',
    );
    $build = parent::buildContent($entity, $view_mode, $langcode, $content);
    return $build;
  }

  /**
   * Provide the token entity status.
   *
   * Status are 'completed_closed', 'completed reusale', 'open', 'soon',
   * 'expired' and 'unknown'.
   *
   * @param $entity
   *   The entity object.
   *
   * @return
   *   A numeric defined as a constant corresponding to a defined LS stage.
   */
  protected function lss_status($entity) {

    if (is_array($entity)) {
      $entity = (object) $entity;
    }
    if (isset($entity->ls_completed) && $entity->ls_completed !== NULL) {
      return ($entity->ls_usesleft < 1) ? LIMESURVEY_SYNC_TOKEN_COMPLETED_CLOSED : LIMESURVEY_SYNC_TOKEN_COMPLETED_REUSABLE;
    }
    elseif ($entity->ls_usesleft < 1) {
      return LIMESURVEY_SYNC_TOKEN_CLOSED;
    }
    elseif (($entity->ls_validfrom === NULL || $entity->ls_validfrom <= REQUEST_TIME) && ($entity->ls_validuntil === NULL || $entity->ls_validuntil > REQUEST_TIME)) {
      return LIMESURVEY_SYNC_TOKEN_OPEN;
    }
    elseif ($entity->ls_validfrom !== NULL && $entity->ls_validfrom > REQUEST_TIME) {
      return LIMESURVEY_SYNC_TOKEN_SOON;
    }
    elseif ($entity->ls_validuntil !== NULL && $entity->ls_validuntil < REQUEST_TIME) {
      return LIMESURVEY_SYNC_TOKEN_EXPIRED;
    }
    return LIMESURVEY_SYNC_TOKEN_UNKNOWN;
  }

  /**
   * Permanently saves the given entity, synchronizing it.
   *
   * If the 'activate_token' is set to TRUE : create or update token in order to
   * access to the filling answer page in LS.
   *
   * @param $entity
   *   The entity to save.
   *
   * @return
   *   SAVED_NEW or SAVED_UPDATED is returned depending on the operation
   *   performed.
   */
  public function save($entity, DatabaseTransaction $transaction = NULL) {

    // Revisions should not be activated.
    $is_default_revision = entity_revision_is_default($this->entityType, $entity);
    if ($is_default_revision && !empty($entity->activate_token)) {
      // Edit the LS token entry in order to make it available.
      $tochange = limesurvey_sync_ls_api_token_activate_token($entity);
      // Apply thoses changes on LS.
      module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_fields');
      $result = limesurvey_sync_ls_api_edit_row($entity, $tochange);
      if (is_numeric($result)) {
        // A new entry has been inserted : retrieve the tid value.
        $entity->ls_tid = $result;
      }
      if ($result) {
        // The create/update is successfull.
        // Apply the changes to the entity object.
        foreach ($tochange as $key => $value) {
          $entity->$key = $value;
        }
        // Update lss_sync and lss_status according to thoses changes.
        $entity->lss_sync = LIMESURVEY_SYNC_SYNCHRONIZED;
        $entity->lss_status = LIMESURVEY_SYNC_TOKEN_OPEN;
        // Set a log message
        $entity->log = t('Token invitation activated.');
      }

    }
    parent::save($entity, $transaction);
    // Thanks to the token synchronization process, the entity will be updated
    // with the new lss_status.
  }

}

/**
 * The UI Controller for token entities
 */
class LimesurveySyncTokenUIController extends LimesurveySyncUIController {
  public $ls_entity_type = 'token';
  protected $menu_weight = -1;
  protected $ls_sub_entity = TRUE;
  protected $disallow_add = TRUE;
  protected $disallow_edit = TRUE;
}

/**
 * The entity info class for token entities.
 */
class LimesurveySyncInfosToken extends LimesurveySyncInfos {
  public $ls_entity_type = 'token';
  //public $root_ui = 'limesurvey/response';
  public $root_admin_ui = 'admin/structure/limesurvey/surveys';
  protected $revision = TRUE;
  protected $primary_key = 'dp_tid';
  protected $date_fields = TRUE;
  protected $ls_sync_field = TRUE;
  protected $ls_key = 'ls_tid';
  protected $ls_date_key = 'ls_completed';
  protected $sub_entity = TRUE;
  protected $label = 'Token';
  protected $plural_label = 'Tokens';
  protected $exportable = TRUE;

  protected function schema_dp_table($sid = NULL) {

    return array(
      'description' => 'Stores token invitations from the LimeSurvey surveys #' . $sid . '.',
    );
  }

  protected function create_dp_fields_default_values($sid = NULL) {

    return array(
      'lss_status' => 1,
      'ls_editing' => 1,
    );
  }

  protected function schema_dp_fields($sid = NULL, $revision = FALSE) {

    $schema = parent::schema_dp_fields($sid, $revision);
    $schema['lss_status'] = array(
      'type' => 'int',
      'unsigned' => TRUE,
      'not NULL' => TRUE,
      'default' => 1,
      'size' => 'tiny',
      'description' => 'Status of this token from the sid #' . $sid . ' (1 and 2 for completed, 3 for open, and 4 to 7 for closed).',
    );
    return $schema;
  }

  protected function schema_ls_fields_known($field) {
    // Note : by default on drupal 'unsight' -> FALSE and 'not null' -> FALSE.
    $fields = array(
      'ls_tid' => array(
        'description' => 'Primary Key of the token table',
        'type' => 'int',
        'unsigned' => TRUE,
        'not null' => FALSE,
        'default' => NULL,
      ),
      // Defaults to NULL in order to avoid a brief period of potential
      // deadlocks on the index.
      'ls_participant_id' => array(
        'type' => 'varchar',
        'length' => 50,
        'default' => NULL,
        'description' => 'corresponding participant ID in the LimeSurvey participant panel.',
      ),
      'ls_firstname' => array(
        'type' => 'varchar',
        'length' => 40,
        'default' => NULL,
        'description' => 'First name.',
      ),
      'ls_lastname' => array(
        'type' => 'varchar',
        'length' => 40,
        'default' => NULL,
        'description' => 'Last name.',
      ),
      'ls_email' => array(
        'type' => 'text',
        'default' => NULL,
        'description' => "Email, 'OK' for a valid email.",
      ),
      'ls_emailstatus' => array(
        'type' => 'text',
        'default' => NULL,
        'description' => 'Email status.',
      ),
      'ls_token' => array(
        'type' => 'varchar',
        'length' => 35,
        'default' => NULL,
        'description' => 'Invitation token.',
      ),
      'ls_language' => array(
        'type' => 'varchar',
        'length' => 25,
        'default' => NULL,
        'description' => 'The LimeSurvey language.',
      ),
      'ls_blacklisted' => array(
        'type' => 'varchar',
        'length' => 17,
        'default' => NULL,
        'description' => 'Blacklisted entry.',
      ),
      'ls_sent' => array(
        'type' => 'varchar',
        'length' => 17,
        'default' => 'N',
        'description' => 'email sent.',
      ),
      'ls_remindersent' => array(
        'type' => 'varchar',
        'length' => 17,
        'default' => 'N',
        'description' => 'Reminder sent.',
      ),
      'ls_remindercount' => array(
        'type' => 'int',
        'unsigned' => TRUE,
        'default' => 0,
        'description' => 'Reminder count.',
      ),
      'ls_completed' => array(
        'type' => 'varchar',
        'length' => 17,
        'default' => 'N',
        'description' => "Completed status, 'N' for uncompleted.",
      ),
      'ls_usesleft' => array(
        'type' => 'int',
        'default' => 1,
        'description' => 'number of times a token can be used, default to one.',
      ),
      // Need to convert those fields values from datetime to timestamp.
      'ls_validfrom' => array(
        'type' => 'int',
        'default' => NULL,
        'description' => 'Token invitation valid from this date, unlimited if NULL.',
      ),
      'ls_validuntil' => array(
        'type' => 'int',
        'default' => NULL,
        'description' => 'Token invitation valid until this date, unlimited if NULL..',
      ),
      'ls_mpid' => array(
        'type' => 'int',
        'default' => NULL,
        'description' => 'mpid.',
      ),
    );

    return (isset($fields[$field])) ? $fields[$field] : FALSE;
  }

  /**
   * Return the corresponding Drupal database schema of a LimeSurvey field.
   *
   * This fonction handles LimeSurvey token or answer fields that are specific to
   * a particular LimeSurvey survey. For the token table it corresponds to the
   * attribute fields, and for the answer table it corresponds to the questions
   * fields. The LimeSurvey datetime fields will be converted to timestamp on
   * Drupal.
   *
   * @param $type
   *   A string, 'token' or 'answer'.
   * @param $field
   *   A string. The LimeSurvey field name.
   * @param $sid
   *   A numeric. The LimeSurvey survey ID.
   *
   * @return
   *   An array of Drupal schema corresponding to the provided LimeSurvey field or
   *   FALSE if the provided field is not a variable LimeSurvey field.
   */
  protected function schema_ls_fields_variables($field, $sid) {

    $attribute = array(
      'type' => 'varchar',
      'length' => 255,
      'not null' => FALSE,
      'default' => NULL,
    );
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
    $this_survey = limesurvey_sync_survey_properties($sid);
    $attributedescriptions = $this_survey['attributedescriptions'];
    $fields = array();
    foreach ($attributedescriptions as $attribute_field => $datas) {
      $fields[$attribute_field] = $attribute;
      $fields[$attribute_field]['description'] = $datas['description'];
    }

    return (isset($fields[$field])) ? $fields[$field] : FALSE;

  }


  protected function entity_property_info_dp_fields($sid = NULL) {

    $info = parent::entity_property_info_dp_fields($sid);
    $info['lss_status'] = array(
      'label' => t('DP Token status'),
      'description' => t('statut ID for the invitation (1 and 2 for completed, 3 for open, and 4 to 7 for closed).'),
      'type' => 'integer',
      //'validation callback' => 'entity_metadata_validate_integer_positive',
      'schema field' => 'lss_status',
      'options list' => 'limesurvey_sync_api_token_lss_status_list',
    );
    return $info;
  }

  /**
   * Return the corresponding Drupal entity_property_info of a LimeSurvey field.
   *
   * This fonction is used for generating the drupal hook_entity_property_info().
   * The main benefit of this fonction is to display the questions values instead
   * of the fieldname into Drupal Views. Handle common LimeSurvey fields of the
   * token and answer tables.
   *
   * @param $field
   *   A string. The LimeSurvey field name.
   * @param $sid
   *   A numeric. The LimeSurvey survey ID.
   *
   * @return An array of Drupal entity_property_info corresponding to the provided
   *   LimeSurvey field or FALSE if the provided field is not a generic LimeSurvey
   *   field.
   */
  protected function entity_property_info_ls_fields_known($field, $sid = NULL) {
    // Note : by default on drupal 'unsight' -> FALSE and 'not null' -> FALSE.
    $arg = array('%sid' => $sid);
    $fields = array(
      'ls_tid' => array(
        'label' => t('_ID, LimeSurvey token ID'),
        'description' => t('LimeSurvey token ID into the LimeSurvey database.'),
        'type' => 'integer',
        'validation callback' => 'entity_metadata_validate_integer_positive',
        'schema field' => 'ls_tid',
        'required' => TRUE,
      ),
      'ls_participant_id' => array(
        'label' => t('LS admin : Participant LimeSurvey ID'),
        'description' => t('Participant LimeSurvey ID into the LimeSurvey Software.'),
        'type' => 'integer',
        'validation callback' => 'entity_metadata_validate_integer_positive',
        'schema field' => 'ls_participant_id',
      ),
      'ls_firstname' => array(
        'label' => t('LS admin : First name'),
        'description' => t('Token first name (survey#%sid)', $arg),
        'type' => 'text',
        'schema field' => 'ls_firstname',
      ),
      'ls_lastname' => array(
        'label' => t('LS admin : Last name'),
        'description' => t('Token last name (survey#%sid)', $arg),
        'type' => 'text',
        'schema field' => 'ls_lastname',
      ),
      'ls_email' => array(
        'label' => t('LS admin : Email'),
        'description' => t('Token email (survey#%sid)', $arg),
        'type' => 'text',
        'validation callback' => 'valid_email_address',
        'schema field' => 'ls_email',
      ),
      'ls_emailstatus' => array(
        'label' => t('LS admin : Email status'),
        'description' => t('Token email status (survey#%sid)', $arg),
        'type' => 'text',
        'schema field' => 'ls_emailstatus',
      ),
      'ls_token' => array(
        'label' => t('LS admin : Token invitation'),
        'description' => t('Token invitation value (survey#%sid)', $arg),
        'type' => 'token',
        'schema field' => 'ls_token',
      ),
      'ls_language' => array(
        'label' => t('LS admin : LimeSurvey language'),
        'description' => t('LimeSurvey language (survey#%sid)', $arg),
        'type' => 'token',
        'options list' => 'limesurvey_sync_survey_ls_language_list',
        'schema field' => 'ls_language',
      ),
      'ls_blacklisted' => array(
        'label' => t('LS admin : Blacklisted entry'),
        'description' => t('Token blacklisted (survey#%sid)', $arg),
        'type' => 'token',
        'schema field' => 'ls_blacklisted',
      ),
      'ls_sent' => array(
        'label' => t('LS admin : Email sent'),
        'description' => t('Token email sent (survey#%sid)', $arg),
        'type' => 'text',
        'schema field' => 'ls_sent',
      ),
      'ls_remindersent' => array(
        'label' => t('LS admin : Reminder sent'),
        'description' => t('Token reminder sent (survey#%sid)', $arg),
        'type' => 'text',
        'schema field' => 'ls_remindersent',
      ),
      'ls_remindercount' => array(
        'label' => t('LS admin : Reminder count'),
        'description' => t('Token reminder count (survey#%sid)', $arg),
        'type' => 'int',
        'validation callback' => 'entity_metadata_validate_integer_positive',
        'schema field' => 'ls_remindercount',
      ),
      'ls_completed' => array(
        'label' => t('LS admin : Completed'),
        'description' => t('Token completed (survey#%sid)', $arg),
        'type' => 'date',
        'schema field' => 'ls_completed',
      ),
      'ls_usesleft' => array(
        'label' => t('LS admin : Uses Left'),
        'description' => t('Token uses left (survey#%sid)', $arg),
        'type' => 'int',
        'validation callback' => 'entity_metadata_validate_integer_positive',
        'schema field' => 'ls_usesleft',
      ),
      // Need to convert those fields values from datetime to timestamp.
      'ls_validfrom' => array(
        'label' => t('LS admin : Valid from'),
        'description' => t('Token valid from (survey#%sid)', $arg),
        'type' => 'date',
        'schema field' => 'ls_validfrom',
      ),
      'ls_validuntil' => array(
        'label' => t('LS admin : Valid untill'),
        'description' => t('Token valid untill (survey#%sid)', $arg),
        'type' => 'date',
        'schema field' => 'ls_validuntil',
      ),
      'ls_mpid' => array(
        'label' => t('LS admin : Reminder count'),
        'description' => t('Token mpid (survey#%sid)', $arg),
        'type' => 'int',
        'validation callback' => 'entity_metadata_validate_integer_positive',
        'schema field' => 'ls_mpid',
      ),
    );

    return (isset($fields[$field])) ? $fields[$field] : FALSE;
  }

  /**
   * Return the corresponding Drupal entity_property_info of a LimeSurvey field.
   *
   * This fonction is used for generating the drupal hook_entity_property_info().
   * The main benefit of this fonction is to display the questions values instead
   * of the fieldname into Drupal Views. Handle variable LimeSurvey fields of the
   * token and answer tables.
   *
   * @param $field
   *   A string. The LimeSurvey field name.
   * @param $sid
   *   A numeric. The LimeSurvey survey ID.
   *
   * @return
   *   An array of Drupal entity_property_info corresponding to the provided
   *   LimeSurvey field or FALSE if the provided field is not a variable
   *   LimeSurvey field.
   */
  protected function entity_property_info_ls_fields_variables($field, $sid) {
    $attribute = array(
      'type' => 'text',
    );

    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
    $this_survey = limesurvey_sync_survey_properties($sid);
    $attributedescriptions = $this_survey['attributedescriptions'];
    $fields = array();
    foreach ($attributedescriptions as $attribute_field => $datas) {
      $fields[$attribute_field] = $attribute;
      $fields[$attribute_field]['label'] = t('LS data !field : !attribute', array('!attribute' => $datas['description'], '!field' => $attribute_field));
      $fields[$attribute_field]['description'] = t('Token %attribute (survey #%sid)', array('%attribute' => $datas['description'], '%sid' => $sid));
      $fields[$attribute_field]['schema field'] = $attribute_field;
    }
    return (isset($fields[$field])) ? $fields[$field] : FALSE;
  }

  /**
   * Return the corresponding Drupal default value of a LS known field.
   *
   * This fonction is used for creating a new answer or token Drupal entity.
   * Handle common LimeSurvey fields of the token and answer tables.
   *
   * @param $field
   *   A string. The LimeSurvey field name.
   * @param $sid
   *   A numeric. The LimeSurvey survey ID. Default is NULL.
   *
   * @return
   *   A string or numeric or NULL : the Drupal default value for the provided
   *   LimeSurvey field or FALSE if the provided field is not a generic (known)
   *   LimeSurvey field.
   *
   * @see LimesurveySyncInfos::entity_default_values()
   */
  protected function create_ls_fields_default_values_known($field, $sid = NULL) {
    $variable_name = "limesurvey_sync_survey_{$sid}_options";
    $options = variable_get($variable_name, array());

    //$config = limesurvey_sync_api_bundle_config_retrieve('token', $sid);
    //$options = $config['options'];
    // @TODDO !
    $set_invitation = FALSE;
    if (empty($account)) {
      global $user;
      $account = $user;
    }
    switch ($field) {
      case 'ls_tid':
      case 'ls_blacklisted':
      case 'ls_mpid':
      case 'ls_token':
      case 'ls_participant_id':
      case 'ls_validfrom':
      case 'ls_validuntil':
        return NULL;
        break;
      case 'ls_emailstatus':
        // @TODO: may need to edit this depending of the email value.
        return 'OK';
        break;
      case 'ls_language':
        // @TODO: may need to edit this depending of the drupal user language.
        module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
        $this_survey = limesurvey_sync_survey_properties($sid);
        return ($this_survey['default_language']) ? $this_survey['default_language'] : NULL;
        break;
      case 'ls_sent':
      case 'ls_remindersent':
      case 'ls_completed':
        return 'N';
        break;
      case 'ls_remindercount':
        return 0;
        break;
      case 'ls_usesleft':
        return ($options['ls_import_dp2ls_datas'] && isset($options['usesleft'])) ? $options['usesleft'] : 1;
        break;
      case 'ls_firstname':
        $field = (in_array($options['firstname'], array('name', 'mail', 'roles'))) ? $options['firstname'] : 'field_' . $options['firstname'];
        return ($options['ls_import_dp2ls_datas'] && $options['firstname'] != 'none' && isset($account->$field)) ? ((!is_array($account->$field)) ? $account->$field : implode(',', $account->$field)) : NULL;
        break;
      case 'ls_lastname':
        $field = (in_array($options['lastname'], array('name', 'mail', 'roles'))) ? $options['lastname'] : 'field_' . $options['lastname'];
        return ($options['ls_import_dp2ls_datas'] && $options['lastname'] != 'none' && isset($account->$field)) ? ((!is_array($account->$field)) ? $account->$field : implode(',', $account->$field)) : NULL;
        break;
      case 'ls_email':
        $field = (in_array($options['email'], array('name', 'mail'))) ? $options['email'] : 'field_' . $options['email'];
        return ($options['ls_import_dp2ls_datas'] && $options['email'] != 'none' && isset($account->$field)) ? $account->$field : NULL;
        break;
    }


    return FALSE;
  }

  /**
   * Return the corresponding Drupal default value of a LS variable field.
   *
   * This fonction is used for creating a new answer or token Drupal entity.
   * Handle variable LimeSurvey fields of the token and answer tables.
   *
   * @param $field
   *   A string. The LimeSurvey field name.
   * @param $sid
   *   A numeric. The LimeSurvey survey ID. Default is NULL.
   *
   * @return
   *   A string or numeric or NULL : the Drupal default value for the provided
   *   LimeSurvey field or FALSE if the provided field is not a variable
   *   LimeSurvey field.
   *
   * @see LimesurveySyncInfos::entity_default_values()
   */
  protected function create_ls_fields_default_values_variables($field, $sid = NULL) {
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
    $this_survey = limesurvey_sync_survey_properties($sid);
    $attributedescriptions = $this_survey['attributedescriptions'];
    $variable_name = "limesurvey_sync_survey_{$sid}_options";
    $options = variable_get($variable_name, array());
    $fields = array();
    foreach ($attributedescriptions as $attribute_field => $datas) {
      $fields[$attribute_field] = (!$options['ls_import_dp2ls_datas'] || $options[$attribute_field] == 'none') ? NULL : $options[$attribute_field];
    }

    return (isset($fields[$field])) ? $fields[$field] : FALSE;
  }

}

/**
 * Helper for the limesurvey_sync_ls_api_fields_convert_values() function.
 *
 * Manage converting datas from LimeSurvey to Drupal for common LimeSurvey fields.
 *
 * @param $field
 *   A string. The LimeSurvey field name.
 * @param $value
 *   A string or numeric or decimal or NULL. The LimeSurvey value to convert.
 * @param $way
 *   A string, 'ls2dp' or 'dp2ls' for synchronization way. Default is 'ls2pd'.
 *
 * @return
 *   A string or numeric or decimal or NULL : the Drupal converted value.
 */
function limesurvey_sync_token_fields_convert_values_known($field, $value, $way = 'ls2dp') {
  switch ($field) {
    case 'ls_tid':
    case 'ls_participant_id':
    case 'ls_firstname':
    case 'ls_lastname':
    case 'ls_email':
    case 'ls_emailstatus':
    case 'ls_token':
    case 'ls_language':
    case 'ls_blacklisted':
    case 'ls_sent':
    case 'ls_remindersent':
    case 'ls_remindercount':
    case 'ls_usesleft':
    case 'ls_mpid':
      // Sanitize.
      if ($way == 'ls2dp') {
        return (($value === NULL) ? NULL : check_plain($value));
      }
      else {
        // The entity_load function return null values as empty strings.
        return (($value === NULL || $value === '') ? NULL : check_plain($value));
      }
      break;
    case 'ls_validfrom':
    case 'ls_validuntil':
    case 'ls_completed':
      if ($way == 'ls2dp') {
        // Need to convert those values from datetime to timestamp.
        if ($value === NULL || ($field == 'ls_completed' && $value == 'N')) {
          return NULL;
        }
        module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
        return limesurvey_sync_api_fixdate_ls2dp(check_plain($value));
        break;
      }
      else {
        // Need to convert those values from timestamp to datetime.
        if ($field == 'ls_completed') {
          if ($value === NULL) {
            return 'N';
          }
          else {
            // The completed date does not save seconds.
            return limesurvey_sync_api_fixdate_dp2ls(check_plain($value), 'Y-m-d H:i');
          }
        }
        elseif ($value === NULL) {
          return NULL;
        }
        module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
        return limesurvey_sync_api_fixdate_dp2ls(check_plain($value));
        break;
      }

    default:
      return FALSE;
  }

}

/**
 * Helper for the limesurvey_sync_ls_api_fields_convert_values() function.
 *
 * Manage converting datas from LimeSurvey to Drupal for variable LimeSurvey
 * fields.
 *
 * @param $sid
 *   A numeric. The LimeSurvey survey ID.
 * @param $field
 *   A string. The LimeSurvey field name.
 * @param $value
 *   A string or numeric or decimal or NULL. The LimeSurvey value to convert.
 *
 * @return
 *   A string or numeric or decimal or NULL : the Drupal converted value.
 */
function limesurvey_sync_token_fields_convert_values_variables($sid, $field, $value, $way) {
  module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_survey');
  $this_survey = limesurvey_sync_survey_properties($sid);
  $attributedescriptions = $this_survey['attributedescriptions'];
  foreach ($attributedescriptions as $attribute_field => $datas) {
    if ($attribute_field == $field) {
      return (($value === NULL) ? NULL : check_plain($value));
    }
  }

  return FALSE;
}

/**
 * Builder for the token form page.
 *
 * Not available yet. Will be usefull for 'dp2ls' synchronizations.
 *
 * @return
 *   An array, the form elements.
 */
function limesurvey_sync_token_form($form, &$form_state, $entity, $op = 'edit', $entity_type) {

  if ($op == 'clone') {
    $entity->title .= ' (cloned)';
    $entity->type = '';
  }

  $form['#entity'] = $entity;

  $form['token'] = array(
    '#markup' => 'Coding in progress.',
  );
  $form['ls_sid'] = array(
    '#type' => 'hidden',
    '#value' => $entity->ls_sid,
  );

  $form['actions'] = array(
    '#type' => 'actions',
  );
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => !empty($entity->type) ? t('Update LimeSurvey token') : t('Save LimeSurvey token'),
    '#weight' => 40,
  );

  if (!$entity->isLocked() && $op != 'add' && $op != 'clone') {
    $form['actions']['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete token'),
      '#weight' => 45,
      '#limit_validation_errors' => array(),
      '#submit' => array(
        'limesurvey_sync_token_form_submit_delete',
      ),
    );
  }
  field_attach_form($entity_type, $entity, $form, $form_state);
  return $form;
}

/**
 * Builder for the token form page validation.
 *
 * Not available yet. Will be usefull for 'dp2ls' synchronizations.
 *
 * @return
 *   An array, the form elements.
 */
function limesurvey_sync_token_form_validate($form, &$form_state) {
  // Debug : don't save.
  form_set_error('', t('Token is not available for saving, coding in progress...'));
}


/**
 * List of the token lss_status possible values.
 *
 * Used for compatibility with Views and Rules modules, throught
 * hook_entity_property_info().
 *
 * @return
 *   An array, the keys are the lss_status value and values are the human
 *   corresponding display message.
 */
function limesurvey_sync_api_token_lss_status_list() {
  $status = array(
    LIMESURVEY_SYNC_TOKEN_COMPLETED_CLOSED,
    LIMESURVEY_SYNC_TOKEN_COMPLETED_REUSABLE,
    LIMESURVEY_SYNC_TOKEN_OPEN,
    LIMESURVEY_SYNC_TOKEN_SOON,
    LIMESURVEY_SYNC_TOKEN_EXPIRED,
    LIMESURVEY_SYNC_TOKEN_CLOSED,
    LIMESURVEY_SYNC_TOKEN_UNKNOWN,
  );
  $list = array();
  module_load_include('inc', 'limesurvey_sync', 'api/limesurvey_sync_api.helpers');
  foreach ($status as $value) {
    $list[$value] = limesurvey_sync_api_display_field('token', 'lss_status', $value);
  }
  return $list;
}


/**
 * Return a drupal LS field for display.
 *
 * As an example, return timestamp as formated date, LS language code as human
 * language, etc...
 *
 * @param $field
 *   A string. The Drupal ls_field to display (ex : 'startlanguage').
 * @param $value
 *   A string or any. corresponding Drupal LS field to display (ex:
 *   'de-informal').
 * @param $return
 *   A string, 'value' or 'help' or 'css'. Default to 'value'.
 * @param $ls_sid
 *   A numeric, the LS survey ID.
 *
 * @return
 *   A string. The value ready to be displayed or an help text or a css class.
 */
function limesurvey_sync_token_display_field($field, $value, $return = 'value', $ls_sid = FALSE) {

  if (in_array($field, array(
    'ls_completed',
    'ls_language',
    'ls_sent',
    'ls_remindersent',
    'ls_blacklisted',
    'ls_validfrom',
    'ls_validuntil',
  ))) {
    if (in_array($return, array(
      'help',
      'css',
    ))) {
      return '';
      // No need to explicit css or help.
    }
    else {
      switch ($field) {
        case 'ls_completed':
          if ($value == '') {
            return t('Not completed');
          }
          return format_date($value);
          break;
        case 'ls_language':
          return limesurvey_sync_api_display_field('token', 'ls_lang', $value, $return, $ls_sid);
          break;
        case 'ls_sent':
        case 'ls_remindersent':
          if ($value == 'N') {
            return t('No');
          }
          // Is this a date ?
          // @TODO : Need to convert to timestamp this field if it is a date.
          return (!is_numeric($value)) ? $value : format_date($value);
        case 'ls_blacklisted':
          if ($value == '') {
            return t('Not blaklisted');
          }
        case 'ls_validfrom':
        case 'ls_validuntil':
          if ($value == '') {
            return t('any time');
          }
          return format_date($value);
          break;
      }
    }
  }

  switch ($field) {
    case 'lss_status':
      switch ($value) {
        case LIMESURVEY_SYNC_TOKEN_COMPLETED_CLOSED:
          switch ($return) {
            case 'help':
              t('The token invitation is completed and closed.');
            case 'css':
              return 'limesurvey-sync-field-lss-status-token-open';
            default:
              return t('Completed');
          }

        case LIMESURVEY_SYNC_TOKEN_COMPLETED_REUSABLE:
          switch ($return) {
            case 'help':
              return t('The token invitation is completed but reusable.');
            case 'css':
              return 'limesurvey-sync-field-lss-status-token-completed-reusable';
            default:
              return t('Completed reusable');
          }

        case LIMESURVEY_SYNC_TOKEN_OPEN:
          switch ($return) {
            case 'help':
              return t("The token invitation is open.");
            case 'css':
              return 'limesurvey-sync-field-lss-status-token-open';
            default:
              return t('Open');
          }

        case LIMESURVEY_SYNC_TOKEN_SOON:
          switch ($return) {
            case 'help':
              return t('The token invitation will be open in the future.');
            case 'css':
              return 'limesurvey-sync-field-lss-status-token-soon';
            default:
              return t('Not open yet');
          }

        case LIMESURVEY_SYNC_TOKEN_EXPIRED:
          switch ($return) {
            case 'help':
              return t('The token invitation has expired.');
            case 'css':
              return 'limesurvey-sync-field-lss-status-token-expired';
            default:
              return t('Expired');
          }

        case LIMESURVEY_SYNC_TOKEN_CLOSED:
          switch ($return) {
            case 'help':
              return t('The token invitation is uncompleted and closed.');
            case 'css':
              return 'limesurvey-sync-field-lss-status-token-closed';
            default:
              return t('Closed');
          }

        default:
          switch ($return) {
            case 'help':
              return t('Unknown token status.');
            case 'css':
              return 'limesurvey-sync-field-lss-status-token-unknown';
            default:
              return t('Unknown');
          }
      }
  }

  return $value;
}

/**
 * Alter a token entry for editing it from Drupal.
 *
 * @param $entity
 *   An object, the token entity.
 *
 * @return
 *   An array, the LS token field to change in order to activate the token.
 */
function limesurvey_sync_ls_api_token_activate_token($entity) {
  $tochange = array();
  $variable_name = "limesurvey_sync_survey_{$entity->ls_sid}_options";
  $options = variable_get($variable_name, array());
  $tochange['ls_validfrom'] = $options['use_secure_token'] ? (REQUEST_TIME - 10) : NULL;
  $duration = $options['use_secure_token'] ? (($options['secure_validuntil_period'] != 'custom') ? $options['secure_validuntil_period'] : $options['secure_validuntil_period_custom']) : 0;
  $tochange['ls_validuntil'] = $options['use_secure_token'] ? (REQUEST_TIME + $duration) : NULL;

  if (empty($entity->ls_token)) {
    module_load_include('inc', 'limesurvey_sync', 'ls_api/limesurvey_sync_ls_api_token');
    $tochange['ls_token'] = limesurvey_sync_ls_api_token_create_unique_token($entity->ls_sid);
  }
  // The ls_language may have change.
  $tochange['ls_language'] = $entity->ls_language;

  if ($entity->ls_usesleft < 1) {
    $tochange['ls_usesleft'] = 1;
  }
  if ($entity->ls_completed != NULL) {
    $tochange['ls_completed'] = NULL;
  }

  return $tochange;
}

